@@ -153,7 +153,7 @@ impl_Galeon_Browser_addTempBookmark (impl_POA_Galeon_Browser * servant,
 {
 	CORBA_boolean retval = TRUE;
 
-	add_temp_bookmark (SITE, name, url, NULL);
+	add_temp_bookmark (BM_SITE, name, url, NULL);
 	
 	return retval;
 }
