@@ -67,19 +67,19 @@ embed_notebook_switch_page_cb (GtkNotebook *notebook,
 	window = embed->parent_window;
 	return_if_not_window (window);
 
-	/* set this as the active embed */
+	/* set this as the active embed and save the content of the
+	 * location bar of the old embed */
 	if (window->active_embed != NULL)
 	{
 		window->active_embed->is_active = FALSE;
+		if (window->active_embed->site_location) 
+			g_free(window->active_embed->site_location);
+		window->active_embed->site_location = 
+			g_strdup(gtk_entry_get_text(GTK_ENTRY(window->toolbar_entry)));
 	}
 	embed->is_active = TRUE;
 	window->active_embed = embed;
 
-	/* set the state of the view source toggle */
-	gtk_check_menu_item_set_active 
-		(GTK_CHECK_MENU_ITEM (window->view_source_mode), 
-		 embed->viewing_source);
-
 	/* set the global title and location -- this is the lazy way! */
 	embed_update_page_title (embed);
 	embed_update_page_location (embed);
@@ -114,3 +114,40 @@ embed_notebook_close_clicked_cb (GtkWidget *b, GaleonEmbed *embed)
 	return_if_not_embed (embed);
 	embed_close (embed);
 }
+
+/**
+ * embed_label_draw_cb: displays the close button or an inactive transparent
+ *			close button pixmap depending on whether or not the
+ *			label is on the active page
+ */
+void
+embed_tab_draw_cb (GtkWidget *widget, GdkRectangle *area, GaleonEmbed *embed)
+{
+	GtkWidget *button, *pixmap;
+
+	return_if_not_embed (embed);
+
+	/* the label container could also be searched for these but this
+	   is simpler and cleaner */
+	button = gtk_object_get_data (GTK_OBJECT (widget), "button");
+	pixmap = gtk_object_get_data (GTK_OBJECT (widget), "pixmap");
+
+	g_return_if_fail (button != NULL && pixmap != NULL);
+
+	if (embed == embed->parent_window->active_embed)
+	{		
+		if (!GTK_WIDGET_VISIBLE (button))
+		{
+			gtk_widget_show (button);
+			gtk_widget_hide (pixmap);
+		}
+	}
+	else
+	{
+		if (!GTK_WIDGET_VISIBLE (pixmap))
+		{
+			gtk_widget_show (pixmap);
+			gtk_widget_hide (button);
+		}
+	}
+}		
