@@ -37,8 +37,8 @@ void find_next_button_clicked_cb (GtkButton *button, GaleonEmbed *embed)
 	gtk_widget_set_sensitive (embed->find_dialog->prev_button, TRUE);
 	if (!find_go (embed, FALSE))
 	{
-		gtk_widget_set_sensitive(GTK_WIDGET(embed->find_dialog->next_button), 
-					 FALSE);
+		gtk_widget_set_sensitive
+			(GTK_WIDGET(embed->find_dialog->next_button), FALSE);
 	}
 }
 
@@ -49,11 +49,11 @@ void
 find_prev_button_clicked_cb (GtkButton *button, GaleonEmbed *embed)
 {
 	return_if_not_embed (embed);
-	gtk_widget_set_sensitive (embed->find_dialog->prev_button, TRUE);
+	gtk_widget_set_sensitive (embed->find_dialog->next_button, TRUE);
 	if (!find_go (embed, TRUE))
 	{
-		gtk_widget_set_sensitive(GTK_WIDGET(embed->find_dialog->prev_button), 
-					 FALSE);
+		gtk_widget_set_sensitive
+			(GTK_WIDGET(embed->find_dialog->prev_button),  FALSE);
 	}
 }
 
@@ -123,19 +123,48 @@ find_reset (GaleonEmbed *embed)
 
 gboolean find_go (GaleonEmbed *embed, gboolean backwards)
 {
-	gchar *exp;
-	gboolean match;
-	
-	exp = gtk_editable_get_chars(GTK_EDITABLE (embed->find_dialog->entry), 0, -1);
-	match = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(embed->find_dialog->case_checkbutton));
+	gchar *exp = gtk_editable_get_chars(GTK_EDITABLE (embed->find_dialog->entry), 0, -1);
+	WrapperSearchProperties *properties;
+	gboolean result;
+
+	properties = g_new0 (WrapperSearchProperties,1);
+	properties->search_string = exp;
+	properties->backwards =  backwards;
+	properties->match_case = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(embed->find_dialog->case_checkbutton));
 
 	if (exp[0] == '\0') return FALSE;
 
 	if (embed->find_dialog->string_changed)
 	{
-		gnome_entry_append_history(GNOME_ENTRY(embed->find_dialog->gnome_entry),FALSE,exp);
+		gnome_entry_append_history(GNOME_ENTRY(embed->find_dialog->gnome_entry),
+								   TRUE, exp);
 		embed->find_dialog->string_changed = FALSE;
 	}
 
-	return mozilla_find(embed, exp, match, backwards);
+	result = mozilla_find(embed, properties);
+
+	g_free (properties);
+
+	return result;
+}
+
+/**
+ * find_next: find the next occurance of a string, used by smart bookmarks
+ * phrase searching
+ */
+gboolean
+find_next (GaleonEmbed *embed, const gchar *string)
+{
+	WrapperSearchProperties *properties;
+	gboolean result;
+
+	properties = g_new0 (WrapperSearchProperties, 1);
+	properties->search_string = string;
+	properties->backwards = FALSE;
+	properties->match_case = FALSE;
+
+	result = mozilla_find (embed, properties);
+	g_free (properties);
+
+	return result;
 }
