@@ -23,6 +23,7 @@
 #include "nsIGlobalHistory.h"
 #include "nsIDocShellHistory.h"
 #include "nsIDocShellTreeItem.h"
+#include "nsIDocShellTreeNode.h"
 #include "nsIDocShellTreeOwner.h"
 #include "nsIDiskDocument.h"
 #include "nsIWebBrowserFind.h"
@@ -30,14 +31,22 @@
 #include "nsIDocument.h"
 #include "nsFileSpec.h"
 #include "nsISHEntry.h"
+#ifdef CVS_SOURCE
+#include "nsISHistoryInternal.h"
+#include "nsIWebBrowserPrint.h"
+#else
+#include "nsINetDataCacheManager.h"
+#include "nsICachedNetData.h"
+#endif
 #include "nsIURI.h"
 #include "nsIURL.h"
 #include "nsIPresShell.h"
 #include "nsTextServicesCID.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIDOMHTMLElement.h"
+#include "nsIDOMHTMLDocument.h"
+#include "nsIDOMNodeList.h"
 #include "nsIDOMNamedNodeMap.h"
-#include "nsIDOMEventReceiver.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIDOMHTMLAnchorElement.h"
@@ -45,7 +54,12 @@
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
 #include "nsIInterfaceRequestor.h"
-
+#include "nsIChannel.h"
+#include "nsILoadGroup.h"
+#include "nsIWebBrowserPersist.h"
+#include "nsNetUtil.h"
+#include "nsPIDOMWindow.h"
+#include "nsIChromeEventHandler.h"
 #include "galeon-wrapper.h"
 
 static NS_DEFINE_CID(kCTextServicesDocumentCID, NS_TEXTSERVICESDOCUMENT_CID);
@@ -83,32 +97,53 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 			NS_GLOBALHISTORY_CONTRACTID, &result);  
 	if (NS_FAILED(result) || !history) return NS_ERROR_FAILURE;
 	dsHistory->SetGlobalHistory(history);
-	
+
+#if MOZILLA_MILESTONE != 81
   	mChromeNav = do_QueryInterface(mWebBrowser);
 	if (!mChromeNav) return NS_ERROR_FAILURE;
+#else
+	nsCOMPtr<nsIDOMWindow> domWindow;
+	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
+	if (!domWindow) return NS_ERROR_FAILURE;
 
+	nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
+	nsCOMPtr<nsIDOMWindowInternal> rootWindow;
+	domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
+  
+	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
+	nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
+	piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
+  
+	mEventReceiver = do_QueryInterface(chromeHandler);
+
+	mEventListener = new GaleonEventListener();
+	mEventListener->Init(this);
+#endif
 	if (attachListener)
 	return AddListener ();
 }
 
-nsresult GaleonWrapper::Print ()
+#ifdef CVS_SOURCE
+nsresult GaleonWrapper::Print (nsIPrintOptions *options)
 {
-	nsresult result;
+	nsresult rv = NS_OK;
 
 	nsCOMPtr<nsIDocShell> DocShell;
-	result = GetDocShell (getter_AddRefs(DocShell));
-	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
+	rv = GetDocShell (getter_AddRefs(DocShell));
+	if (NS_FAILED(rv) || !DocShell) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIContentViewer> ContentViewer;
-	result = DocShell->GetContentViewer(getter_AddRefs(ContentViewer));
-	if (NS_FAILED(result) || !ContentViewer) return NS_ERROR_FAILURE;
-                
-	nsCOMPtr<nsIContentViewerFile> ContentViewerFile = 
-						do_QueryInterface(ContentViewer);
-	if (!ContentViewerFile) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIContentViewer> contentViewer;
+	DocShell->GetContentViewer(getter_AddRefs(contentViewer));
+	if (contentViewer) {
+		nsCOMPtr<nsIContentViewerFile> contentViewerFile(do_QueryInterface(contentViewer));
+		if (contentViewerFile) {
+			rv = contentViewerFile->Print(PR_TRUE, nsnull, nsnull);
+		}
+	}
 
-	return ContentViewerFile->Print(PR_TRUE, nsnull);
+	return rv;
 }
+#endif
 
 nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 {
@@ -132,6 +167,14 @@ nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 
 nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
 {
+#if MOZILLA_MILESTONE == 81
+	nsCOMPtr<nsIDocShell> DocShell (do_GetInterface(mWebBrowser));
+	if (!DocShell) return NS_ERROR_FAILURE;
+
+	*aDocShell = DocShell.get();
+
+	NS_IF_ADDREF(*aDocShell);
+#else
 	nsCOMPtr<nsIDocShellTreeItem> browserAsItem;
 	browserAsItem = do_QueryInterface(mWebBrowser);
 	if (!browserAsItem) return NS_ERROR_FAILURE;
@@ -154,6 +197,7 @@ nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
 	*aDocShell = DocShell.get();
 
 	NS_IF_ADDREF(*aDocShell);
+#endif
 }
 
 nsresult GaleonWrapper::Destroy ()
@@ -161,13 +205,28 @@ nsresult GaleonWrapper::Destroy ()
  	if (attachListener)
   		return RemoveListener ();
       	mWebBrowser = nsnull;
+#if MOZILLA_MILESTONE == 81
+	mEventReceiver = nsnull;
+#else
 	mChromeNav = nsnull;
+#endif
 }
 
 nsresult GaleonWrapper::AddListener ()
 {
 	nsresult rv = NS_OK;
 
+#if MOZILLA_MILESTONE == 81
+	if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
+
+	nsIDOMEventListener *eventListener =
+		NS_STATIC_CAST(nsIDOMEventListener *,
+			       NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
+
+	rv = mEventReceiver->AddEventListenerByIID(eventListener,
+						 NS_GET_IID(nsIDOMDragListener));
+	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+#else
 	nsCOMPtr <nsIDOMDocument> domDoc;
 	mChromeNav->GetDocument(getter_AddRefs(domDoc));
 	if (!domDoc) return NS_ERROR_FAILURE;
@@ -191,12 +250,24 @@ nsresult GaleonWrapper::AddListener ()
 	rv = eventReceiver->AddEventListenerByIID(mEventListener,
 						 NS_GET_IID(nsIDOMDragListener));
 	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+#endif
 }
 
 nsresult GaleonWrapper::RemoveListener ()
 {
 	nsresult rv = NS_OK;
 
+#if MOZILLA_MILESTONE == 81
+	if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
+	
+	nsIDOMEventListener *eventListener =
+		NS_STATIC_CAST(nsIDOMEventListener *,
+			       NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
+
+	rv = mEventReceiver->RemoveEventListenerByIID(eventListener,
+						      NS_GET_IID(nsIDOMDragListener));
+	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+#else
 	if (!mEventListener) return NS_ERROR_FAILURE;
 
 	nsCOMPtr <nsIDOMDocument> domDoc;
@@ -212,6 +283,7 @@ nsresult GaleonWrapper::RemoveListener ()
 	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 	mEventListener = NULL;
+#endif
 }
 
 nsresult GaleonWrapper::GoToHistoryIndex (PRInt16 index)
@@ -229,6 +301,8 @@ nsresult GaleonWrapper::GoToHistoryIndex (PRInt16 index)
 	return  ContentNav->GotoIndex (index);
 }
 
+#ifndef CVS_SOURCE
+
 nsresult GaleonWrapper::SetViewSourceMode (PRInt32 mode)
 {
 	nsresult result;
@@ -240,6 +314,8 @@ nsresult GaleonWrapper::SetViewSourceMode (PRInt32 mode)
 	return DocShell->SetViewMode (mode);
 }
 
+#endif
+
 nsresult GaleonWrapper::SetZoom (float aZoom)
 {
 	nsresult result;
@@ -282,6 +358,19 @@ nsresult GaleonWrapper::GetDOMDocument (nsIDOMDocument **aDOMDocument)
 {
 	nsresult result;
 
+	nsCOMPtr<nsIWebBrowserFocus> focus(do_GetInterface(mWebBrowser));
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = focus->GetFocusedWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+
+	return DOMWindow->GetDocument (getter_AddRefs(aDOMDocument));
+}
+
+nsresult GaleonWrapper::GetMainDOMDocument (nsIDOMDocument **aDOMDocument)
+{
+	nsresult result;
+
 	nsCOMPtr<nsIDocShell> DocShell;
 	result = GetDocShell (getter_AddRefs (DocShell));
 	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
@@ -289,7 +378,7 @@ nsresult GaleonWrapper::GetDOMDocument (nsIDOMDocument **aDOMDocument)
 	nsCOMPtr<nsIContentViewer> contentViewer;	
 	result = DocShell->GetContentViewer (getter_AddRefs(contentViewer));
 	if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
-	
+
 	return contentViewer->GetDOMDocument (getter_AddRefs(aDOMDocument));
 }
 
@@ -299,37 +388,79 @@ nsresult GaleonWrapper::SaveMainDocument (const char* filename)
 
 	nsCOMPtr<nsIDOMDocument> aDOMDocument;
 
-	GetDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument)
-	{
-		return NS_ERROR_FAILURE;
-	}
+	GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
 
-	return SaveDocument (aDOMDocument, filename);
+	return SaveDOMDocument (aDOMDocument, filename);
 }
 
-nsresult GaleonWrapper::SaveDocument (nsIDOMDocument *aDOMDocument,
-				      const char* filename)
+nsresult GaleonWrapper::SaveURI (const char *url, const char *filename)
+{
+        nsresult result;
+
+	nsCOMPtr<nsIWebBrowserPersist> persist(do_GetInterface(mWebBrowser));
+	nsString s;
+	s.AssignWithConversion(url);
+	nsCOMPtr<nsIURI> linkURI;
+	result = NS_NewURI(getter_AddRefs(linkURI), s);
+	return persist->SaveURI(linkURI, nsnull, filename);
+}
+
+nsresult GaleonWrapper::SaveDocument (const char* filename)
 {
 	nsresult result;
 
-	nsString mime_type;
-	nsString charset;
-	mime_type.AssignWithConversion("text/html");
-	charset.AssignWithConversion("");
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
 
-	nsCOMPtr<nsILocalFile> file;
-	file = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-	result = file->InitWithPath(filename);
+	GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	return SaveDOMDocument (aDOMDocument, filename);
+}
+
+nsresult GaleonWrapper::SaveDOMDocument (nsIDOMDocument *aDOMDocument, const char* filename)
+{
+	nsresult result;
+	nsCOMPtr<nsIWebBrowserPersist> persist(do_GetInterface(mWebBrowser));
+	return persist->SaveDocument (aDOMDocument, filename, nsnull);
+}
+
+#if MOZILLA_MILESTONE != 81
+
+nsresult GaleonWrapper::SaveURLFromCache (const char *url, 
+		const char *dir, const char *filename)
+{
+	nsresult result;
+
+	nsCOMPtr<nsINetDataCacheManager> ndcm = 
+		do_GetService(NS_NETWORK_CACHE_MANAGER_CONTRACTID, &result);
+
+	nsCOMPtr<nsICachedNetData> cnd;
+	result = ndcm->GetCachedNetData (url, nsnull, 0, 0, getter_AddRefs(cnd));
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDiskDocument> diskDoc = do_QueryInterface(aDOMDocument);
-	result = diskDoc->SaveFile(file, PR_TRUE, PR_TRUE,
-				   mime_type.ToNewUnicode(),
-				   charset.ToNewUnicode(), 0, 80);
-	if (!NS_SUCCEEDED(result)) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIChannel> chan;
+	/* FIXME: dunno if this is needed */
+	nsCOMPtr<nsILoadGroup> buhLoadGroup = null_nsCOMPtr();
+	result = cnd->NewChannel(buhLoadGroup, getter_AddRefs(chan));
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIFile> cfile;
+	result = chan->GetLocalFile(getter_AddRefs(cfile));
+	if (NS_FAILED(result) || !cfile) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsILocalFile> localdir = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
+	result = localdir->InitWithPath(dir);
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+
+	result = cfile->CopyTo(localdir, filename);
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+
+	return NS_OK;
 }
 
+#endif
+
 nsresult GaleonWrapper::GetSHInfo (PRInt32 *count, PRInt32 *index)
 {
 	nsresult result;
@@ -350,7 +481,11 @@ nsresult GaleonWrapper::GetSHTitleAtIndex (PRInt32 index, PRUnichar **title)
 	result = GetSHistory (getter_AddRefs(SessionHistory));
 	if (NS_FAILED(result) || ! SessionHistory) return NS_ERROR_FAILURE;
 
+#ifdef CVS_SOURCE
+	nsCOMPtr<nsIHistoryEntry> he;
+#else
 	nsCOMPtr<nsISHEntry> he;
+#endif
 	result = SessionHistory->GetEntryAtIndex (index, PR_FALSE,
 						  getter_AddRefs (he));
 	if (!NS_SUCCEEDED(result) || (!he)) return NS_ERROR_FAILURE;
@@ -372,9 +507,15 @@ nsresult GaleonWrapper::Find (const char *exp, PRBool matchcase,
 	return finder->FindNext(didFind);
 }
 
-nsresult GaleonWrapper::ReloadDocument (nsIDOMDocument *aDOMDocument)
+nsresult GaleonWrapper::ReloadDocument ()
 {
 	nsresult result;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+	GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
@@ -401,8 +542,15 @@ nsresult GaleonWrapper::ReloadDocument (nsIDOMDocument *aDOMDocument)
 	if (!NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
 }
 
-nsresult GaleonWrapper::GetDocumentUrl (nsIDOMDocument *aDOMDocument, char **url)
+nsresult GaleonWrapper::GetDocumentUrl (char **url)
 {
+	nsresult result;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+	GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
@@ -410,8 +558,7 @@ nsresult GaleonWrapper::GetDocumentUrl (nsIDOMDocument *aDOMDocument, char **url
 	uri->GetSpec (url);
 }
 
-nsresult GaleonWrapper::GetKey (nsIDOMKeyEvent *keyEvent, PRUint32 *keycode,
-				PRInt16 *modifier)
+nsresult GaleonWrapper::GetKeyEventContext (nsIDOMKeyEvent *keyEvent, WrapperKeyEventInfo *info)
 {
 	PRBool mod_key;
 	nsresult res;
@@ -419,6 +566,7 @@ nsresult GaleonWrapper::GetKey (nsIDOMKeyEvent *keyEvent, PRUint32 *keycode,
 	/* if we are in a text area do not process keys */
 	nsCOMPtr<nsIDOMEventTarget>  targetNode;
 	res = keyEvent->GetTarget(getter_AddRefs(targetNode));
+	if (NS_FAILED(res) || !targetNode) return NS_ERROR_FAILURE;
 	if (targetNode)
 	{
 		nsCOMPtr<nsIDOMNode> node = do_QueryInterface(targetNode);
@@ -432,55 +580,44 @@ nsresult GaleonWrapper::GetKey (nsIDOMKeyEvent *keyEvent, PRUint32 *keycode,
 			{
 				nsAutoString tag;
 				element->GetTagName(tag);
-				if (!tag.EqualsWithConversion("html", PR_TRUE))
+
+				if (!tag.EqualsWithConversion("a", PR_TRUE) &&
+				    !tag.EqualsWithConversion ("html",PR_TRUE))
+				{
 					return NS_ERROR_FAILURE;
+				}
 			}
 		}
 	}
+	
+	/* Get the context */
 
-	*modifier = 0;
+	res = GetEventContext (targetNode, &info->ctx);
+	if (NS_FAILED(res)) return res;
+	
+	/* Get the key/modifier */
 
-	keyEvent->GetCharCode(keycode);
+	info->modifier = 0;
 
-	if (*keycode==0) 
+	keyEvent->GetCharCode((PRUint32*)&info->key);
+
+	if (info->key==0) 
 	{
-		keyEvent->GetKeyCode(keycode);
-		*modifier |= KEY_CODE;
+		keyEvent->GetKeyCode((PRUint32*)&info->key);
+		info->modifier |= KEY_CODE;
 	}
 
 	keyEvent->GetAltKey(&mod_key);
-	if (mod_key) *modifier |= ALT_KEY;
+	if (mod_key) info->modifier |= ALT_KEY;
 
 	keyEvent->GetShiftKey(&mod_key);
-	if (mod_key) *modifier |= SHIFT_KEY;
+	if (mod_key) info->modifier |= SHIFT_KEY;
 
 	keyEvent->GetMetaKey(&mod_key);
-	if (mod_key) *modifier |= META_KEY;
+	if (mod_key) info->modifier |= META_KEY;
 	
 	keyEvent->GetCtrlKey(&mod_key);
-	if (mod_key) *modifier |= CTRL_KEY;
-}
-
-nsresult GaleonWrapper::GetModKey (nsIDOMMouseEvent *mouseEvent,
-				   PRInt16 *modifier)
-{
-	PRBool mod_key;
-
-	*modifier = 0;
-
-	mouseEvent->GetAltKey(&mod_key);
-	if (mod_key) *modifier |= ALT_KEY;
-
-	mouseEvent->GetShiftKey(&mod_key);
-	if (mod_key) *modifier |= SHIFT_KEY;
-
-	mouseEvent->GetMetaKey(&mod_key);
-	if (mod_key) *modifier |= META_KEY;
-	
-	mouseEvent->GetCtrlKey(&mod_key);
-	if (mod_key) *modifier |= CTRL_KEY;
-
-	return NS_OK;
+	if (mod_key) info->modifier |= CTRL_KEY;
 }
 
 nsresult GaleonWrapper::GetDOMAttribute (nsIDOMNode *node, char *tag,
@@ -511,18 +648,11 @@ nsresult GaleonWrapper::GetDOMAttribute (nsIDOMNode *node, char *tag,
 	nsMemory::Free (cstr);
 }
 
-nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
-					      int *context, char **img, 
-					      nsIDOMDocument **document,
-					      char **link)
+nsresult GaleonWrapper::GetEventContext (nsCOMPtr<nsIDOMEventTarget> EventTarget, WrapperContextInfo *info)
 {
 	nsresult result;
-	*img = *link = NULL;
-	nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent*)event;
-
-	nsCOMPtr<nsIDOMEventTarget> EventTarget;
-	result = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
-	if (NS_FAILED(result) || !EventTarget) return NS_ERROR_FAILURE;
+	
+	info->img = info->link = NULL;
 
 	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(EventTarget);
 	if (!node) return NS_ERROR_FAILURE;
@@ -531,8 +661,6 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
 	result = node->GetOwnerDocument(getter_AddRefs(domDoc));
 	if (!NS_SUCCEEDED (result) || !domDoc) return NS_ERROR_FAILURE;
 
-	*document = domDoc;
-
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
 	if(!doc) return NS_ERROR_FAILURE;
 
@@ -542,12 +670,13 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
    
 	nsString mime;  
 	doc->GetContentType(mime);  
-	if (strcmp (mime.ToNewCString(),"text/xul") == 0)
+	if (mime.EqualsWithConversion ("text/xul"))
 		return NS_ERROR_FAILURE;
 
-	*context = CONTEXT_NONE;
+	info->context = CONTEXT_NONE;
 	nsCOMPtr<nsIDOMHTMLElement> element;
-	do {
+	do 
+	{
 		PRUint16 type;
 		node->GetNodeType(&type);
 
@@ -559,11 +688,11 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
 
 			if (tag.EqualsWithConversion("input", PR_TRUE))
 			{
-				*context |= CONTEXT_INPUT;
+				info->context |= CONTEXT_INPUT;
 			}
 			else if (tag.EqualsWithConversion("img", PR_TRUE))
 			{
-				*context |= CONTEXT_IMAGE;
+				info->context |= CONTEXT_IMAGE;
 
 				char *src;
 				result = GetDOMAttribute (node, g_strdup("src"),
@@ -571,14 +700,14 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
 				if (NS_FAILED(result) || !src)
 					return NS_ERROR_FAILURE;
 			
-				result = baseURI->Resolve(src, img);
-				if (NS_FAILED(result) || !img)
+				result = baseURI->Resolve(src, &info->img);
+				if (NS_FAILED(result) || !info->img)
 				return NS_ERROR_FAILURE;
 
 			}
 			else
 			{
-				*context |= CONTEXT_OTHER;
+				info->context |= CONTEXT_OTHER;
 			}
 
 			nsCOMPtr<nsIDOMNamedNodeMap> attributes;
@@ -592,7 +721,7 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
 						       getter_AddRefs(hrefNode));
 				if (hrefNode)
 				{
-					*context |= CONTEXT_LINK;
+					info->context |= CONTEXT_LINK;
 
 					char *href;
 					result = GetDOMAttribute (node,
@@ -600,8 +729,8 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
 					if (NS_FAILED(result) || !href)
 						return NS_ERROR_FAILURE;
 				
-					result = baseURI->Resolve(href, link);
-					if (NS_FAILED(result) || !link)
+					result = baseURI->Resolve(href, &info->link);
+					if (NS_FAILED(result) || !info->link)
 						return NS_ERROR_FAILURE;
 					break;
 				}
@@ -614,33 +743,70 @@ nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event,
 			if (!parentNode)
 			{
 				node = nsnull;
-				*context |= CONTEXT_DOCUMENT;
+				info->context |= CONTEXT_DOCUMENT;
 				break;
 			}
 			node = parentNode;
-	} while (node);
+	} 
+	while (node);
 
+	/* Is page framed ? */
+	PRBool framed;
+	IsPageFramed (&framed);
+	info->framed_page = (gboolean)framed;
+	
+	return NS_OK;
 }
 
-nsresult GaleonWrapper::ScrollbarClicked (nsIDOMMouseEvent *aMouseEvent)
+nsresult GaleonWrapper::GetMouseEventContext (nsIDOMMouseEvent *event, WrapperMouseEventInfo *info)
 {
 	nsresult result;
+	nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent*)event;
+
+	aMouseEvent->GetButton ((PRUint16*)&info->button);
+
+	/* be sure we are not clicking on the scroolbars */
 
 	nsCOMPtr<nsIDOMEventTarget> OriginalTarget;
 	result = aMouseEvent->GetOriginalTarget(getter_AddRefs(OriginalTarget));
 	if (NS_FAILED(result) || !OriginalTarget) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(OriginalTarget);
-	if (!node) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMNode> OriginalNode = do_QueryInterface(OriginalTarget);
+	if (!OriginalNode) return NS_ERROR_FAILURE;
 
 	nsString nodename;
-	node->GetNodeName(nodename);
+	OriginalNode->GetNodeName(nodename);
 
-	if (!strcmp(nodename.ToNewCString(), "xul:thumb") ||
-	    !strcmp(nodename.ToNewCString(), "xul:slider"))
-		return NS_OK;
-	else
+	if (nodename.EqualsWithConversion ("xul:thumb") ||
+	    nodename.EqualsWithConversion ("xul:slider"))
 		return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMEventTarget> EventTarget;
+	result = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
+	if (NS_FAILED(result) || !EventTarget) return NS_ERROR_FAILURE;
+
+	result = GetEventContext (EventTarget, &info->ctx);
+	if (NS_FAILED(result)) return result;
+
+	/* Get the modifier */
+
+	PRBool mod_key;
+
+	info->modifier = 0;
+
+	aMouseEvent->GetAltKey(&mod_key);
+	if (mod_key) info->modifier |= ALT_KEY;
+
+	aMouseEvent->GetShiftKey(&mod_key);
+	if (mod_key) info->modifier |= SHIFT_KEY;
+
+	aMouseEvent->GetMetaKey(&mod_key);
+	if (mod_key) info->modifier |= META_KEY;
+	
+	aMouseEvent->GetCtrlKey(&mod_key);
+	if (mod_key) info->modifier |= CTRL_KEY;
+
+	return NS_OK;
 }
 
 nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
@@ -654,11 +820,11 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 
 	nsCOMPtr<nsIWebNavigation> wn_src = do_QueryInterface (DocShell,
 							       &result);
-	if (!wn_src) return FALSE;
+	if (!wn_src) return NS_ERROR_FAILURE;
 	
 	nsCOMPtr<nsISHistory> h_src;
 	result = wn_src->GetSessionHistory (getter_AddRefs (h_src));
-	if (!NS_SUCCEEDED (result) || (!h_src)) return FALSE;
+	if (!NS_SUCCEEDED(result) || (!h_src)) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDocShell> destDocShell;
 	result = dest->GetDocShell (getter_AddRefs(destDocShell));
@@ -666,17 +832,26 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 
 	nsCOMPtr<nsIWebNavigation> wn_dest = do_QueryInterface (destDocShell,
 								&result);
-	if (!wn_dest) return FALSE;
+	if (!wn_dest) return NS_ERROR_FAILURE;
 	
 	nsCOMPtr<nsISHistory> h_dest;
 	result = wn_dest->GetSessionHistory (getter_AddRefs (h_dest));
-	if (!NS_SUCCEEDED (result) || (!h_dest)) return FALSE;
+	if (!NS_SUCCEEDED (result) || (!h_dest)) return NS_ERROR_FAILURE;
+#ifdef CVS_SOURCE
+	nsCOMPtr<nsISHistoryInternal> hi_dest = do_QueryInterface (h_dest);
+	if (!hi_dest) return NS_ERROR_FAILURE;
+#endif
 
 	h_src->GetCount (&count);
 	h_src->GetIndex (&index);
 
 	if (count) {
+#ifdef CVS_SOURCE
+		nsCOMPtr<nsIHistoryEntry> he;
+		nsCOMPtr<nsISHEntry> she;
+#else		
 		nsCOMPtr<nsISHEntry> he;
+#endif
 
 		for (PRInt32 i = 0; i < count; i++) {
 
@@ -684,9 +859,16 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 							 getter_AddRefs (he));
 			if (!NS_SUCCEEDED(result) || (!he))
 				return NS_ERROR_FAILURE;
+#ifdef CVS_SOURCE
+			she = do_QueryInterface (he);
+			if (!she) return NS_ERROR_FAILURE;
 
-			result = h_dest->AddEntry(he, PR_TRUE);
+			result = hi_dest->AddEntry (she, PR_TRUE);
+			if (!NS_SUCCEEDED(result) || (!she))
+#else
+			result = h_dest->AddEntry (he, PR_TRUE);
 			if (!NS_SUCCEEDED(result) || (!he))
+#endif
 				return NS_ERROR_FAILURE;
 		}
 
@@ -699,21 +881,40 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 nsresult GaleonWrapper::SetSitePermission (gboolean block, PRInt32 type)
 {
 	nsresult result;
-	nsCOMPtr<nsIWebBrowserFocus> focus(do_GetInterface(mWebBrowser));
+
+#ifdef CVS_SOURCE
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
+	result = GetDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface (DOMDocument);
+	if (!doc) return NS_ERROR_FAILURE;
+	
+	nsIURI *uri = doc->GetDocumentURL ();
+	char *url;
+	uri->GetSpec (&url);
+	
+	nsCOMPtr<nsIPermissionManager> 
+		permissionManager = do_CreateInstance (PERMISSIONMANAGER_ID);
+	
+	result = permissionManager->Add (url,block ? PR_TRUE : PR_FALSE, type);
+#else
+	nsCOMPtr<nsIWebBrowserFocus> focus (do_GetInterface(mWebBrowser));
 
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
 	result = focus->GetFocusedWindow (getter_AddRefs(DOMWindow));
 	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDOMWindowInternal> DOMWindowInternal = do_QueryInterface (DOMWindow);
+	nsCOMPtr<nsIDOMWindowInternal> 
+			DOMWindowInternal = do_QueryInterface (DOMWindow);
 	if (!DOMWindowInternal) return NS_ERROR_FAILURE;	
 
-	nsCOMPtr<nsICookieViewer> cookieviewer = do_CreateInstance
-							      (COOKIEVIEWER_ID);
-	result = cookieviewer->AddPermission(DOMWindowInternal,
-					     block ? PR_TRUE : PR_FALSE, 
-					     type);
-	g_print ("aaaaa");
+	nsCOMPtr<nsICookieViewer> 
+			cookieviewer = do_CreateInstance (COOKIEVIEWER_ID);
+	result = cookieviewer->AddPermission (DOMWindowInternal,
+					      block ? PR_TRUE : PR_FALSE, 
+					      type);
+#endif
 	return result;
 }
 
@@ -740,58 +941,30 @@ nsresult GaleonWrapper::ForceCharacterSet (char *charset)
 	return result;
 }
 
-nsresult GaleonWrapper::GetCharacterSets (GHashTable **charsets,
-         GList **sorted_charset_titles)
+nsresult GaleonWrapper::IsPageFramed (PRBool *Framed)
 {
-   nsresult rv;
-   PRUint32 cscount;
+	nsCOMPtr<nsIDocShellTreeItem> browserAsItem;
+	browserAsItem = do_QueryInterface(mWebBrowser);
+	if (!browserAsItem) return NS_ERROR_FAILURE;
 
-   nsCOMPtr<nsIAtom> docCharsetAtom;
-   nsCOMPtr<nsICharsetConverterManager2> ccm2 = do_GetService (
-				   NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-   if (!NS_SUCCEEDED(rv)) return rv;
+	// get the owner for that item
+	nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
+	browserAsItem->GetTreeOwner(getter_AddRefs(treeOwner));
+	if (!treeOwner) return NS_ERROR_FAILURE;
 
-   nsCOMPtr <nsISupportsArray> cs_list;
-   rv = ccm2->GetDecoderList(getter_AddRefs(cs_list));
-   if (!NS_SUCCEEDED(rv)) return rv;
+	// get the primary content shell as an item
+	nsCOMPtr<nsIDocShellTreeItem> contentItem;
+	treeOwner->GetPrimaryContentShell(getter_AddRefs(contentItem));
+	if (!contentItem) return NS_ERROR_FAILURE;
 
-   rv = cs_list->Count(&cscount);
-   *charsets = g_hash_table_new(g_str_hash,g_str_equal);
+	nsCOMPtr<nsIDocShellTreeNode> contentNode;	
+	contentNode = do_QueryInterface (contentItem);
+	if (!contentNode) return NS_ERROR_FAILURE;
 
-   for(PRUint32 i = 0; i < cscount; i++) {
-      nsCOMPtr<nsISupports> cssupports = (dont_AddRef)(cs_list->ElementAt(i));
-      nsCOMPtr<nsIAtom> csatom ( do_QueryInterface(cssupports) );
+	PRInt32 childCount;
+	contentNode->GetChildCount (&childCount);
 
-		nsString charset_ns = NULL, charset_title_ns = NULL;
-
-		/* charset name */
-		rv = csatom->ToString(charset_ns);
-		char *charset_str = charset_ns.ToNewCString();
-
-		/* charset readable title */
-		rv = ccm2->GetCharsetTitle2(csatom, &charset_title_ns);
-		char *charset_title_str = charset_title_ns.ToNewCString();
-
-		for (PRUint32 j = 0; j < NUM_TRANSLATED_CHARSETS; j++) 
-		{
-			if (g_strcasecmp (charset_title_str, charset_trans_array[j]) == 0)
-			{
-				charset_title_str = 	_(charset_trans_array[j]);
-			}
-		}
-
-		if (strlen(charset_title_str) == 0)
-			charset_title_str = g_strdup(charset_str);
-
-		/* fill the hash and the sorted list */
-		g_hash_table_insert (*charsets, charset_title_str, charset_str);
-		*sorted_charset_titles = g_list_insert_sorted (
-						(GList*)*sorted_charset_titles,
-	                    			(gpointer)charset_title_str,
-						(GCompareFunc)g_strcasecmp); 
-	}
-
-   return rv;
+	*Framed = (childCount>0);
 }
 
 GaleonEventListener::GaleonEventListener(void)
@@ -839,18 +1012,27 @@ NS_IMETHODIMP GaleonEventListener::DragDrop(nsIDOMEvent* aMouseEvent)
 }
 NS_IMETHODIMP GaleonEventListener::DragGesture(nsIDOMEvent* aMouseEvent)
 {
-	gchar *link;
-	gpointer ds;
-	gchar *img;
-	int type;
 	gchar *old_dragging_link;
+	WrapperMouseEventInfo *info;
+	gchar *url = NULL;
 
-	type = mozilla_get_event_context(wrapper->embed, aMouseEvent, 
-					 &img, &ds, &link);
+	info = g_new0 (WrapperMouseEventInfo,1);
 
-	if (type & CONTEXT_LINK)
+	mozilla_get_mouse_event_info(wrapper->embed, aMouseEvent, info);
+
+
+	if (info->ctx.context & CONTEXT_LINK)
 	{
-		if ( ! link_drag_types_tl)
+		url = info->ctx.link;
+	}
+	else if (info->ctx.context & CONTEXT_IMAGE)
+	{	
+		url = info->ctx.img;
+	}
+	
+	if (url)
+	{
+		if (!link_drag_types_tl)
 			link_drag_types_tl = gtk_target_list_new 
 				(link_drag_types, link_drag_types_num_items);
 
@@ -859,7 +1041,7 @@ NS_IMETHODIMP GaleonEventListener::DragGesture(nsIDOMEvent* aMouseEvent)
 					"dragging_link");
 		if (old_dragging_link) g_free (old_dragging_link);
 		gtk_object_set_data (GTK_OBJECT(wrapper->mGtkMozEmbed),
-				     "dragging_link", g_strdup (link));
+				     "dragging_link", g_strdup (url));
 		gtk_drag_begin (GTK_WIDGET(wrapper->mGtkMozEmbed),
 				link_drag_types_tl, 
 				(GdkDragAction)(GDK_ACTION_COPY | 
@@ -867,6 +1049,8 @@ NS_IMETHODIMP GaleonEventListener::DragGesture(nsIDOMEvent* aMouseEvent)
 				1, NULL);
 	}
 
+	g_free (info);
+
 	return NS_OK;
 }
 
@@ -875,4 +1059,68 @@ NS_IMETHODIMP GaleonEventListener::DragOver(nsIDOMEvent* aMouseEvent)
 	return NS_OK;
 }
 
+nsresult GaleonWrapper::GetFaviconURL (char **favicon_url)
+{
+	nsresult result;
+	PRUint32 links_count;
+	char *relstr;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	nsString strname;
+	strname.AssignWithConversion("LINK");
+
+	nsCOMPtr<nsIDOMNodeList> aLinks;
+	result = aDOMDocument->GetElementsByTagName (strname, 
+						     getter_AddRefs (aLinks));
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	result = aLinks->GetLength (&links_count);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	for (PRUint32 i = 0; i < links_count; i++)
+	{
+		nsCOMPtr<nsIDOMNode> aLink;
+		result = aLinks->Item (i, getter_AddRefs (aLink));
+		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+		
+		result = GetDOMAttribute (aLink, "rel", &relstr);
+		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+		
+		if (relstr == NULL) continue;
+
+		if (strcmp(relstr, "SHORTCUT ICON") == 0)
+		{
+			char *link = NULL;
+			
+			g_free (relstr);
+			
+			result = GetDOMAttribute (aLink, "href", &link);
+			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+			
+			if (link == NULL)
+			{
+				*favicon_url = NULL;
+				return NS_OK;
+			}
+			
+			gchar *url = NULL;
+			
+			nsCOMPtr<nsIDocument> doc = 
+				do_QueryInterface (aDOMDocument);
+			if(!doc) return NS_ERROR_FAILURE;
+			
+			nsIURI *uri = doc->GetDocumentURL();
+			result = uri->Resolve (link, favicon_url);
+			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+			
+			g_free (link);
+			return NS_OK;
+		}
+	}
+
+	return NS_ERROR_FAILURE;
+}
 
