@@ -45,11 +45,25 @@
 #include "nsCOMPtr.h"
 #endif
 
-#ifndef __gen_nsICookieViewer_h__
-#include "nsICookieViewer.h"
+#include "nsIDOMEventReceiver.h"
+#include "nsIPrintOptions.h"
+
+#ifdef CVS_SOURCE
+#	include "nsICookieManager.h"
+#	include	"nsICookie.h"
+#	include "nsIPermissionManager.h"
+#	include "nsIPermission.h"
+#	include "nsIImgManager.h"
+#	define COOKIEMANAGER_ID "@mozilla.org/cookiemanager;1"
+#	define PERMISSIONMANAGER_ID "@mozilla.org/permissionmanager;1"
+#	define IMAGEMANAGER_ID "@mozilla.org/imgmanager;1"
+#	define CACHESERVICE_ID "@mozilla.org/network/cache-service;1"
+#else
+#	include "nsICookieViewer.h"
+#	define COOKIEVIEWER_ID "@mozilla.org/cookieviewer/cookieviewer-world;1"
 #endif
 
-#define COOKIEVIEWER_ID "@mozilla.org/cookieviewer/cookieviewer-world;1"
+class GaleonEventListener;
 
 class GaleonWrapper
 {
@@ -68,45 +82,57 @@ public:
 	nsresult GetZoom (float *aTextZoom);
 
 	nsresult SaveMainDocument (const char *filename);
-	nsresult SaveDocument (nsIDOMDocument *aDomDocument,
-			       const char *filename);
-
-	nsresult Print ();
+	nsresult SaveDocument (const char *filename);
 
+#if MOZILLA_MILESTONE != 81
+	nsresult SaveURLFromCache (const char *url, 
+				   const char *dir, const char *filename);
+#endif
+	nsresult SaveURI (const char *url, const char *filename);
+#ifdef CVS_SOURCE
+	nsresult Print (nsIPrintOptions *options);
+#endif
 	nsresult Find (const char *exp, PRBool matchcase, 
 		       PRBool search_backwards, PRBool *didFind);
 
-	nsresult GetDocumentUrl (nsIDOMDocument *aDOMDocument, char **url);
+	nsresult GetDocumentUrl (char **url);
 
-	nsresult ReloadDocument (nsIDOMDocument *aDOMDocument);
+	nsresult GaleonWrapper::ReloadDocument ();
 
 	nsresult GetSHInfo (PRInt32 *count, PRInt32 *index);
 	nsresult GetSHTitleAtIndex (PRInt32 index, PRUnichar **title);
 
-	nsresult GetKey (nsIDOMKeyEvent *keyEvent, PRUint32 *keycode,
-			 PRInt16 *modifier);
-	nsresult GetModKey (nsIDOMMouseEvent *mouseEvent, PRInt16 *modifier);
-	nsresult GetMouseEventContext (nsIDOMMouseEvent *event, int *context,
-				       char **img, nsIDOMDocument **document,
-				       char **link);
+	nsresult GetEventContext (nsCOMPtr<nsIDOMEventTarget> EventTarget, WrapperContextInfo *info);
+	nsresult GetKeyEventContext (nsIDOMKeyEvent *keyEvent, WrapperKeyEventInfo *info);
+	nsresult GetMouseEventContext (nsIDOMMouseEvent *event, WrapperMouseEventInfo *info);
 	nsresult ScrollbarClicked (nsIDOMMouseEvent *aMouseEvent);
 
 	nsresult CopyHistoryTo (GaleonWrapper *embed);
-
+#ifndef CVS_SOURCE
 	nsresult SetViewSourceMode (PRInt32 mode);
-
+#endif
 	nsresult GoToHistoryIndex (PRInt16 index);
 
 	nsresult SetSitePermission( PRBool permission, PRInt32 type );
 
 	nsresult ForceCharacterSet (char *charset);
 
-	nsresult GetCharacterSets (GHashTable **charsets,
-				   GList **sorted_charset_titles);
+	nsresult GaleonWrapper::IsPageFramed (PRBool *Framed);
+	
+	nsresult GetFaviconURL (char **favicon_url);
 
 	nsCOMPtr<nsIWebBrowser>           mWebBrowser;
+
+#if MOZILLA_MILESTONE != 81
 	nsCOMPtr<nsIDOMEventListener>     mEventListener;
+#else
+	GaleonEventListener *mEventListener;
+#endif
+#if MOZILLA_MILESTONE != 81
 	nsCOMPtr<nsIWebNavigation>        mChromeNav;
+#else
+	nsCOMPtr<nsIDOMEventReceiver>  mEventReceiver;
+#endif
 	GtkMozEmbed *mGtkMozEmbed;
 	GaleonEmbed *embed;
 	PRBool attachListener;
@@ -115,6 +141,8 @@ private:
 	nsresult GetDOMAttribute (nsIDOMNode *node, char *tag,
 				  char **attribute);
 	nsresult GetDOMDocument (nsIDOMDocument **aDOMDocument);
+	nsresult GetMainDOMDocument (nsIDOMDocument **aDOMDocument);
+	nsresult SaveDOMDocument (nsIDOMDocument *aDOMDocument, const char *filename);
 	nsresult GetDocShell (nsIDocShell **aDocShell);
 	nsresult GetSHistory (nsISHistory **aSHistory);
 };
