@@ -20,18 +20,22 @@
 
 
 /* local function prototypes */
-
 static void new_shell (CORBA_ORB orb);
 static gboolean new_view_on_running_shell (char *bookmark_url, char **urls,
 					   int viewstate, int existing);
 
-Galeon_Browser
-impl_Galeon_Browser__create (PortableServer_POA poa, CORBA_Environment *ev);
+/* import from Galeon-impl.c */
+Galeon_Browser impl_Galeon_Browser__create (PortableServer_POA poa, 
+					    CORBA_Environment *ev);
+
+/* imported form embed.c, we choose not to export this in galeon.h as
+ * we use it in a very specific way here which is not general purpose */
+GaleonEmbed *embed_create_with_window (GaleonWindow *window, 
+				       gboolean force_jump);
 
 /* global variables */
-gboolean tabbed_mode = FALSE;
-
-GConfClient* gconf_client;
+gboolean galeon_server_mode;
+GConfClient *gconf_client;
 
 /**
  * get_viewstate:
@@ -39,14 +43,18 @@ GConfClient* gconf_client;
  * Determine the viewstate. A viewstate defines how the URI should be opened.
  * Seemingly cryptic. Might be redone if additional viewstates are added 
  * -Nate
+ *
+ * You're evil ;-)
+ *   -- MattA
  * 
- * DEFAULT: Use default (setting) viewstate
- * WINDOW:  Open up in window
- * TAB:     Open up in tab
- * NONE:    Open up in existing window if there is one
+ * VIEWSTATE_DEFAULT: Use default (setting) viewstate
+ * VIEWSTATE_WINDOW:  Open up in window
+ * VIEWSTATE_TAB:     Open up in tab
+ * VIEWSTATE_NONE:    Open up in existing window if there is one
  */
  
-#define get_viewstate(new_tab, new_win, existing) 2*(!existing & (new_tab ^ new_win)) + (!new_win & !existing)
+#define get_viewstate(new_tab, new_win, existing) \
+        (2 * (!existing & (new_tab ^ new_win)) + (!new_win & !existing))
 
 
 /**
@@ -66,9 +74,10 @@ main (int argc, char *argv[])
 	CORBA_Environment ev;
 
 	gint xpos, ypos, width, height;
+	gchar *restart_argv[] = { "galeon", "--server", NULL };
 
 	gboolean existing = FALSE;  /* load in existing win? */
-	ViewState viewstate = NONE; /* viewstate set to NONE */
+	ViewState viewstate = VIEWSTATE_NONE; /* viewstate set to NONE */
 	gboolean new_tab = FALSE;   /* open new tab in existing win */
 	gboolean new_win = FALSE;
 	int fullscreen = FALSE;     /* open galeon in full screen ? */
@@ -77,6 +86,7 @@ main (int argc, char *argv[])
 
 	GnorbaInitFlags gnorba_flags = 0;
 	GnomeClient *client;
+	gboolean new_user;
 
 	struct poptOption popt_options[] =
 	{
@@ -96,6 +106,10 @@ main (int argc, char *argv[])
 		{ "existing", 'x', POPT_ARG_NONE, &existing, 0,
 		  N_("Attempt to load URL in existing Galeon window"),
 		  NULL },
+		{ "server", 's', POPT_ARG_NONE, &galeon_server_mode, 0,
+		  N_("Don't open any windows; instead act as a server "
+		     "for quick startup of new Galeon instances"),
+		  NULL },		  
 		{ "add-temp-bookmark", 't', POPT_ARG_STRING, &bookmark_url,
 		  0, N_("Add a temp bookmark (don't open any window)"), 
 		  N_("URL")},
@@ -110,7 +124,6 @@ main (int argc, char *argv[])
 	char **urls = NULL;
 	poptContext ctx;
 	char fullurl[PATH_MAX];
-	
 
 	 /* Make stdout line buffered - we only use it for debug info */
 	setvbuf (stdout, NULL, _IOLBF, 0);
@@ -132,36 +145,37 @@ main (int argc, char *argv[])
 		urls = (char **) poptGetArgs(ctx);
 
 	orb = oaf_init (argc, argv);
-	gnome_vfs_init ();
-
-	/* initialise gconf */
-	/* FIXME: check errors */
-	gconf_init(argc, argv, NULL);
-
-	gconf_client = gconf_client_get_default();
 
 	viewstate = get_viewstate(new_tab, new_win, existing);
 
 	/* check if galeon is already running */
 	if (new_view_on_running_shell(bookmark_url, urls, viewstate, existing))
 	{
-		exit (0);
+		return 0;
 	}
 	else
 	{
 		new_shell (orb);
 	}
 
-	/* initialize gmodule */
+	gnome_vfs_init ();
+
+	/* initialise gconf */
+	/* FIXME: check errors */
+	gconf_init(argc, argv, NULL);
+	gconf_client = gconf_client_get_default();
+
+ 	/* initialize gmodule */
 	mod_self = g_module_open ( NULL, 0 );
-		
+
 	/* initialise GLADE */
 	glade_gnome_init();
 
 	/* Add a temp bookmark and exit if needed */
-	if (bookmark_url) {
+	if (bookmark_url)
+	{
 		temp_bookmarks_init ();
-		add_temp_bookmark (SITE, NULL, bookmark_url, NULL);
+		add_temp_bookmark (BM_SITE, NULL, bookmark_url, NULL);
 		temp_bookmarks_save ();
 		exit (0);
 	}
@@ -173,7 +187,7 @@ main (int argc, char *argv[])
 	initialise_colours ();
 
 	/* check if this is the first time this user has run Galeon */
-	newuser_check ();
+	new_user = newuser_check ();
 
 	/* set the profile dir for gtkmozembed */
 	gtk_moz_embed_set_profile_path
@@ -187,10 +201,11 @@ main (int argc, char *argv[])
 	}
 
 	/* connect our session recovery to GNOME session */
-	/* FIXME: much more could be done here to allow gnome to restart
-	 * galeon properly, I need to look at the gnome-client stuff
-	 * some more -- MattA 27/2/2001 */
 	client = gnome_master_client ();
+	gnome_client_set_restart_command (client, galeon_server_mode ? 2 : 1,
+					  restart_argv);
+	gnome_client_set_restart_style (GNOME_CLIENT (client),
+					GNOME_RESTART_IF_RUNNING);
 	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
 			    (GtkSignalFunc)client_save_yourself_cb, NULL);
 
@@ -198,8 +213,8 @@ main (int argc, char *argv[])
 	gnome_config_set_int("/galeon/General/crashed", TRUE);
 	gnome_config_sync();
 
-	
-	if (urls) {
+	if (urls != NULL)
+	{
 		/* create a browser for each argument popt did not filter */
 		for (i = 0; urls[i]; i++)
 		{
@@ -218,38 +233,53 @@ main (int argc, char *argv[])
 	}
 
 	/* check if browser exists, or create default one */
-	if (browser_cnt == 0) {
+	if (browser_cnt == 0 && !galeon_server_mode && !new_user)
+	{
 		GaleonEmbed *embed;
-		embed = embed_create_default (NULL, TRUE);
+		GaleonWindow *window;
 
+		/* create a default window */
+		window = window_create ();
 
+		/* show as fullscreen? */
 		if (fullscreen)
 		{
-			GtkWidget *mi;
-
-			mi = embed->parent_window->view_fullscreen;
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mi),
-						       TRUE);
+			gtk_check_menu_item_set_active
+				(GTK_CHECK_MENU_ITEM (window->view_fullscreen),
+				 TRUE);
 		} 
 		/* collect geometry information */
 		else if (gnome_parse_geometry (geometry, &xpos, &ypos,
-					       &width, &height)) 
+					       &width, &height))
 		{
-			GaleonWindow *window = embed->parent_window;
-			GdkWindow *gdk_window = window->WMain->window;
+			width = MIN (width, gdk_screen_width ());
+			height = MIN (height, gdk_screen_height ());
 
-			width = MIN (width, gdk_screen_width());
-			height = MIN (height, gdk_screen_height());
-
-			gdk_window_move_resize (gdk_window,
+			gdk_window_move_resize (window->WMain->window,
 						xpos, ypos,
 						width, height);
 		}
+
+		/* show it */
+		window_set_visibility (window, TRUE);
+
+		/* make sure the user can see it */
+		while (gtk_events_pending ())
+		{
+			gtk_main_iteration();
+		}
+
+		/* build the default embed inside it */
+		embed = embed_create_with_window (window, FALSE);
+		embed_go_default (NULL, embed);
+		mozilla_init_promptService ();
 		browser_cnt++;
 	}
 
 	if (ctx)
+	{
 		poptFreeContext(ctx);
+	}
 
 	/* enter the main GTK event processing loop */
 	gtk_main ();
