@@ -17,108 +17,7 @@
  */
 
 #include "galeon.h"
-#include <gnome.h>
-
-/* local function prototyprs */
-void window_menu_zoom_in_activate_cb (GtkCheckMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
-				       GaleonWindow *window);
-void window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
-					     GaleonWindow *window);
-void window_menu_about_activate_cb (GtkMenuItem *menuitem,
-				    GaleonWindow *window);
-void window_menu_manual_activate_cb (GtkMenuItem *menuitem, 
-				     GaleonWindow *window);
-void window_menu_new_window_activate_cb (GtkMenuItem *menuitem,
-					 GaleonWindow *window);
-void window_menu_new_tab_activate_cb (GtkMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_open_activate_cb (GtkMenuItem *menuitem,
-				   GaleonWindow *window);
-void window_menu_save_as_activate_cb (GtkMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_exit_activate_cb (GtkMenuItem *menuitem,
-				   GaleonWindow *window);
-void window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem, 
-						GaleonWindow *window);
-void window_menu_print_activate_cb (GtkMenuItem *menuitem,
-				    GaleonWindow *window);
-void window_menu_close_activate_cb (GtkMenuItem *menuitem,
-				    GaleonWindow *window);
-void window_menu_view_source_mode_activate_cb (GtkCheckMenuItem *menuitem,
-					       GaleonWindow *window);
-void window_menu_show_temp_bookmarks_activate_cb (GtkMenuItem *menuitem,
-						  GaleonWindow *window);
-void window_menu_add_temp_bookmark_activate_cb (GtkMenuItem *menuitem,
-						GaleonWindow *window);
-void window_menu_toolbar_activate_cb (GtkCheckMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
-					GaleonWindow *window);
-void window_menu_menubar_activate_cb (GtkCheckMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_fullscreen_activate_cb (GtkCheckMenuItem *menuitem,
-					 GaleonWindow *window);
-void window_menu_own_colors_activate_cb (GtkCheckMenuItem *menuitem,
-					 GaleonWindow *window);
-void window_menu_enable_javascript_activate_cb (GtkCheckMenuItem *menuitem, 
-						GaleonWindow *window);
-void window_menu_enable_proxy_activate_cb (GtkCheckMenuItem *menuitem,
-                                                GaleonWindow *window);
-void window_menu_own_fonts_activate_cb (GtkCheckMenuItem *menuitem,
-					GaleonWindow *window);
-void window_menu_galeon_mime_handling_activate_cb (GtkCheckMenuItem *menuitem, 
-						   GaleonWindow *window);
-void window_menu_enable_java_activate_cb (GtkCheckMenuItem *menuitem, 
-					  GaleonWindow *window);
-void window_menu_images_never_activate_cb (GtkCheckMenuItem *menuitem, 
-					   GaleonWindow *window);
-void window_menu_images_current_server_activate_cb (GtkCheckMenuItem *menuitem,
-						    GaleonWindow *window);
-void window_menu_images_always_activate_cb (GtkCheckMenuItem *menuitem,
-					    GaleonWindow *window);
-void window_menu_animate_always_activate_cb (GtkCheckMenuItem *menuitem, 
-					     GaleonWindow *window);
-void window_menu_animate_once_through_activate_cb (GtkCheckMenuItem *menuitem,
-						   GaleonWindow *window);
-void window_menu_animate_never_activate_cb (GtkCheckMenuItem *menuitem,
-					    GaleonWindow *window);
-void window_menu_back_activate_cb (GtkMenuItem *menuitem,
-				   GaleonWindow *window);
-void window_menu_forward_activate_cb (GtkMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_stop_activate_cb (GtkMenuItem *menuitem,
-				   GaleonWindow *window);
-void window_menu_reload_activate_cb (GtkMenuItem *menuitem,
-				     GaleonWindow *window);
-void window_menu_start_page_activate_cb (GtkMenuItem *menuitem,
-					 GaleonWindow *window);
-void window_menu_find_activate_cb (GtkMenuItem *menuitem,
-				   GaleonWindow *window);
-void window_menu_select_dir_activate_cb (GtkMenuItem *menuitem,
-					 GaleonWindow *window);
-void window_menu_prefs_activate_cb (GtkMenuItem *menuitem,
-				    GaleonWindow *window);
-void window_menu_view_source_activate_cb (GtkMenuItem *menuitem,
- 					  GaleonWindow *window);
-void window_menu_history_activate_cb (GtkMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_menu_open_url_activate_cb (GtkMenuItem *menuitem,
-				       GaleonWindow *window);
-void window_block_images_from_activate_cb (GtkMenuItem *menuitem,
-					   GaleonWindow *window);
-void window_allow_images_from_activate_cb (GtkMenuItem *menuitem,
-					   GaleonWindow *window);
-void window_view_stored_sites_activate_cb (GtkMenuItem *menuItem,
-					   GaleonWindow *window);
-void window_cookie_manager_launch_cb (GtkMenuItem *menuitem,
-				      GaleonWindow *window);
-void window_block_cookies_from_activate_cb (GtkMenuItem *menuitem,
-					    GaleonWindow *window);
-void window_allow_cookies_from_activate_cb (GtkMenuItem *menuitem,
-					    GaleonWindow *window);
-
+#include "menubar.h"
 
 /**
  * window_menu_select_dir_activate_cb: show the download directory dialog (menu)
@@ -127,27 +26,22 @@ void
 window_menu_select_dir_activate_cb (GtkMenuItem *menuitem,
 				    GaleonWindow *window)
 {
-	GtkWidget *dirselect;
 	GtkWidget *dialog;
 	char *olddir;
 	GladeXML *gxml;
 
 	return_if_not_window (window);
 
-	gxml = glade_xml_new(glade_file(), "dirselection");
-	dialog = glade_xml_get_widget (gxml, "dirselection");
-	glade_xml_signal_autoconnect_full (gxml,
-		(GladeXMLConnectFunc) glade_signal_connect_func, window);
-	dirselect = glade_xml_get_widget(gxml, "dirselection");
+	gxml = glade_widget_new ("dirselection", &dialog, window);
 
 	olddir = gnome_config_get_string("/galeon/Downloading/download_dir");
 
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(dirselect), olddir);
+	gtk_file_selection_set_filename(GTK_FILE_SELECTION (dialog), olddir);
   
 	gtk_widget_set_sensitive(GTK_WIDGET(GTK_FILE_SELECTION(
-						dirselect)->file_list), FALSE);
-	gtk_widget_show(dirselect);
-	window_set_layer(dirselect);
+						dialog)->file_list), FALSE);
+	gtk_widget_show(dialog);
+	window_set_layer(dialog);
 }
 
 /**
@@ -156,73 +50,7 @@ window_menu_select_dir_activate_cb (GtkMenuItem *menuitem,
 void
 window_menu_prefs_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
-	GladeXML *gxml;
-	GtkWidget *default_charset_combo;
-	GtkWidget *hpane;
-	gint x, y, width, height, pane_pos;
-
-	return_if_not_window (window);
-
-	while (gtk_events_pending())
-		gtk_main_iteration();
-  
-	if (GTK_IS_WIDGET(prefs))
-	{
-		gdk_window_raise(prefs->window);
-		return;
-	}
-
-	gxml = glade_xml_new(glade_file(), "Preferences");
-	prefs = glade_xml_get_widget(gxml, "Preferences");
-	glade_xml_signal_autoconnect_full (gxml, glade_signal_connect_func, 
-					   prefs);
-
-	/* get the saved geometry */
-	gnome_config_push_prefix("/galeon/State/");
-	x = gnome_config_get_int("prefs_dialog_x=-1");
-	y = gnome_config_get_int("prefs_dialog_y=-1");
-	width = gnome_config_get_int("prefs_dialog_width=570");
-	height = gnome_config_get_int("prefs_dialog_height=480");
-	pane_pos = gnome_config_get_int("prefs_dialog_hpane=150");
-	gnome_config_pop_prefix();
-
-	if (x != -1 && y != -1)
-		gtk_widget_set_uposition(prefs, x, y);
-	gtk_window_set_default_size(GTK_WINDOW(prefs), width, height);
-
-	hpane = glade_lookup_widget(prefs, "hpaned2");
-	gtk_paned_set_position(GTK_PANED(hpane), pane_pos);
-
-	/* initialize the default charset combo */
-	default_charset_combo = glade_xml_get_widget(gxml,"default_charset_combo");
-	gtk_combo_set_popdown_strings(GTK_COMBO(default_charset_combo),sorted_charset_titles);
-	
-	preferences_set_dialog_to_config(prefs);
-
-	spinner_fill_iconlist (GNOME_ICON_LIST(glade_xml_get_widget(gxml, "spinner_iconlist")));
-
-	gtk_widget_show(prefs);
-	window_set_layer(prefs);
-}
-
-void 
-window_menu_bookmark_activate_cb (GtkMenuItem *menuitem, gpointer user_data) 
-{
-        GtkWidget *WMain = window_lookup_wmain(GTK_WIDGET (menuitem));
-        GaleonWindow *window = (GaleonWindow*) gtk_object_get_data
-		(GTK_OBJECT (WMain), "GaleonWindow");
-	BookMarkItem *bm = (BookMarkItem *)user_data;
-
-	return_if_not_window (window);
-
-	if (strstr(bm->url, "%s"))
-	{
-		bookmarks_smart_bm_dialog(window, bm);
-	}
-	else
-	{
-	    window_load_url(window, bm->url);
-	}
+	preferences_open ();
 }
 
 /**
@@ -234,7 +62,7 @@ window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_edit_bookmarks (window->active_embed);
+	bookmarks_editor_show_dialog (window, STANDARD_BOOKMARKS_EDITOR);
 }
 
 /**
@@ -253,10 +81,10 @@ window_menu_find_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 void 
 window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
-        GtkWidget *WMain = window_lookup_wmain (GTK_WIDGET (menuitem));
+        GtkWidget *WMain = window_lookup_widget (GTK_WIDGET(menuitem), "WMain");
         GaleonWindow *window = (GaleonWindow *) gtk_object_get_data
 		(GTK_OBJECT (WMain), "GaleonWindow");
-        history_item *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));
+        HistoryItem *hi = gtk_object_get_user_data (GTK_OBJECT (menuitem));
 
 	return_if_not_window (window);
 	window_load_url(window, hi->url);
@@ -285,7 +113,7 @@ window_menu_view_source_activate_cb (GtkMenuItem *menuitem,
 	return_if_not_window (window);
 
 	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-	embed_view_source (window->active_embed, NULL, !tabbed_mode);
+	embed_view_source (window->active_embed, TRUE, !tabbed_mode);
 }
 
 /**
@@ -295,7 +123,7 @@ void window_menu_history_activate_cb (GtkMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	window_show_history (window);
+	history_show_dialog (window);
 }
 
 /**
@@ -324,7 +152,7 @@ window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	if (!about)
 	{
 		about = gnome_about_new (_(PACKAGE), VERSION,
-				 _("Copyright (C) 2000 Marco Pesenti Gritti"),
+				 _("Copyright (C) 2000-2001 Marco Pesenti Gritti"),
 				 author, _("A GNOME browser based on mozilla"),
 				 SHARE_DIR"/logo.png");
 
@@ -333,7 +161,7 @@ window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 		gtk_widget_show (about);
 	}
 
-	window_set_layer(about);
+	window_set_layer (about);
 }
 
 /**
@@ -397,7 +225,7 @@ void
 window_menu_save_as_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
-	embed_show_save_dialog (window->active_embed, NULL);
+	embed_save_document (window->active_embed, TRUE);
 }
 
 /** 
@@ -407,7 +235,13 @@ void
 window_menu_exit_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
-	galeon_quit (window);
+
+	/* We call the quit function indirectly, to make sure control returns
+	   to the main loop first.  This is because we might have been
+	   activated from within the mozembed_key_press callback, and if the
+	   widget is destroyed before returning from the callback, very bad
+	   things happen.  --Josh */
+	gtk_timeout_add (50, (GtkFunction) galeon_quit, window);
 }
 
 /** 
@@ -427,16 +261,8 @@ window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
 void
 window_menu_print_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
-	GtkWidget *dialog = NULL;
-
 	return_if_not_window (window);
-
-	if (!mozilla_print (window->active_embed))
-	{
-		dialog = gnome_error_dialog ("Not yet implemented.");
-		gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
-		window_set_layer (dialog);
-	}
+	print_show_dialog (window);
 }
 
 
@@ -453,7 +279,7 @@ window_menu_close_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 
 /** 
  * window_menu_back_activate_cb: back menu activated
-*/
+ */
 void
 window_menu_back_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
@@ -462,10 +288,20 @@ window_menu_back_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	gtk_moz_embed_go_back (window->active_embed->mozEmbed);
 }
 
+/** 
+ * window_menu_up_activate_cb: up menu activated
+ */
+void
+window_menu_up_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	embed_go_up (window->active_embed, 0);
+}
 
 /** 
  * window_menu_forward_activate_cb: forward menu activated
-*/
+ */
 void
 window_menu_forward_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
@@ -474,10 +310,31 @@ window_menu_forward_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	gtk_moz_embed_go_forward(window->active_embed->mozEmbed);
 }
 
+/** 
+ * window_menu_prev_tab_activate_cb: previous tab activated
+ */
+void
+window_menu_prev_tab_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	gtk_notebook_prev_page (GTK_NOTEBOOK (window->notebook));
+}
+
+/** 
+ * window_menu_next_tab_activate_cb: next tab activated
+ */
+void
+window_menu_next_tab_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	gtk_notebook_next_page (GTK_NOTEBOOK (window->notebook));
+}
 
 /** 
  * window_menu_stop_activate_cb: stop menu activated
-*/
+ */
 void
 window_menu_stop_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
@@ -486,10 +343,9 @@ window_menu_stop_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	gtk_moz_embed_stop_load(window->active_embed->mozEmbed);
 }
 
-
 /** 
  * window_menu_reload_activate_cb: reload menu activated
-*/
+ */
 void
 window_menu_reload_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
@@ -510,26 +366,6 @@ window_menu_start_page_activate_cb (GtkMenuItem *menuitem,
 	window_start_button_clicked_cb (GTK_BUTTON (window->BStart), window);
 }
 
-/**
- * Sets the view source mode for the current window
- */
-void 
-window_menu_view_source_mode_activate_cb (GtkCheckMenuItem *menuitem,
-					  GaleonWindow *window)
-{
-	GaleonEmbed *embed;
-
-	return_if_not_window (window);
-	embed = window->active_embed;
-	return_if_not_embed (embed);
-
-	/* toggle only if needed */
-	if (embed->viewing_source != menuitem->active)
-	{
-		embed_toggle_view_source_mode (embed);
-	}
-}
-
 /** 
  * window_menu_show_temp_bookmarks_activate_cb: show temp bookmarks window
  */
@@ -538,7 +374,7 @@ window_menu_show_temp_bookmarks_activate_cb (GtkMenuItem *menuitem,
 					     GaleonWindow *window)
 {
 	return_if_not_window (window);
-	window_show_temporary_bookmarks (window);
+	bookmarks_editor_show_dialog (window, COMPACT_BOOKMARKS_EDITOR);
 }
 
 void
@@ -601,6 +437,40 @@ void window_menu_statusbar_activate_cb (GtkCheckMenuItem *menuitem,
 	}
 }
 
+/** 
+ * window_menu_bookmarks_dock_activate_cb: hide/show the bookmarks dock
+ */
+void window_menu_bookmarks_dock_activate_cb (GtkCheckMenuItem *menuitem,
+					     GaleonWindow *window)
+{
+	return_if_not_window (window);
+	if (menuitem->active)
+	{
+		bookmarks_editor_show_dock (window);
+	}
+	else
+	{
+		bookmarks_editor_hide_dock ();
+	}
+}
+
+/** 
+ * window_menu_history_dock_activate_cb: hide/show the history dock
+ */
+void window_menu_history_dock_activate_cb (GtkCheckMenuItem *menuitem,
+					   GaleonWindow *window)
+{
+	return_if_not_window (window);
+	if (menuitem->active)
+	{
+		history_show_dock (window);
+	}
+	else
+	{
+		history_hide_dock ();
+	}
+}
+
 /** 
  * window_menu_fullscreen_activate_cb: set fullscreen mode
  *
@@ -617,7 +487,7 @@ window_menu_own_colors_activate_cb (GtkCheckMenuItem *menuitem,
 				    GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_use_provided_colors (!menuitem->active);
+	preferences_set_use_own_colors (menuitem->active);
 }
 
 void
@@ -642,7 +512,7 @@ window_menu_own_fonts_activate_cb (GtkCheckMenuItem *menuitem,
 				   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_use_provided_fonts (!menuitem->active);
+	preferences_set_use_own_fonts (menuitem->active);
 }
 
 void
@@ -701,7 +571,8 @@ window_block_images_from_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	mozilla_set_image_permission (window->active_embed, FALSE);
+
+	mozilla_set_permission (window->active_embed, FALSE, IMAGEPERMISSION);
 }
 
 void
@@ -710,7 +581,8 @@ window_allow_images_from_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	mozilla_set_image_permission (window->active_embed, TRUE);
+
+	mozilla_set_permission (window->active_embed, TRUE, IMAGEPERMISSION);
 }
 
 void
@@ -718,7 +590,7 @@ window_view_stored_sites_activate_cb (GtkMenuItem *menuItem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	cookies_show_imagesites();
+	cookies_show_page (IMAGESITES);
 }
 
 void
@@ -768,7 +640,8 @@ window_block_cookies_from_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	mozilla_set_cookie_permission (window->active_embed, FALSE);
+
+	mozilla_set_permission (window->active_embed, FALSE, COOKIEPERMISSION);
 }
 
 void
@@ -777,7 +650,8 @@ window_allow_cookies_from_activate_cb (GtkMenuItem *menuitem,
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	mozilla_set_cookie_permission (window->active_embed, TRUE);
+
+	mozilla_set_permission (window->active_embed, TRUE, COOKIEPERMISSION);	
 }
 
 void
@@ -808,7 +682,25 @@ window_menu_encoding_activate_cb (GtkMenuItem *menuitem,
 	return_if_not_embed (window->active_embed);
 
 	gtk_label_get (GTK_LABEL(GTK_BIN(menuitem)->child), &menulabel);
-	mozilla_force_character_set (g_hash_table_lookup (charsets, menulabel),
-				     window->active_embed);
+	mozilla_force_character_set (window->active_embed,
+				     g_hash_table_lookup (charsets,
+							  menulabel));
 	embed_reload (window->active_embed);
 }
+
+#ifdef CVS_SOURCE
+void
+window_view_passwords_activate_cb (GtkMenuItem *menuItem,
+			           GaleonWindow *window)
+{
+	return_if_not_window (window);
+	cookies_show_page (PASSWORDS);
+}
+void
+window_view_password_sites_activate_cb (GtkMenuItem *menuItem,
+				        GaleonWindow *window)
+{
+	return_if_not_window (window);
+	cookies_show_page (PASSWORDSITES);
+}
+#endif
