@@ -28,7 +28,7 @@ mime_get_action (const gchar *mime_type)
 	for (l = mime_types; l != NULL; l = l->next)
 		if (!strcmp (mime_type, ((MimeItem *) l->data)->name))
 			return ((MimeItem *) l->data)->action;
-	return UNKNOWN;
+	return MIME_UNKNOWN;
 }
 
 void 
@@ -52,23 +52,21 @@ void
 mime_ask_action (const gchar *url, const gchar *mime_type, 
 		 GaleonEmbed *embed)
 {
-	mime_ask_action_dialog *dialog = g_new0 (mime_ask_action_dialog, 1);
+	MimeAskActionDialog *dialog = g_new0 (MimeAskActionDialog, 1);
 	GtkWidget *label;
 	gchar *msg;
 		
 	dialog->url = g_strdup (url);
 	dialog->mime_type = g_strdup (mime_type);
 	dialog->embed = embed;
-	dialog->gxml = glade_xml_new (glade_file (), "mime_ask_action_dialog");
+	dialog->gxml = glade_widget_new ("mime_ask_action_dialog", NULL,
+					 dialog);
 
 	label = glade_xml_get_widget (dialog->gxml, "mime_ask_action_message");
 	msg = g_strdup_printf (_("The url %s has MIME type %s.\n"
 				 "What do you want to do with it?"), url, mime_type);
 	gtk_label_set_text (GTK_LABEL (label), msg);
 	g_free (msg);
-	glade_xml_signal_autoconnect_full 
-		(dialog->gxml, (GladeXMLConnectFunc) glade_signal_connect_func, dialog);
-
 }
 
 MimeItem *
