@@ -19,6 +19,12 @@
 #include "galeon.h"
 #include <dirent.h>
 
+/* external function prototypes imported from libxml1 */
+extern int isolat1ToUTF8(unsigned char* out, int outlen,
+			 const unsigned char* in, int inlen);
+extern int UTF8Toisolat1(unsigned char* out, int outlen,
+			 const unsigned char* in, int inlen);
+
 /** Styles for coloured text */
 GtkStyle *red_text_style;
 GtkStyle *yellow_text_style;
@@ -39,6 +45,9 @@ const char *known_protocols[] =
 	"resource",
 	"javascript",
 	"jar",
+#ifdef CVS_SOURCE
+	"view-source",
+#endif
        	NULL /* terminator: must be last! */
 };
 
@@ -78,11 +87,24 @@ const char *known_mime_types[] =
 
 /* Local prototypes */
 static void add_bf_menu (GtkWidget *menu, char* label, int index, 
-			 GaleonEmbed *embed);
+			 GaleonEmbed *embed, int level);
 static void free_string_array (char *array[], int size);
 static void unrecognised_protocol_cb(gint reply, gpointer url);
-inline gchar * glade_file(void);
 static gboolean is_punctuation (gchar c);
+gint file_copy_status (GnomeVFSXferProgressInfo *info,gpointer data);
+void favicon_download_completed_cb (gpointer data);
+gchar *favicon_filename (gchar *favicon_url);
+static void glade_signal_connect_func (const gchar *cb_name, GtkObject *obj, 
+				       const gchar *signal_name, 
+				       const gchar *signal_data,
+				       GtkObject *conn_obj, 
+				       gboolean conn_after,
+				       gpointer user_data);
+static void misc_ask_question_cb (gint reply, gpointer data);
+static gchar *new_num_accel_str (gint num, gchar *text, gboolean lettersok);
+static GtkWidget * new_num_accel_menu_item (gint num, gchar *origtext, 
+					    gboolean lettersok, 
+					    GtkWidget *menu);
 
 /**
  * glade_lookup_widget
@@ -192,52 +214,100 @@ user_file(gchar *fname, gint critical)
 	return NULL;
 }
 
-/* shortcut for old code */
-inline gchar * glade_file(void)
+/**
+ * glade_widget_new: build a new widget of the provided name, with all
+ * signals attached and data set to the provided parameter.
+ */
+GladeXML *
+glade_widget_new (const gchar *widget_name, GtkWidget **root, gpointer data)
 {
-	return user_file("galeon.glade", TRUE);
+	GladeXML *gxml;
+	static gchar *glade_file = NULL;
+
+	/* find the glade file */
+	if (glade_file == NULL)
+	{
+		glade_file = user_file ("galeon.glade", TRUE);
+		g_return_val_if_fail (glade_file != NULL, NULL);
+	}
+
+	/* build the widget */
+	/* note that libglade automatically caches the parsed file,
+	 * so we don't need to worry about the efficiency of this */
+	gxml = glade_xml_new (glade_file, widget_name);
+	g_return_val_if_fail (gxml != NULL, NULL);
+
+	/* lookup the root widget if requested */
+	if (root != NULL)
+	{
+		*root = glade_xml_get_widget (gxml, widget_name);
+	}
+
+	/* connect signals and data */
+	glade_xml_signal_autoconnect_full
+		(gxml, glade_signal_connect_func, data);
+
+	/* return xml document for subsequent widget lookups */
+	return gxml;
 }
 
 /*
  * glade_signal_connect_func: used by glade_xml_signal_autoconnect_full
  */
-void
-glade_signal_connect_func(const gchar *cb_name, GtkObject *obj, 
-	const gchar *signal_name, const gchar *signal_data, GtkObject *conn_obj,
-	gboolean conn_after, gpointer user_data)
+static void
+glade_signal_connect_func (const gchar *cb_name, GtkObject *obj, 
+			   const gchar *signal_name, const gchar *signal_data,
+			   GtkObject *conn_obj, gboolean conn_after,
+			   gpointer user_data)
 {
 	gpointer handler_func;
 	
 	/*g_print( "glade_signal_connect_func: cb_name = '%s', signal_name = '%s', signal_data = '%s'\n",
 	  cb_name, signal_name, signal_data ); */
 	
-	if( g_module_symbol(mod_self, cb_name, &handler_func ) ) {
+	if (g_module_symbol(mod_self, cb_name, &handler_func))
+	{
 		/* found callback */
-		if( conn_obj ) {
-			if( conn_after ) {
-				gtk_signal_connect_object_after( obj, signal_name, 
-					handler_func, conn_obj );
-			} else {
-				gtk_signal_connect_object( obj, signal_name, 
-					handler_func, conn_obj );
+		if (conn_obj)
+		{
+			if (conn_after)
+			{
+				gtk_signal_connect_object_after
+					(obj, signal_name, 
+					 handler_func, conn_obj);
 			}
-		} else {
+			else
+			{
+				gtk_signal_connect_object
+					(obj, signal_name, 
+					 handler_func, conn_obj);
+			}
+		}
+		else
+		{
 			/* no conn_obj; use standard connect */
 			gpointer data = NULL;
 			
 			data = user_data;
 			
-			if( conn_after ) {
-				gtk_signal_connect_after( obj, signal_name, 
-					handler_func, data );
-			} else {
-				gtk_signal_connect( obj, signal_name, 
-					handler_func, data );
+			if (conn_after)
+			{
+				gtk_signal_connect_after
+					(obj, signal_name, 
+					 handler_func, data);
+			}
+			else
+			{
+				gtk_signal_connect
+					(obj, signal_name, 
+					 handler_func, data);
 			}
 		}
 	}
-	else g_warning("callback function not found: %s", cb_name);
-
+	else
+	{
+		g_warning("callback function not found: %s", cb_name);
+	}
 }
 
 /**
@@ -299,17 +369,17 @@ handle_foreign_mime_types (const char *url, GaleonEmbed *embed)
 	/* see if we already know what to */
 	action = mime_get_action (mime_type);
 	switch (action) {
-	case UNKNOWN:
-	case ASK_ACTION:
+	case MIME_UNKNOWN:
+	case MIME_ASK_ACTION:
 		mime_ask_action (url, mime_type, embed);
 		break;
-	case LEAVE_TO_MOZILLA:
+	case MIME_LEAVE_TO_MOZILLA:
 		return FALSE;
 		break;
-	case SAVE_TO_DISK:
+	case MIME_SAVE_TO_DISK:
 	       	save_url(url);
 		break;
-	case RUN_PROGRAM:
+	case MIME_RUN_PROGRAM:
 		/* TODO */
 		gnome_error_dialog ("Not Implemented!");
 		break;
@@ -338,7 +408,7 @@ handle_foreign_protocols (const char *url)
 
 	/* find the colon */
 	length = strlen(url);
-	for (i = 0; i < length && url[i] != ':'; i++)
+	for (i = 0; i < length && url[i] != ':' && url[i] != '.'; i++)
 		;
 	
 	/* did we run over? */
@@ -348,6 +418,13 @@ handle_foreign_protocols (const char *url)
 		return FALSE;
 	}
 
+	/* did we find a period? */
+	if (url[i] == '.')
+	{
+		/* must be a site designation, not a protocol */
+		return FALSE;
+	}
+
 	/* okay, duplicate this */
 	protocol = g_strndup(url, i);
 
@@ -528,9 +605,7 @@ ask_dir (const gchar *url)
 	char *olddir;
 	
 	/* build the directory selector */
-	gxml = glade_xml_new (glade_file(), "dirselection2");
-	glade_xml_signal_autoconnect (gxml);
-	dirselect = glade_xml_get_widget (gxml, "dirselection2");
+	gxml = glade_widget_new ("dirselection2", &dirselect, NULL);
 
 	/* set it up with the standard download directory */
 	olddir = gnome_config_get_string ("/galeon/Downloading/download_dir");
@@ -576,7 +651,7 @@ int g_str_is_url(gchar * string)
 	return 0;
 }
 
-
+#ifndef CVS_SOURCE
 gchar *
 init_cache_dir(void)
 { 
@@ -592,6 +667,7 @@ init_cache_dir(void)
 	g_free(confdir);
 	return cachedir;
 }
+#endif
 
 /**
  * Frees an array of strings
@@ -605,13 +681,149 @@ free_string_array (char *array[], int size)
 	g_free (array);
 }
 
+/**
+ * Creates a string with a numbered/lettered accel (caller must free)
+ * returns NULL if num is out of the range of acceleratable nums/letters
+ */
+static gchar *
+new_num_accel_str (gint num, gchar *text, gboolean lettersok)
+{
+	gchar *label = NULL;
+	if (num < 9)
+		label = g_strdup_printf	("_%i. %s", num+1, text);
+	else if (num == 9)
+		label = g_strdup_printf	("_%i. %s", 0, text);
+	else if (num < 36 && lettersok)
+		label = g_strdup_printf	("_%c. %s", 'a'+num-10, text);
+	return label;
+}
+
+/**
+ * Sets a labels text with shortcut key handling in the standard _<key> accel way
+ */
+void
+label_set_accel_text (gchar *text, GtkWidget *label, GtkWidget *menu, 
+		      GtkWidget *item)
+{
+	gint tmp_key = gtk_label_parse_uline (GTK_LABEL (label), text);
+	GtkAccelGroup *menu_accels = 
+		gtk_menu_ensure_uline_accel_group (GTK_MENU (menu));
+	gtk_widget_add_accelerator (item, "activate_item", menu_accels,
+			tmp_key, 0, 0);
+}
+
+/**
+ * Strip the _ out of a string like gtk_label_parse_uline would do
+ * caller is responsible for freeing the returned string
+ */
+gchar *
+strip_uline_accel (const gchar *text)
+{
+	GString *out;
+	const gchar *u, *cur = text;
+	out = g_string_new (NULL);
+	while ((u = strchr (cur, '_')))
+	{
+		if (*(u+1) == '_')
+		{
+			/* __ in the string is equal to _ in the output 
+			 * so include the _ in the output, skip over the 
+			 * second _ and continue scanning. */
+			g_string_sprintfa (out, "%.*s", u - cur + 1, cur);
+			cur = u + 2;
+		} else {
+			/* copy everything before the _ and skip over it */
+			g_string_sprintfa (out, "%.*s", u - cur , cur);
+			cur = u + 1;
+			/* only one accel per string, so we are done now */
+			break;
+		}
+	}
+	if (cur && *cur)
+	{
+		/* attach rest of string */
+		g_string_append (out, cur);
+	}
+
+	u = out->str;
+	g_string_free (out, FALSE); /* don't free char data, caller must */
+	return (gchar *)u;
+}
+
+/**
+ * Escape _'s in string such that the gtk_label_parse_uline will display as normal
+ * caller is responsible for freeing the returned string
+ */
+gchar *
+escape_uline_accel (const gchar *text)
+{
+	GString *out;
+	const gchar *u, *cur = text;
+	out = g_string_new (NULL);
+	while ((u = strchr (cur, '_')))
+	{
+		/* All we need to do is double every _, so include the _ in 
+		 * the output, add another _, and continue scanning. */
+		g_string_sprintfa (out, "%.*s_", u - cur + 1, cur);
+		cur = u + 1;
+	}
+	if (cur && *cur)
+	{
+		/* attach rest of string */
+		g_string_append (out, cur);
+	}
+
+	u = out->str;
+	g_string_free (out, FALSE); /* don't free char data, caller must */
+	return (gchar *)u;
+}
+
+/**
+ * Creates a label with a numbered/lettered accel
+ */
+GtkWidget *
+new_num_accel_label (gint num, gchar *origtext, gboolean lettersok, 
+		     GtkWidget *menu, GtkWidget *item)
+{
+	gchar *text = new_num_accel_str(num, origtext, lettersok);
+	if (text == NULL)
+		return gtk_label_new (origtext);
+	else
+	{
+		GtkWidget *label = gtk_label_new ("");
+		label_set_accel_text (text, label, menu, item);
+		g_free(text);
+		return label;
+	}
+}
+
+/**
+ * Creates a menu item with a numbered/lettered accel
+ */
+static GtkWidget *
+new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok, 
+			 GtkWidget *menu)
+{
+	gchar *text = new_num_accel_str(num, origtext, lettersok);
+	if (text == NULL)
+		return gtk_menu_item_new_with_label (text);
+	else
+	{
+		GtkWidget *item = gtk_menu_item_new_with_label ("");
+		label_set_accel_text (text, GTK_BIN (item)->child, menu, item);
+		g_free(text);
+		return item;
+	}
+}
+
 /**
  * Adds a menuitem to a back/forward history menu
  */
 static void
-add_bf_menu (GtkWidget *menu, char* label, int index, GaleonEmbed *embed)
+add_bf_menu (GtkWidget *menu, char* label, int index, GaleonEmbed *embed, int level)
 {
-	GtkWidget *item = gtk_menu_item_new_with_label (label);
+	GtkWidget *item = new_num_accel_menu_item (level, label, TRUE, menu);
+
 	gtk_widget_show (item);
 	gtk_object_set_user_data (GTK_OBJECT (item), embed);
 	gtk_menu_append (GTK_MENU (menu), item);
@@ -626,7 +838,7 @@ add_bf_menu (GtkWidget *menu, char* label, int index, GaleonEmbed *embed)
 GtkMenu *
 create_back_menu (GaleonEmbed *embed)
 {
-	int index, count, i;
+	int index, count, i, level;
 	char **titles;
 	GtkWidget *menu = gtk_menu_new ();
 
@@ -635,9 +847,9 @@ create_back_menu (GaleonEmbed *embed)
 		return NULL;
 	}
 
-	for (i = index - 1; i >= 0; i--) 
+	for (i = index - 1, level = 0; i >= 0; i--, level++) 
 	{
-		add_bf_menu (menu, titles[i], i, embed);
+		add_bf_menu (menu, titles[i], i, embed, level);
 	}
 	
 	free_string_array (titles, count);
@@ -650,7 +862,7 @@ create_back_menu (GaleonEmbed *embed)
 GtkMenu *
 create_forward_menu (GaleonEmbed *embed)
 {
-	int index, count, i;
+	int index, count, i, level;
 	char **titles;
 	GtkWidget *menu = gtk_menu_new ();
 
@@ -659,53 +871,84 @@ create_forward_menu (GaleonEmbed *embed)
 		return NULL;
 	}	
 
-	for (i = index + 1; i < count; i++)
+	for (i = index + 1, level = 0; i < count; i++, level++)
 	{
-		add_bf_menu (menu, titles[i], i, embed);
+		add_bf_menu (menu, titles[i], i, embed, level);
 	}
 	
 	free_string_array (titles, count);
 	return GTK_MENU(menu);
 }
 
+/** 
+ * Creates the multi-level up menu 
+ */
+GtkMenu *
+create_up_menu (GaleonEmbed *embed)
+{
+	GnomeVFSURI *uri, *up_uri;
+	GtkWidget *menu, *item;
+	gint level;
+
+	/* check embed location is valid */
+	if (embed->site_location == NULL || strlen (embed->site_location) == 0)
+	{
+		return NULL;
+	}
+
+	/* create a vfs entry for this level */
+	uri = gnome_vfs_uri_new (embed->site_location);
+	if (uri == NULL)
+	{
+		return NULL;
+	}
+
+	/* create the menu */
+	menu = gtk_menu_new ();
+
+	/* create each possible up entry */
+	for (level = 0;; level++)
+	{
+		up_uri = gnome_vfs_uri_get_parent (uri);
+		gnome_vfs_uri_unref (uri);
+		uri = up_uri;
+		
+		/* get out of loop if no parent */
+		if (uri == NULL)
+		{
+			break;
+		}
+
+		/* create the menu entry */
+		item = new_num_accel_menu_item (level, uri->text, TRUE, menu);
+		gtk_widget_show (GTK_WIDGET (item));
+		gtk_menu_append (GTK_MENU (menu), GTK_WIDGET (item));
+	}
+
+	/* the menu is completed */
+	return GTK_MENU (menu);
+}
+
 void
 menu_position_under_widget (GtkMenu *menu, gint *x, gint *y, 
 			    gpointer user_data)
 {
 	GtkWidget *w = GTK_WIDGET(user_data);
-	gint px, py, width, height, depth;
-	gdk_window_get_geometry(w->window, &px, &py, &width, &height, &depth);
-	gdk_window_get_deskrelative_origin(w->window, &px, &py);
-	*y = py + height;
-	*x = px;
-}
+	gint width, height;
+	gint screen_width, screen_height;
+	GtkRequisition requisition;
 
-/**
- * Returns a temp filename wich ends in a given suffix and does not exist
- */
-char *tmpnam_ex (char *suffix)
-{
-	gchar *s;
-	gchar *t = NULL;
-	gboolean found = FALSE;
-	int tries = 0;
-	while ((!found) && (tries++ < 100)) {
-		FILE *f;
-		s = g_malloc(L_tmpnam);
-		tmpnam (s);
-		if (t) g_free(t);
-		t = g_strconcat (s, suffix, NULL);
-		f = fopen (t, "r");
-		if (f == NULL) {
-			found = TRUE;
-		} else {
-			fclose (f);
-			g_free (t);
-			t = NULL;
-		}
-		g_free (s);
-	}
-	return t;
+	gdk_window_get_size (w->window, &width, &height);
+	gdk_window_get_deskrelative_origin(w->window, x, y);
+	*y = *y + height;
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+      
+	screen_width = gdk_screen_width ();
+	screen_height = gdk_screen_height ();
+	  
+	*x = CLAMP (*x, 0, MAX (0, screen_width - requisition.width));
+	*y = CLAMP (*y, 0, MAX (0, screen_height - requisition.height));
 }
 
 /**
@@ -737,6 +980,10 @@ galeon_close_all_windows (void)
 {
 	GList *wl, *next;
 
+	/* force ourselves out of server mode -- this means that closing
+	 * the last window really will quit out of galeon */
+	galeon_server_mode = FALSE;
+
 	/* close all windows: this in turn will close all embeds */
 	/* at the closing of the last window galeon_exit will be called */
 	for (wl = all_windows; wl != NULL;)
@@ -801,6 +1048,9 @@ void galeon_exit (void)
 	g_assert (g_list_length (all_embeds) == 0);
 	g_assert (g_list_length (all_windows) == 0);
 
+	/* absolutely no more mozembeds */
+	gtk_moz_embed_pop_startup ();
+
 	gtk_main_quit();
 }
 
@@ -889,6 +1139,12 @@ shorten_name(const gchar *input_name, gint target_length)
 			length -= 5;
 			memmove(name, name + 5, length + 1);
 		}
+		else if (strncasecmp (name, "www.", 4) == 0)
+		{
+			length -= 4;
+			memmove(name, name + 4, length + 1);
+		}
+		else 
 		{
 			/* prefixes that can be followed by other
 			 * prefixes */
@@ -1087,12 +1343,16 @@ read_line_from_file (FILE *f)
 	return line;
 }
 
-static gint 
-strcasestr_cb (gconstpointer a, gconstpointer b)
+/**
+ * g_strcasestr: test if a string b is a substring of string a, independent
+ * of case.
+ */
+const gchar *
+g_strcasestr (const gchar *a, const gchar *b)
 {
 	gchar *down_a;
 	gchar *down_b;
-	gint result; 
+	gchar *ptr;
 
 	/* copy and lower case the strings */
 	down_a = g_strdup (a);
@@ -1101,14 +1361,20 @@ strcasestr_cb (gconstpointer a, gconstpointer b)
 	g_strdown (down_b);
 
 	/* compare */
-	result = (strstr (down_a, down_b) == NULL ? 1 : 0);
+	ptr = strstr (down_a, down_b);
 
 	/* free allocated strings */
 	g_free (down_a);
 	g_free (down_b);
 	
 	/* return result of comparison */
-	return result;
+	return ptr == NULL ? NULL : (a + (ptr - down_a));
+}
+
+static gint 
+strcasestr_cb (gconstpointer a, gconstpointer b)
+{
+	return g_strcasestr (a, b) == NULL ? 1 : 0;
 }
 
 /*
@@ -1121,13 +1387,14 @@ create_charset_submenus (GtkMenuItem *encoding_menu_item, GList *charset_titles,
 	GtkWidget *encoding_menu, *tempw, *tempw2, *tempw3;
 	GList *templ, *templ2;
 	int i = 0;
+	int lgroups_count = get_lgroups_count ();
 	
  	encoding_menu = gtk_menu_new();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM(encoding_menu_item), encoding_menu);
 
 	templ = g_list_copy(charset_titles);
 	
-	for (i = 0;i < NUM_TRANSLATED_CHARSET_GROUPS;i++) {
+	for (i = 0;i < lgroups_count;i++) {
 		tempw = gtk_menu_item_new_with_label (_(lgroups[i]));
 		gtk_menu_append (GTK_MENU(encoding_menu), tempw);
 		gtk_widget_show (tempw);
@@ -1186,3 +1453,381 @@ gchar* get_mozilla_prefs_directory ()
 
 	return prefs_dir;
 }
+
+void
+launch_external_viewer (const gchar *filename)
+{
+	pid_t pid;
+	g_return_if_fail(filename != NULL);
+
+	/* Fork and invoke the external program. */
+	pid = fork ();
+	if (pid == 0) {
+		/* child process */
+		gchar *prog = gnome_config_get_string
+			(CONF_HANDLERS_EXTERNAL_SOURCE_VIEWER);
+		execlp (prog, prog, filename, NULL);
+		g_warning (_("Error executing external program"));
+		_exit (0);
+	} else if (pid < 0) {
+		/* error */
+		g_error ("Error creating subprocess");
+		return;
+	} else {
+		/* parent process */
+		/* fork again and give the external program time to start  
+		   before removing the temp file */
+		pid_t pid2 = fork ();
+		if (pid2 == 0) {
+				/* child */
+			sleep (45); /* 45 secs should be enough */
+			remove (filename);
+			_exit (0);
+		} else if (pid2 < 0) {
+			g_error ("Error creating subprocess");
+		}
+	}
+}
+
+gint file_copy_status (GnomeVFSXferProgressInfo *info,gpointer data)
+{
+        /* FIXME show overwrite dialog */
+        return TRUE;
+}
+
+gboolean
+copy_file (const char *src, const char *dest)
+{
+	gboolean result;
+	GnomeVFSURI *src_uri, *dest_uri;
+
+	src_uri = gnome_vfs_uri_new (src);
+	dest_uri = gnome_vfs_uri_new (dest);
+
+	if (!src_uri || !dest_uri) return FALSE;
+
+	result = gnome_vfs_xfer_uri(src_uri, dest_uri,
+				    GNOME_VFS_XFER_DEFAULT,
+				    GNOME_VFS_XFER_ERROR_MODE_QUERY, 
+				    GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
+				    (GnomeVFSXferProgressCallback)file_copy_status, 
+				    (gpointer)NULL) == GNOME_VFS_OK ? TRUE : FALSE;
+
+	gnome_vfs_uri_unref (src_uri);
+	gnome_vfs_uri_unref (dest_uri);
+
+	return result;
+}
+
+/**
+ * xmlSetPropISOLatin1: set a property in an XML file which has value
+ * encoded as ISOLatin1. This works around the strictness of libxml1.
+ */
+xmlAttrPtr
+xmlSetPropISOLatin1 (xmlNodePtr node, 
+		     const xmlChar *name, 
+		     const xmlChar *value)
+{
+	xmlChar buffer[4096];
+	gint count;
+
+	/* check for nullification */
+	if (value == NULL)
+	{
+		return xmlSetProp (node, name, NULL);
+	}
+
+	/* do the conversion */
+	count = isolat1ToUTF8 (buffer, 4096, value, strlen (value));
+	buffer[count] = '\0';
+	
+	/* store in the node */
+	return xmlSetProp (node, name, g_strdup (buffer));
+}
+
+/**
+ * xmlGetPropISOLatin1: get a property in an XML file which has value
+ * encoded as UTF8 but was originally ISOLatin1. This works around the
+ * strictness of libxml1.
+ */
+xmlChar *
+xmlGetPropISOLatin1 (xmlNodePtr node, const xmlChar *name)
+{
+	xmlChar *value;
+	xmlChar buffer[4096]; /* FIXME? */
+	gint count;
+
+	/* get the value */
+	value = xmlGetProp (node, name);
+	if (value == NULL)
+	{
+		return NULL;
+	}
+
+	/* do the conversion */
+	count = UTF8Toisolat1 (buffer, 4096, value, strlen (value));
+
+	/* check and do the best we can */
+	if (count >= 0)
+	{
+		/* terminate string */
+		buffer[count] = '\0';
+		xmlFree (value);
+		
+		/* copy and return */
+		return g_strdup (buffer);
+	}
+	else
+	{
+		/* return the undecoded result */
+		return value;
+	}
+}
+
+static void
+misc_ask_file_name_ok_cb (GtkButton *button, GtkWidget *dialog) 
+{
+	FileCallbackFunc f = gtk_object_get_data (GTK_OBJECT (dialog), "callback_ok");
+	GtkWidget *fentry = gtk_object_get_data (GTK_OBJECT (dialog), "fentry");
+	gchar *fname = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (fentry), FALSE);
+	gpointer data = gtk_object_get_data (GTK_OBJECT (dialog), "user_data");
+	gboolean must_exist = GPOINTER_TO_INT 
+		(gtk_object_get_data (GTK_OBJECT (dialog), "must_exist"));
+	gboolean warn_overwrite = GPOINTER_TO_INT 
+		(gtk_object_get_data (GTK_OBJECT (dialog), "warn_overwrite"));
+	gboolean proceed = TRUE;
+	if (must_exist && !g_file_exists (fname)) {
+		GtkWidget *err_dialog = gnome_error_dialog_parented 
+			(_("The file must exist.\n"
+			   "Please select an existing file or chose \"Cancel\""),
+			 GTK_WINDOW (dialog));
+		proceed = FALSE;
+	        window_set_layer (err_dialog);
+	        gnome_dialog_run (GNOME_DIALOG (err_dialog));
+	}
+	if (warn_overwrite && g_file_exists (fname)) {
+		proceed = misc_ask_question
+			(_("The file already exists.\n"
+			   "Do you want to overwrite it?"),
+			 GTK_WINDOW (dialog));
+	}
+	if (proceed) {
+		if (f)
+			f (fname, data);
+		gtk_widget_destroy (dialog);
+	}
+	g_free (fname);
+}
+
+static void
+misc_ask_file_name_cancel_cb (GtkButton *button, GtkWidget *dialog) 
+{
+	FileCallbackFunc f = gtk_object_get_data (GTK_OBJECT (dialog), 
+						  "callback_cancel");
+	GtkWidget *fentry = gtk_object_get_data (GTK_OBJECT (dialog), "fentry");
+	gchar *fname = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY (fentry), FALSE);
+	gpointer data = gtk_object_get_data (GTK_OBJECT (dialog), "user_data");
+	if (f)
+		f (fname, data);
+	g_free (fname);
+	gtk_widget_destroy (dialog);
+}
+
+/**
+ * Show a dialog to ask for a file. When the user selects ok/cancel the given 
+ * callbacks will be called.
+ * TODO: extend to handle file overwriting, inexistent files...
+ */
+void
+misc_ask_file_name (const gchar *title, const gchar *label_title, const gchar *def,
+		    FileCallbackFunc callback_ok, FileCallbackFunc callback_cancel, 
+		    gboolean must_exist, gboolean warn_overwrite,
+		    gpointer user_data)
+{
+	GladeXML *gxml = glade_xml_new (user_file ("galeon.glade", TRUE), 
+			"file_selection_dialog");
+	GtkWidget *ok_button = glade_xml_get_widget 
+		(gxml, "file_selection_dialog_ok_button");
+	GtkWidget *cancel_button = glade_xml_get_widget 
+		(gxml, "file_selection_dialog_cancel_button");
+	GtkWidget *fentry = glade_xml_get_widget 
+		(gxml, "file_selection_dialog_fileentry");
+	GtkWidget *label = glade_xml_get_widget 
+		(gxml, "file_selection_dialog_label");
+	GtkWidget *dialog = glade_xml_get_widget
+		(gxml, "file_selection_dialog");
+	gtk_object_set_data (GTK_OBJECT (dialog), "callback_ok", callback_ok);
+	gtk_object_set_data (GTK_OBJECT (dialog), "callback_cancel", callback_cancel);
+	gtk_object_set_data (GTK_OBJECT (dialog), "must_exist", 
+			     GINT_TO_POINTER (must_exist));
+	gtk_object_set_data (GTK_OBJECT (dialog), "warn_overwrite", 
+			     GINT_TO_POINTER (warn_overwrite));
+	gtk_object_set_data (GTK_OBJECT (dialog), "user_data", user_data);
+	gtk_object_set_data (GTK_OBJECT (dialog), "fentry", fentry);
+	gtk_window_set_title (GTK_WINDOW (dialog), title);
+	gtk_label_set_text (GTK_LABEL (label), label_title);
+	gnome_file_entry_set_title (GNOME_FILE_ENTRY (fentry), title);
+	gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry
+				       (GNOME_FILE_ENTRY (fentry))), def);
+	gtk_object_unref (GTK_OBJECT (gxml));
+	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", 
+			    misc_ask_file_name_ok_cb, dialog);
+	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked", 
+			    misc_ask_file_name_cancel_cb, dialog);
+}
+
+/** Shared var for the next two functions */
+static gboolean misc_ask_question_ret = FALSE;
+
+/**
+ * Asks a yes/no question and returns a boolean
+ */
+gboolean 
+misc_ask_question (const gchar *question, GtkWindow *parent)
+{
+	GtkWidget *dialog = gnome_question_dialog_parented
+		(question, misc_ask_question_cb, NULL, parent);
+	window_set_layer (dialog);
+	gnome_dialog_run (GNOME_DIALOG (dialog));
+	return misc_ask_question_ret;
+}
+
+static void
+misc_ask_question_cb (gint reply, gpointer data) {
+	misc_ask_question_ret = !reply;
+}
+
+char *messages [5] = {NULL, N_("Looking for a bookmarks icon"), NULL};
+
+/**
+ * get_siteicon: Parse an http URL ans get its favicon if there is one
+ */
+void
+get_siteicon (GaleonEmbed *embed, gchar *url)
+{
+	GnomeVFSURI *vfs_uri;
+	gchar *favicon_url, *favicon_path;
+	GList *source_uris = NULL, *target_uris = NULL;
+
+	g_return_if_fail (url != NULL);
+	
+	vfs_uri = gnome_vfs_uri_new (url);
+	g_return_if_fail (vfs_uri != NULL);
+	
+	if (embed == NULL || !mozilla_get_favicon_location (embed, &favicon_url))
+	{
+		GnomeVFSURI *vfs_favicon_uri;
+		
+		while (gnome_vfs_uri_has_parent (vfs_uri))
+		{
+			GnomeVFSURI *vfs_parent_uri;
+			vfs_parent_uri = gnome_vfs_uri_get_parent (vfs_uri);
+			gnome_vfs_uri_unref (vfs_uri);
+			vfs_uri = vfs_parent_uri;
+		} 
+		vfs_favicon_uri = gnome_vfs_uri_append_file_name (vfs_uri, 
+								 "favicon.ico");
+		favicon_url = gnome_vfs_uri_to_string (vfs_favicon_uri, 0);
+		gnome_vfs_uri_unref (vfs_favicon_uri);
+	}
+	else if (favicon_url == NULL)
+	{
+		gnome_vfs_uri_unref (vfs_uri);
+		return;
+	}
+
+	gnome_vfs_uri_unref (vfs_uri);
+
+	favicon_path = favicon_filename (favicon_url);
+
+	source_uris = g_list_append(source_uris, favicon_url);
+        target_uris = g_list_append(target_uris, favicon_path);
+
+        file_operations_copy_move (source_uris, target_uris,
+                                   FALSE,
+                                   messages, 
+				   favicon_download_completed_cb,
+				   favicon_path);
+	g_free (favicon_url);
+}
+
+void favicon_download_completed_cb (gpointer data)
+{
+	GdkPixbuf *test;
+	char *favicon_path = (char*)data;
+	
+	test = gdk_pixbuf_new_from_file (favicon_path);
+	if (test == NULL)
+	{
+		gnome_vfs_unlink (favicon_path);
+	}
+	else
+	{
+		gdk_pixbuf_unref (test);
+	}
+
+	bookmarks_save ();
+	g_free(data);
+}
+
+PixmapData *
+pixmap_data_new_from_file (const gchar *file)
+{
+	GdkPixbuf *pixbuf;
+	PixmapData *pixmap_data = g_new (PixmapData, 1);
+
+	pixbuf = gdk_pixbuf_new_from_file (file);
+	if (pixbuf == NULL)
+	{
+		g_warning ("%s does not exist or is not an image file\n", file);
+		return NULL;
+	}
+	
+	gdk_pixbuf_render_pixmap_and_mask (pixbuf,
+					   &(pixmap_data->pixmap),
+					   &(pixmap_data->mask),
+					   100);
+	gdk_pixbuf_unref (pixbuf);
+	
+	return pixmap_data; 
+}
+
+gchar *
+favicon_filename (gchar *favicon_url)
+{
+	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (favicon_url);
+	gchar *filename, *slashpos, *result;
+	
+	filename = gnome_vfs_uri_to_string (vfs_uri, 
+					    GNOME_VFS_URI_HIDE_USER_NAME |
+					    GNOME_VFS_URI_HIDE_PASSWORD |
+					    GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);	
+	while ((slashpos = strstr (filename, "/")) != NULL) *slashpos = '_';
+	
+	result = g_strconcat (g_get_home_dir (), "/.galeon/favicons/",
+			      filename, NULL);
+	g_free (filename);
+	
+	return result;
+}
+
+gchar *
+time_to_string (GTime t)
+{
+	if (t == 0) 
+		return g_strdup ("");
+	else {
+	        gchar *s = g_new0 (gchar, 250);
+		struct tm *stm = g_new0 (struct tm, 1);
+		int i;
+		gchar *ret;
+        
+		stm = localtime_r ( (time_t *) &t, stm);
+		i = strftime (s, 250, _("%-m/%-d/%Y"), stm);
+		ret = g_strndup (s, i);
+
+		g_free (s);
+		g_free (stm);
+		return ret;
+	}
+}
