@@ -22,14 +22,14 @@
 gchar *open_url_str = NULL;
 
 /* local function prototypes */
-void file_open_ok_button_clicked_cb (GtkButton *button, GaleonWindow *window);
+void file_open_ok_button_clicked_cb (const gchar *filename, gpointer data);
 void file_open_cancel_button_clicked_cb (GtkButton *button, GaleonWindow *window);
 void file_save_cancel_button_clicked_cb(GtkButton *button, GaleonWindow *window);
 void file_save_ok_button_clicked_cb(GtkButton *button, GaleonWindow *window);
 void download_dir_cancel_button_clicked_cb (GtkButton *button, GaleonWindow *window);
 void download_dir_ok_button_clicked_cb (GtkButton *button, GaleonWindow *window);
-void mime_ask_dialog_ok_clicked_cb (GtkButton *button, mime_ask_action_dialog *dialog);
-void mime_ask_dialog_cancel_clicked_cb (GtkButton *button, mime_ask_action_dialog *dialog);
+void mime_ask_dialog_ok_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog);
+void mime_ask_dialog_cancel_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog);
 
 /**
  * download_dir_cancel_button_clicked_cb: download directory dialog cancel clicked (menu)
@@ -90,41 +90,30 @@ open_url_ok_button_clicked_cb    (gchar *string, GaleonWindow *window)
  * file_open_ok_button_clicked_cb: file open dialog ok button clicked
  */
 void
-file_open_ok_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+file_open_ok_button_clicked_cb (const gchar *file, gpointer data)
 {
-	char * url;
-	GtkWidget *fs = glade_lookup_widget(GTK_WIDGET(button), "fsopen");
-
-	return_if_not_window (window);
-	url = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-	gtk_entry_set_text(GTK_ENTRY(window->toolbar_entry),url);
-                                        
-	window_load_url(window, url);
+	GaleonWindow *window = (GaleonWindow *) data;
+	window_load_url(window, file);
 	
-	gnome_config_set_string(CONF_DIR_OPEN,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+	gnome_config_set_string (CONF_DIR_OPEN, file);
 	gnome_config_sync();
-
-	gtk_widget_destroy(fs);
 }
 
 /**
- * file_open_cancel_button_clicked_cb: file open dialog cancel button clicked
+ * history_menu_menuitem_activate_cb: go to a selected item in the 
+ * back/forward history menu 
  */
 void
-file_open_cancel_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+history_menu_menuitem_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
-	GtkWidget *w = glade_lookup_widget(GTK_WIDGET(button), "fsopen");
+	GaleonEmbed *embed;
+	gint index;
 
-	return_if_not_window (window);
-	gtk_widget_destroy(w);
-}
-
-/** Go to a selected item in the back/forward history menu */
-void history_menu_menuitem_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
-{
-	int index = GPOINTER_TO_INT (user_data);
-	GaleonEmbed *embed = gtk_object_get_user_data (GTK_OBJECT (menuitem));
+	/* fetch data */
+	index = GPOINTER_TO_INT (user_data);
+	embed = gtk_object_get_user_data (GTK_OBJECT (menuitem));
 	return_if_not_embed (embed);
+
 	mozilla_session_history_go (embed, index);
 }
 
@@ -146,17 +135,35 @@ file_save_cancel_button_clicked_cb(GtkButton *button, GaleonWindow *window)
 void 
 file_save_ok_button_clicked_cb(GtkButton *button, GaleonWindow *window)
 {
-	char * fname;
-	gpointer target;
+	char *fname;
 	GtkWidget *fs = glade_lookup_widget (GTK_WIDGET(button), "fssave");
+	gboolean result;
+	gboolean save_main;
+	gchar *image;
 
 	return_if_not_window (window);
 	fname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
 
-	target = gtk_object_get_data (GTK_OBJECT(fs), "target");
+	save_main = (gboolean)gtk_object_get_data (GTK_OBJECT(fs), "main");
+	image = (char*)gtk_object_get_data (GTK_OBJECT(fs), "image");
 
-	if (!mozilla_save (window->active_embed, fname, target))
-		gnome_error_dialog(_("Cannot save the file"));
+	if (image)
+	{
+		g_free (image_save_dir);
+		image_save_dir = g_dirname (fname);
+
+		result = mozilla_save_image (window->active_embed, image, fname);
+	}
+	else if (save_main)
+	{
+		result = mozilla_save_main_document (window->active_embed, fname);
+	}
+	else
+	{
+		result = mozilla_save (window->active_embed, fname);
+	}
+
+	if (!result) gnome_error_dialog(_("Cannot save the file"));
 
 	gtk_widget_destroy(fs);
 }
@@ -164,7 +171,7 @@ file_save_ok_button_clicked_cb(GtkButton *button, GaleonWindow *window)
 /** MIME ask dialog callbacks */
 
 void
-mime_ask_dialog_ok_clicked_cb (GtkButton *button, mime_ask_action_dialog *dialog)
+mime_ask_dialog_ok_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 {
 	MimeAction action;
 	GtkWidget *leave_to_mozilla_check = glade_xml_get_widget 
@@ -181,29 +188,29 @@ mime_ask_dialog_ok_clicked_cb (GtkButton *button, mime_ask_action_dialog *dialog
 					       "mime_ask_action_dialog"));
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (leave_to_mozilla_check)))
-		action = LEAVE_TO_MOZILLA;
+		action = MIME_LEAVE_TO_MOZILLA;
 	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (save_check)))
-		action = SAVE_TO_DISK;
+		action = MIME_SAVE_TO_DISK;
 	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (run_program_check)))
-		action = RUN_PROGRAM;
+		action = MIME_RUN_PROGRAM;
 	else
-		action = UNKNOWN;
+		action = MIME_UNKNOWN;
 
 	switch (action) {
-	case LEAVE_TO_MOZILLA:
+	case MIME_LEAVE_TO_MOZILLA:
 		dialog->embed->mime_ignore_once = TRUE;
 		embed_load_url (dialog->embed, dialog->url);
 		break;
-	case SAVE_TO_DISK:
+	case MIME_SAVE_TO_DISK:
 		save_url (dialog->url);
 		break;
-	case RUN_PROGRAM:
+	case MIME_RUN_PROGRAM:
 		/* TODO */
 		gnome_error_dialog ("Not implmented, sorry :(");
-		action = UNKNOWN;
+		action = MIME_UNKNOWN;
 		break;
-	case ASK_ACTION:
-	case UNKNOWN:
+	case MIME_ASK_ACTION:
+	case MIME_UNKNOWN:
 		/* shouldn't get here */
 		g_warning ("Unexpected MIME action");
 		break;
@@ -222,7 +229,7 @@ mime_ask_dialog_ok_clicked_cb (GtkButton *button, mime_ask_action_dialog *dialog
 }
 
 void
-mime_ask_dialog_cancel_clicked_cb (GtkButton *button, mime_ask_action_dialog *dialog)
+mime_ask_dialog_cancel_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 {
 	g_free (dialog->mime_type);
 	g_free (dialog->url);
@@ -246,5 +253,5 @@ client_save_yourself_cb (GnomeClient *client, gint phase,
 	galeon_quit_with_session ();
 
 	/* we're done */
-	return FALSE;
+	return TRUE;
 }
