@@ -18,6 +18,18 @@
 
 #include "galeon.h"
 
+/* Flag to enable/disable mouse motion listener */
+static gboolean mouse_listener_active = FALSE;
+
+typedef struct
+{
+	gint original_x, original_y;
+	GaleonEmbed *embed;
+	WrapperMouseEventInfo *info;
+} MouseInfo;
+
+static gint mouse_listener (MouseInfo *minfo);
+
 /* type of drag and drop for links */
 const GtkTargetEntry link_drag_types[] =
 {
@@ -29,6 +41,7 @@ const gint link_drag_types_num_items = (sizeof (link_drag_types) /
 					sizeof (GtkTargetEntry));
 GtkTargetList *link_drag_types_tl = NULL;
 
+
 /**
  * mozembed_new_window_cb: GTKMOZEMBED SIGNAL, emitted any time a new 
  * window is requested by the document 
@@ -38,15 +51,19 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 			guint chromemask, GaleonEmbed *embed) 
 {
 	GaleonEmbed *new_embed;
+	GaleonWindow *window;
 	gboolean open_in_tab;
 
 	/* check callback args */
 	return_if_not_embed (embed);
+	window = embed->parent_window;
+	return_if_not_window (window);
 	g_assert (retval != NULL);
 
 	/* check popups aren't filtered */
-	if (gnome_config_get_int (CONF_GENERAL_NEWWINDOWS) != 0)
+	if (!(gnome_config_get_bool (CONF_GENERAL_ALLOW_POPUPS)))
 	{
+		window_update_temp_message (window, _("Popup blocked"));
 		*retval = embed->mozEmbed;
 		return;
 	}
@@ -61,11 +78,19 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	/* set the popup flag */
 	new_embed->is_popup = TRUE;
 
-	/* set chrome settings unless in a tab */
+	/* set chrome settings and turn off tab listing if this is actually
+	 * a new window instead of a new tab */
 	if (!open_in_tab)
 	{
 		gtk_moz_embed_set_chrome_mask (new_embed->mozEmbed,
 					       chromemask);
+
+		/* FIXME: this is messy, since the tab display is getting
+		 * turned on in embed_create_hidden.  no flicker visible
+		 * here, though. */
+		gtk_notebook_set_show_tabs 
+			(GTK_NOTEBOOK (new_embed->parent_window->notebook),
+			 FALSE);
 	}
 
 	/* return the new browser to gtkmozembed */
@@ -137,6 +162,13 @@ mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	/* autosave the updated session */
 	session_autosave ();
 	
+	/* get the new location */
+	if (embed->site_location != NULL)
+	{
+		g_free (embed->site_location);
+	}
+	embed->site_location = gtk_moz_embed_get_location (embed->mozEmbed);
+
 	/* update in gui */
 	embed_update_page_location (embed);
 }
@@ -150,6 +182,13 @@ mozembed_title_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	return_if_not_embed (embed);
 
+	/* get the new title */
+	if (embed->site_title != NULL)
+	{
+		g_free (embed->site_title);
+	}
+	embed->site_title = gtk_moz_embed_get_title (embed->mozEmbed);
+
 	/* update in gui */
 	embed_update_page_title (embed);
 }
@@ -170,12 +209,13 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 
 	/* if we haven't already, start the window spinner, set the
 	 * load_started tag, and clear the info about the progress */
-	if (!(embed->load_started))
+	if (embed->load_started == 0)
 	{
 		spinner_start (window);
-		embed->load_started = TRUE;
+		embed_set_notebook_label_status (embed, TAB_LOADING);
 		embed_progress_clear (embed);
 	}
+	embed->load_started++;
 
 	/* update the buttons view */
 	if (embed == window->active_embed)
@@ -193,8 +233,7 @@ void
 mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	GaleonWindow *window;
-	gchar *title, *url;
-	history_item *hi;
+	HistoryItem *hi;
 
 	/* check callback args */
 	return_if_not_embed (embed);
@@ -204,9 +243,8 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	/* add the url to the history */
 	if (embed->bytesLoaded)
 	{
-		title = gtk_moz_embed_get_title (embed->mozEmbed);
-		url = gtk_moz_embed_get_location (embed->mozEmbed);
-		hi = history_visited (url, title);
+		hi = history_visited (embed->site_location,
+				      embed->site_title);
 
 		/* add url to recent menu */
 		if (hi != NULL)
@@ -215,16 +253,17 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		}
 	}
 
+	/* set in the embed */
+	embed->load_started--;
+	embed_progress_clear (embed);
+
 	/* stop the window spinner */
-	if (embed->load_started)
+	if (embed->load_started == 0)
 	{
 		spinner_stop (window);
+		embed_set_notebook_label_status (embed, TAB_NORMAL);
 	}
 
-	/* set in the embed */
-	embed->load_started = FALSE;
-	embed_progress_clear (embed);
-
 	/* update the buttons view */
 	if (embed == window->active_embed)
 	{
@@ -245,7 +284,13 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 	return_if_not_embed (embed);
 	window = embed->parent_window;
 	return_if_not_window (window);
-	
+
+	/* clear temporary message, if any */
+	if (embed == window->active_embed)
+	{
+		window_update_temp_message (window, NULL);
+	}
+
 	if (flags & GTK_MOZ_EMBED_FLAG_IS_REQUEST)
 	{
 		if (flags & GTK_MOZ_EMBED_FLAG_REDIRECTING)
@@ -280,12 +325,10 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 		if (flags & GTK_MOZ_EMBED_FLAG_START)
 		{
 			embed->statusMessage = _("Loading site...");
-			embed_set_notebook_label_status (embed, LOADING);
 		}
 		else if (flags & GTK_MOZ_EMBED_FLAG_STOP)
 		{
 			embed->statusMessage = _("Done.");
-			embed_set_notebook_label_status (embed, NORMAL);
 		}
 	}
 
@@ -305,11 +348,10 @@ void mozembed_progress_change_cb (GtkMozEmbed *dummy, gint cur, gint max,
 	return_if_not_embed (embed);
 	window = embed->parent_window;
 	return_if_not_window (window);
-	
+
 	/* compute percentages */
 	if (max == -1)
 	{
-		/* length of the document cannot be determined */
 		embed->loadPercent = 0;
 		embed->bytesLoaded = cur;
 		embed->maxBytesLoaded = 0;
@@ -317,7 +359,7 @@ void mozembed_progress_change_cb (GtkMozEmbed *dummy, gint cur, gint max,
 	else if (cur > max)
 	{
 		/* sometimes length of the downloaded document is 
-		 * greater than  the length of the document */
+		 * greater than the length of the document */
 		embed->loadPercent = 100;
 		embed->bytesLoaded = cur;
 		embed->maxBytesLoaded = max;
@@ -355,9 +397,9 @@ mozembed_link_message_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	message = gtk_moz_embed_get_link_message (embed->mozEmbed);
 
 	/* update browser message */
+	window_update_temp_message (window, message);
 	if (message)
 	{
-		window_update_temp_message (window, message);
 		g_free (message);
 	}
 }
@@ -398,7 +440,7 @@ gint
 mozembed_open_uri_cb (GtkMozEmbed *dummy, const char *uri, GaleonEmbed *embed)
 {
 	return_val_if_not_embed (embed, TRUE);
-	
+
 	/* handle foreign protocols, things like "mailto:" */
 	if (handle_foreign_protocols(uri))
 		return TRUE;
@@ -411,6 +453,65 @@ mozembed_open_uri_cb (GtkMozEmbed *dummy, const char *uri, GaleonEmbed *embed)
 	return FALSE;
 }
 
+/**
+ * mozembed_dom_mouse_down_cb: emitted on a button press event
+ */
+gint
+mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event, 
+			    GaleonEmbed *embed)
+{
+	WrapperMouseEventInfo *info;
+	gint button_action = gnome_config_get_int
+				("/galeon/Mouse/right_button_action=0");
+
+	return_val_if_not_embed (embed, FALSE);
+
+	if (button_action == 0)
+		return FALSE;
+
+	info = g_new0 (WrapperMouseEventInfo, 1);
+	if (!mozilla_get_mouse_event_info (embed, dom_event, info)) 
+	{
+		g_free(info);
+		return FALSE;
+	}
+
+#if MOZILLA_MILESTONE == 81
+	info->button ++;
+#endif
+
+	if (info->button == 3)
+	{
+		if (button_action == 2)
+		{
+			MouseInfo *minfo;
+			GdkWindow *win = embed->parent_window->WMain->window;
+
+			if (mouse_listener_active)
+				return FALSE;
+
+			minfo = g_new0 (MouseInfo, 1);
+			gdk_window_get_pointer (win, &minfo->original_x,
+						&minfo->original_y, NULL);
+			minfo->embed = embed;
+			minfo->info = info;
+
+			/* start the mouse listener timeout */
+			gtk_timeout_add (10, (GtkFunction) mouse_listener,
+					 minfo);
+			mouse_listener_active = TRUE;
+
+			return FALSE;
+		}
+		else
+			context_show_menu (embed, &info->ctx);
+	}
+
+	g_free (info);
+
+	return FALSE;
+}
+
 /**
  * mozembed_dom_mouse_click_cb: GTKMOZEMBED SIGNAL, emitted when user 
  * clicks on the document
@@ -419,62 +520,65 @@ gint
 mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event, 
 			     GaleonEmbed *embed)
 {
-	int type, modifier;
-	gchar *img, *href;
-	gpointer target;
 	GaleonWindow *window;
 	gboolean tabbed_mode;
+	WrapperMouseEventInfo *info;
+	gboolean handled = FALSE;
 
 	return_val_if_not_embed (embed, FALSE);
 	window = embed->parent_window;
 	return_val_if_not_window (window, FALSE);
 
-	/* Do not process the event if it occurred on a scrollbar */
-	if (mozilla_scrollbar_clicked(embed, dom_event))
-		return FALSE;
-
 	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-	type = mozilla_get_event_context (embed, dom_event,
-					  &img, &target, &href);
 
-	switch (mozilla_get_mouse_button(dom_event))
+	info = g_new0 (WrapperMouseEventInfo,1);
+	if (!mozilla_get_mouse_event_info (embed, dom_event, info)) 
+	{
+		g_free(info);
+		return FALSE;
+	}
+
+#if MOZILLA_MILESTONE == 81
+	info->button ++;
+#endif
+
+	/* make a sound if a link was clicked */
+	if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link && info->button != 3)
+	{
+		gnome_triggers_do ("", "program", "galeon", "url_clicked",
+				   NULL);
+	}
+
+	switch (info->button)
 	{
 	case 3:
-		if ((type & CONTEXT_IMAGE) && (type & CONTEXT_LINK))
+		if (gnome_config_get_int("/galeon/Mouse/right_button_action")
+		    == 2)
 		{
-			context_show_link_img_menu(embed, img, href);
-		}	
-		else if (type & CONTEXT_IMAGE)
-		{
-			context_show_img_menu(embed, img);
-		}		
-		else if (type & CONTEXT_LINK)
-		{
-			context_show_link_menu(embed, href);
-		}		
-		else if (type & CONTEXT_DOCUMENT)
-		{
-			context_show_doc_menu (embed, target);
-		}	
-		else if (type  & CONTEXT_OTHER)
-		{
-			g_print("clicked other components");
-		}	
-		return TRUE;
-		break;
+			/* kill the motion listener, if it exists */
+			if (mouse_listener_active)
+			{
+				mouse_listener_active = FALSE;
+			}
+			gtk_moz_embed_go_back (embed->mozEmbed);
+		}
+		else
+			context_show_menu (embed, &info->ctx);
 
+		handled = TRUE;
+		break;
 	case 2:
-		if ((type & CONTEXT_LINK) && href)
+		if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link)
 		{
-			mozilla_get_mod_key(embed, dom_event, &modifier);
-			if ((modifier & SHIFT_KEY) &&
-			    !((modifier & ALT_KEY) || (modifier & CTRL_KEY)))
-				embed_create_from_url (embed, href,
+			if ((info->modifier & SHIFT_KEY) &&
+			    !((info->modifier & ALT_KEY) || 
+			    (info->modifier & CTRL_KEY)))
+				embed_create_from_url (embed, info->ctx.link,
 						       tabbed_mode);
 			else
-				embed_create_from_url (embed, href,
+				embed_create_from_url (embed, info->ctx.link,
 						       !tabbed_mode);
-			return TRUE;
+			handled = TRUE;
 		} 
 		else 
 		{
@@ -492,17 +596,26 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		break;
 
 	case 1: 
-		if ((type & CONTEXT_LINK) && href)
+		if ((info->ctx.context & CONTEXT_IMAGE) &&
+		    (info->modifier & CTRL_KEY))
 		{
-			return handle_foreign_protocols (href);
+			embed_save_image_default (embed, info->ctx.img);
+			break;
+		}	
+#if MOZILLA_MILESTONE != 81
+		else if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link)
+		{
+			handled = handle_foreign_protocols (info->ctx.link);
 		}
+#endif
 		break;
-
 	default:
 		break;
 	}
 
-	return FALSE;
+	g_free (info);
+
+	return handled;
 }
 
 /**
@@ -513,71 +626,98 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 				GaleonEmbed *embed)
 {
 	GaleonWindow *window;
-	int modifier, state = 0;
-	long key;
-	gboolean handled;
+	int state = 0;
+	gboolean handled = FALSE;
+	WrapperKeyEventInfo *info;
 
 	return_val_if_not_embed (embed, FALSE);
 	window = embed->parent_window;
 	return_val_if_not_window (window, FALSE);
 
-	key = mozilla_get_key_pressed(embed, dom_event, &modifier);
+	info = g_new0 (WrapperKeyEventInfo,1);
+	if (!mozilla_get_key_event_info(embed, dom_event, info))
+	{
+		g_free(info);
+		return FALSE;
+	}
 
-	/* g_print("key =%ld\n",key); 
-	   g_print("modifier = %d\n", modifier); */
+	/* g_print("key = %ld  modifier = %d\n",info->key, info->modifier); */
 
 	/* Keypresses that are not handled here are now passed on to the main
 	   window's acceltable, so you should *not* handle any functions here
 	   that have an accelerator key in the menubar. --Josh */
 
-	if (modifier == 0)
+	if (info->modifier == 0)
 	{
-		switch(key)
+		switch(info->key)
 		{
 		default:
 			break;
 		}
 	}
-	else if ((modifier & ALT_KEY) && (modifier & CTRL_KEY))
+	else if ((info->modifier & ALT_KEY) && (info->modifier & CTRL_KEY))
 	{
-		switch(key)
+		switch(info->key)
 		{
 		default:
 			break;
 		}
 	}
-	else if ((modifier & KEY_CODE) && ((modifier & CTRL_KEY) ||
-					   (modifier & ALT_KEY)))
+	else if ((info->modifier & KEY_CODE) && (info->modifier & SHIFT_KEY))
 	{
-		switch(key)
+		switch(info->key)
 		{
+		case DOM_VK_F10:
+			context_show_menu (embed, &info->ctx);
+			handled = TRUE;
+			break;
+		default:
+			break;
+		}
+	}
+	else if ((info->modifier & KEY_CODE) && ((info->modifier & CTRL_KEY) ||
+						(info->modifier & ALT_KEY)))
+	{
+		switch(info->key)
+		{
+/* FIXME: perhaps we should get rid of these bindings, they conflict
+ * with alot of peoples window manager bindings. I'd suggest Ctrl-[ and 
+ * Ctrl-], but I'd like to know what other people think. Changing them
+ * would make it easier to rebind them under GTK+ and remove some code
+ * -- MattA 24/3/2001 */
+/* Nope, bad idea. These are pretty standard accelerators, and anyone
+ * used to Netscape will expect them to work.  GTK+ doesn't allow cursor keys
+ * to be used as accelerators, so these have to be handled here.
+ * --Josh */
 		case DOM_VK_KP_LEFT:
 		case DOM_VK_LEFT:
 			gtk_moz_embed_go_back(embed->mozEmbed);
-			return TRUE;
+			handled = TRUE;
 			break;
 		case DOM_VK_KP_RIGHT:
 		case DOM_VK_RIGHT:
 			gtk_moz_embed_go_forward(embed->mozEmbed);
-			return TRUE;
+			handled = TRUE;
 			break;
+#if 0 /* FIXME, remove unless people complain */
 		case DOM_VK_UP:
 			gtk_notebook_prev_page 
 				(GTK_NOTEBOOK (window->notebook));
-			return TRUE;
+			handled = TRUE;
 			break;
 		case DOM_VK_DOWN:
 			gtk_notebook_next_page 
 				(GTK_NOTEBOOK (window->notebook));
-			return TRUE;
+			handled = TRUE;
 			break;
+#endif
 		default:
 			break;
 		}
 	}
-	else if ((modifier & ALT_KEY) && !(modifier & (CTRL_KEY | SHIFT_KEY)))
+	else if ((info->modifier & ALT_KEY) && !(info->modifier & (CTRL_KEY | SHIFT_KEY)))
 	{
-		switch(key)
+		switch(info->key)
 		{
 		case DOM_VK_1:
 		case DOM_VK_2:
@@ -589,26 +729,26 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		case DOM_VK_8:
 		case DOM_VK_9:
 			gtk_notebook_set_page (GTK_NOTEBOOK (window->notebook),
-					       key - DOM_VK_1);
-			return TRUE;
+					       info->key - DOM_VK_1);
+			handled = TRUE;
 			break;
 		case DOM_VK_0:
 			gtk_notebook_set_page (GTK_NOTEBOOK (window->notebook),
 					       9);
-			return TRUE;
+			handled = TRUE;
 			break;
 		case DOM_VK_EQUALS:
 			gtk_spin_button_spin (GTK_SPIN_BUTTON (window->zoom_spin), GTK_SPIN_STEP_FORWARD, 10);
-			return TRUE;
+			handled = TRUE;
 			break;
 		default:
 			break;
 
 		}
 	}
-	else if (modifier & CTRL_KEY)
+	else if (info->modifier & CTRL_KEY)
 	{		
-		switch(key)
+		switch(info->key)
 		{
 		case DOM_VK_g:
 		case DOM_VK_G:
@@ -616,13 +756,13 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 				(GTK_EDITABLE(window->toolbar_entry), 0, -1);
 			gtk_window_set_focus (GTK_WINDOW(window->WMain),
 					      window->toolbar_entry);
-			return TRUE;
-			break;   
+			handled = TRUE;
+			break;
 		case DOM_VK_EQUALS:
 			gtk_spin_button_spin
 					(GTK_SPIN_BUTTON(window->zoom_spin),
 					 GTK_SPIN_STEP_FORWARD, 10);
-			return TRUE;
+			handled = TRUE;
 			break;
 		default:
 			break;
@@ -630,88 +770,91 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		}
 	}
 
-	if (modifier & KEY_CODE)
+	if (info->modifier & KEY_CODE)
 	{
 		/* The extended keys have different keycodes in Mozilla/GDK,
 		   so we need to translate them here */
-		switch(key)
+		/* FIXME cleanup */
+		switch(info->key)
 		{
 		case DOM_VK_PAGE_UP:
-			key = GDK_Page_Up;
+			info->key = GDK_Page_Up;
 			break;
 		case DOM_VK_PAGE_DOWN:
-			key = GDK_Page_Down;
+			info->key = GDK_Page_Down;
 			break;
 		case DOM_VK_HOME:
-			key = GDK_Home;
+			info->key = GDK_Home;
 			break;
 		case DOM_VK_END:
-			key = GDK_End;
+			info->key = GDK_End;
 			break;
 		case DOM_VK_INSERT:
-			key = GDK_Insert;
+			info->key = GDK_Insert;
 			break;
 		case DOM_VK_ESCAPE:
-			key = GDK_Escape;
+			info->key = GDK_Escape;
 			break;
 		case DOM_VK_F12:
-			key = GDK_F12;
+			info->key = GDK_F12;
 			break;
 		case DOM_VK_F11:
-			key = GDK_F11;
+			info->key = GDK_F11;
 			break;
 		case DOM_VK_F10:
-			key = GDK_F10;
+			info->key = GDK_F10;
 			break;
 		case DOM_VK_F9:
-			key = GDK_F9;
+			info->key = GDK_F9;
 			break;
 		case DOM_VK_F8:
-			key = GDK_F8;
+			info->key = GDK_F8;
 			break;
 		case DOM_VK_F7:
-			key = GDK_F7;
+			info->key = GDK_F7;
 			break;
 		case DOM_VK_F6:
-			key = GDK_F6;
+			info->key = GDK_F6;
 			break;
 		case DOM_VK_F5:
-			key = GDK_F5;
+			info->key = GDK_F5;
 			break;
 		case DOM_VK_F4:
-			key = GDK_F4;
+			info->key = GDK_F4;
 			break;
 		case DOM_VK_F3:
-			key = GDK_F3;
+			info->key = GDK_F3;
 			break;
 		case DOM_VK_F2:
-			key = GDK_F2;
+			info->key = GDK_F2;
 			break;
 		case DOM_VK_F1:
-			key = GDK_F1;
+			info->key = GDK_F1;
 			break;
 		default:
 			break;
 		}
 	}
 
-	if (modifier & CTRL_KEY)
+	if (info->modifier & CTRL_KEY)
 		state |= GDK_CONTROL_MASK;
-	if (modifier & SHIFT_KEY)
+	if (info->modifier & SHIFT_KEY)
 		state |= GDK_SHIFT_MASK;
-	if (modifier & ALT_KEY)
+	if (info->modifier & ALT_KEY)
 		state |= GDK_MOD1_MASK;
 
 	/* We haven't specifically handled the keypress, so send it to the
 	   main window in case it is a menu accelerator key */
-	handled = gtk_accel_groups_activate (GTK_OBJECT (window->WMain),
-					     key, state);
+	if (!handled)
+	{
+		handled = gtk_accel_groups_activate (GTK_OBJECT (window->WMain),
+						     info->key, state);
+	}
+
+	g_free (info);
 
 	/* If it wasn't an accelerator, let Mozilla handle it */
-	if (!handled)
-		return FALSE;
-
-	return TRUE;
+	return handled;
 }
 
 /**
@@ -728,12 +871,11 @@ mozembed_size_to_cb (GtkMozEmbed *dummy, gint width, gint height,
 	window = embed->parent_window;
 	return_if_not_window (window);
 
-	/* hmm, FIXME..? */
-	if (embed->is_active)
-	{
-		gtk_window_set_default_size (GTK_WINDOW (window->WMain), 
-					     width, height);
-	}
+	/* resize the embed */
+	gtk_widget_set_usize (GTK_WIDGET (embed->mozEmbed), width, height);
+
+	/* don't override this by changing the main window size! */
+	window->set_size = TRUE;
 }
 
 /**
@@ -750,13 +892,25 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 	window = embed->parent_window;
 	return_if_not_window (window);
 
-	/* not the active embed anymore */
+	/* no longer the active embed */
 	if (embed->is_active)
 	{
 		embed->is_active = FALSE;
 		embed->parent_window->active_embed = NULL;
 	}
 
+	/* free memory */
+	if (embed->site_location != NULL)
+	{
+		g_free (embed->site_location);
+		embed->site_location = NULL;
+	}
+	if (embed->site_title != NULL)
+	{
+		g_free (embed->site_title);
+		embed->site_title = NULL;
+	}
+
 	/* destroy C++ wrapper */
 	if (embed->wrapper)
 	{
@@ -769,10 +923,11 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 	/* from from list of embeds in parent */
 	window->embed_list = g_list_remove (window->embed_list, embed);
 
-	/* show tabs if more than one embed */
+	/* show tabs if more than one embed or if the user has requested it */
 	n_embeds = g_list_length (window->embed_list);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->notebook), 
-				    n_embeds > 1);
+		(g_list_length (window->embed_list) > 1) |
+		gnome_config_get_bool (CONF_APPEARANCE_TABBED_ALWAYS_SHOW));
 
 	/* if that's the last embed in parent window, destroy it */
 	if (n_embeds == 0)
@@ -799,3 +954,31 @@ mozembed_drag_drop_cb (GtkWidget * widget, GdkDragContext *context,
 	return FALSE;
 }
 
+/**
+ * mouse_listener: Internal function. Used to poll the mouse position to
+ *                 determine if motion has occurred.
+ **/
+static gint
+mouse_listener (MouseInfo *minfo)
+{
+	GdkWindow *window = minfo->embed->parent_window->WMain->window;
+	gint x, y;
+
+	if (!mouse_listener_active)
+	{
+		g_free (minfo->info);
+		g_free (minfo);
+
+		return FALSE;
+	}
+
+	gdk_window_get_pointer (window, &x, &y, NULL);
+
+	if (ABS(x - minfo->original_x) > 1 || ABS(y - minfo->original_y) > 1)
+	{
+		context_show_menu (minfo->embed, &minfo->info->ctx);
+		mouse_listener_active = FALSE;
+	}
+
+	return TRUE;
+}
