@@ -17,6 +17,7 @@
  */
 
 #include "galeon.h"
+#include <locale.h>
 
 static char *autodetect_charset_prefs[] =
 {
@@ -45,6 +46,9 @@ void mozilla_prefs_set_network (void);
 void mozilla_prefs_set_wheel_mouse (void);
 char * get_bracketed_string (char *str);
 
+extern const gchar *lang_encode_name[LANG_ENC_NUM];
+extern const gchar *lang_encode_item[LANG_ENC_NUM];
+
 /**
  * mozilla_prefs_set: set mozilla preferences
  */
@@ -99,6 +103,14 @@ void mozilla_prefs_set_wheel_mouse (void)
 				    gnome_config_get_int ("/galeon/Advanced/wheel_ctrl_step=1"));
 	mozilla_preference_set_int ("mousewheel.withshiftkey.numlines",
 				    gnome_config_get_int ("/galeon/Advanced/wheel_shift_step=1"));
+
+	/* set default search engine */
+	mozilla_preference_set ("keyword.URL",
+				"http://www.google.com/search?q=");
+
+	/* turn off warning since we cant show them properly yet (FIXME) */
+	mozilla_preference_set_boolean ("security.warn_entering_secure",
+					FALSE);
 }
 
 /**
@@ -128,7 +140,9 @@ mozilla_prefs_set_network (void)
 	switch (gnome_config_get_int ("/galeon/Advanced/http_version=1"))
 	{
 		case 0: http_version = g_strdup ("1.0");
+			break;
 		case 1: http_version = g_strdup ("1.1");
+			break;
 	}
       	
 	mozilla_preference_set_boolean ("security.checkloaduri", FALSE); 
@@ -177,33 +191,73 @@ mozilla_prefs_set_animate (void)
 void 
 mozilla_prefs_set_fonts (void)
 {
+	char *font_name;
+	const gchar *default_font_name = "adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1";
+	const gchar *default_font_size = "14";
+	const gchar *default_font_type = "0";
+	const gchar *pref_key_prefix[5] =
+	{
+		"/galeon/Appearance/serif_font",
+		"/galeon/Appearance/sansserif_font",
+		"/galeon/Appearance/monospace_font",
+		"/galeon/Appearance/size_font",
+		"/galeon/Appearance/default_font"
+	};
+
 	gchar *serif_font, *sansserif_font, *monospace_font, *default_font[2] = {"sans-serif","serif"};
-	gchar **font;
-	int size_font,default_font_idx;
+	gchar **font, buf[128];
+	int size_font,default_font_idx,i;
 
-	serif_font = gnome_config_get_string ("/galeon/Appearance/serif_font=-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1");
-	sansserif_font = gnome_config_get_string ("/galeon/Appearance/sansserif_font=-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1");
-	monospace_font = gnome_config_get_string ("/galeon/Appearance/monospace_font=-adobe-times-medium-r-normal-*-14-*-*-*-p-*-iso8859-1");
-	size_font = gnome_config_get_int ("/galeon/Appearance/size_font=14");
-	default_font_idx = gnome_config_get_int("/galeon/Appearance/default_font=0");
+	for ( i = 0; i < LANG_ENC_NUM; i++) {
+	   g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[0],
+		       lang_encode_item[i], default_font_name);
+	   serif_font = gnome_config_get_string (buf);
 
-	font = g_strsplit (serif_font,"-",-1);
-	mozilla_preference_set ("font.name.serif.x-western",g_strconcat(font[1],"-",font[2],"-",font[13],
-								       "-",font[14],NULL));
-	g_strfreev(font);
+	   g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[1],
+		       lang_encode_item[i], default_font_name);
+	   sansserif_font = gnome_config_get_string (buf);
 
-	font = g_strsplit (sansserif_font,"-",-1);
-	mozilla_preference_set ("font.name.sans-serif.x-western",g_strconcat(font[1],"-",font[2],"-",font[13],
-									    "-",font[14],NULL));
-	g_strfreev(font);
+	   g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[2],
+		       lang_encode_item[i], default_font_name);
+	   monospace_font = gnome_config_get_string (buf);
 
-	font = g_strsplit (monospace_font,"-",-1);
-	mozilla_preference_set ("font.name.monospace.x-western",g_strconcat(font[1],"-",font[2],"-",font[13],
-									   "-",font[14],NULL));
-	mozilla_preference_set_int ("font.size.fixed.x-western",atoi(font[7]));
-	g_strfreev(font);
+	   g_snprintf (buf, 128, "%s_%s=%s", pref_key_prefix[3],
+		       lang_encode_item[i], default_font_size);
+	   size_font = gnome_config_get_int (buf);
 
-	mozilla_preference_set_int ("font.size.variable.x-western",size_font);
+	   g_snprintf (buf, 128, "%s=%s", pref_key_prefix[4], default_font_type);
+	   default_font_idx = gnome_config_get_int(buf);
+
+
+	   font = g_strsplit (serif_font,"-",-1);
+	   font_name = g_strconcat(font[1],"-",font[2],"-",
+				   font[13],"-",font[14], NULL); 
+	   g_snprintf (buf, 128, "font.name.serif.%s", lang_encode_item[i]);
+	   mozilla_preference_set (buf, font_name);
+	   g_strfreev(font);
+	   g_free (font_name);
+
+	   font = g_strsplit (sansserif_font,"-",-1);
+	   font_name = g_strconcat(font[1],"-",font[2],"-",font[13],
+				   "-",font[14], NULL);
+	   g_snprintf (buf, 128, "font.name.sans-serif.%s", lang_encode_item[i]);
+	   mozilla_preference_set (buf, font_name);
+	   g_strfreev(font);
+	   g_free (font_name);
+
+	   font = g_strsplit (monospace_font,"-",-1);
+	   font_name = g_strconcat(font[1],"-",font[2],"-",font[13],
+				   "-",font[14], NULL);
+	   g_snprintf (buf, 128, "font.name.monospace.%s", lang_encode_item[i]);
+	   mozilla_preference_set (buf, font_name);
+	   g_snprintf (buf, 128, "font.size.fixed.%s", lang_encode_item[i]);
+	   mozilla_preference_set_int (buf,atoi(font[7]));
+	   g_strfreev(font);
+	   g_free (font_name);
+
+	   g_snprintf (buf, 128, "font.size.variable.%s", lang_encode_item[i]);
+	   mozilla_preference_set_int (buf,size_font);
+	}
 
 	mozilla_preference_set ("font.default",default_font[default_font_idx]);
 }
@@ -216,17 +270,18 @@ mozilla_prefs_set_colors (void)
 {
 	gchar *background_color, *text_color, *visited_link_color;
 	gchar *unvisited_link_color;
-	gboolean use_provided_colors, underline_links;
+	gboolean use_document_colors, use_document_fonts;
+	gboolean underline_links;
 	gboolean enable_syntax_highlighting;
-	gint use_provided_fonts;
 
 	background_color = gnome_config_get_string ("/galeon/Appearance/background_color=#FFFFFF");
 	text_color = gnome_config_get_string ("/galeon/Appearance/text_color=#000000");
 	visited_link_color = gnome_config_get_string ("/galeon/Appearance/visited_link_color=#FF0000");
 	unvisited_link_color = gnome_config_get_string ("/galeon/Appearance/unvisited_link_color=#0000FF");
-	use_provided_colors = (gnome_config_get_int
-			       (CONF_APPEARANCE_USE_PROVIDED_COLORS) == 0) ? TRUE : FALSE;
-	use_provided_fonts = gnome_config_get_int (CONF_APPEARANCE_USE_PROVIDED_FONTS);
+	use_document_colors = 
+		!(gnome_config_get_bool (CONF_APPEARANCE_USE_OWN_COLORS));
+	use_document_fonts = 
+		!(gnome_config_get_bool (CONF_APPEARANCE_USE_OWN_FONTS));
 	underline_links = gnome_config_get_bool ("/galeon/Appearance/underline_links=TRUE");
 	enable_syntax_highlighting = gnome_config_get_bool
 					(CONF_HANDLER_VIEW_SOURCE_COLORS);
@@ -236,13 +291,17 @@ mozilla_prefs_set_colors (void)
 	mozilla_preference_set ("browser.visited_color", visited_link_color);
 	mozilla_preference_set_boolean ("browser.underline_anchors", underline_links);
 	mozilla_preference_set_boolean ("browser.display.use_document_colors",
-					use_provided_colors);
+					use_document_colors);
 	mozilla_preference_set_int ("browser.display.use_document_fonts", 
-				    use_provided_fonts);
+				    use_document_fonts);
 
 	/* syntax highlighting for view source mode */
 	mozilla_preference_set_boolean ("browser.view_source.syntax_highlight",
 					enable_syntax_highlighting);
+
+	mozilla_preference_set_boolean ("browser.display.use_system_colors",
+					FALSE);
+
 	g_free (background_color);
 	g_free (text_color);
 	g_free (visited_link_color);
@@ -340,11 +399,16 @@ mozilla_prefs_set_cache (void)
 
 	if (disk_cache_size > 0)
 	{
-		gchar *cachedir = init_cache_dir();
-		mozilla_preference_set("browser.cache.directory", cachedir);
+#ifndef CVS_SOURCE
+		gchar *cachedir;
+#endif
 		mozilla_preference_set_boolean("browser.enable.cache", TRUE);
 		mozilla_preference_set_int ("browser.cache.disk_cache_size", disk_cache_size);
+#ifndef CVS_SOURCE
+		cachedir = init_cache_dir();
+		mozilla_preference_set("browser.cache.directory", cachedir);
 		g_free(cachedir);
+#endif
 	}
 	else
 	{
