@@ -18,8 +18,7 @@
 
 #include "galeon.h"
 
-/* set this to use a file in /tmp rather than generating on the fly */
-/* this is to hack around problems in CVS mozilla embedding widget */
+/* set this to use a file in /tmp rather than generating on the fly  */
 #define USE_INTERMEDIATE_FILE 0
 
 /* output stream type */
@@ -30,10 +29,10 @@ typedef GtkMozEmbed OutStream;
 #endif
 
 /* local function prototypes */
-static void render_from_item(OutStream *stream, BookMarkItem *b);
+static void render_from_item(OutStream *stream, BookmarkItem *b);
 static void render_from_list(OutStream *stream, GList *l);
-static void render_url(OutStream *stream, BookMarkItem *b);
-static void render_search_form(OutStream *stream, BookMarkItem *b);
+static void render_url(OutStream *stream, BookmarkItem *b);
+static void render_search_form(OutStream *stream, BookmarkItem *b);
 
 /*
  * FIXME: this is eventually due to work by writing the HTML into a file,
@@ -91,6 +90,7 @@ portal_render_into_embed (GaleonEmbed *embed)
 #else
 	stream = embed->mozEmbed;
 	gtk_moz_embed_open_stream(stream, MYPORTAL_URL, "text/html");
+	mozembed_load_started_cb (NULL, embed);
 #endif
 
 	/* render the complete portal */
@@ -113,10 +113,11 @@ portal_render_into_embed (GaleonEmbed *embed)
 	/* finish the rendering */
 #if USE_INTERMEDIATE_FILE
 	fclose (stream);
-	gtk_moz_embed_load_url (embed->mozEmbed, filename);
+	embed_load_url (embed, filename);
 	g_free (filename);
 #else
 	gtk_moz_embed_close_stream(stream);
+	mozembed_load_finished_cb (NULL, embed);
 #endif
 }
 
@@ -124,8 +125,9 @@ portal_render_into_embed (GaleonEmbed *embed)
  * render_from_item: render a single bookmark item
  */
 static void
-render_from_item(OutStream *stream, BookMarkItem *b)
+render_from_item(OutStream *stream, BookmarkItem *b)
 {
+	gchar *strippedname;
 	/* could be the case if there are no bookmarks */
 	if (b == NULL)
 		return;
@@ -133,17 +135,21 @@ render_from_item(OutStream *stream, BookMarkItem *b)
 	/* otherwise do the appropriate thing */
 	switch (b->type)
 	{
-	case CATEGORY:
-	case AUTOBOOKMARKS:
+	case BM_CATEGORY:
+	case BM_AUTOBOOKMARKS:
 		RENDER("<blockquote>\n");
 		RENDER("<strong>");
-		RENDER(b->name);
+		strippedname = strip_uline_accel (b->name);
+		RENDER(strippedname);
+		g_free(strippedname);
 		RENDER("</strong>\n");
-		render_from_list(stream, b->list);
+		/* FIXME: if the bookmark is an alias, it should point to the real bookmark */
+		if (!b->alias_of)
+			render_from_list(stream, b->list);
 		RENDER("</blockquote>\n");
 		break;
 		
-	case SITE:
+	case BM_SITE:
 		if (strstr (b->url, "%s"))
 		{
 			render_search_form(stream, b);
@@ -154,7 +160,7 @@ render_from_item(OutStream *stream, BookMarkItem *b)
 		}
 		break;
 		
-	case SEPARATOR:
+	case BM_SEPARATOR:
 		RENDER("<br>\n");
 		break;
 	}
@@ -176,7 +182,7 @@ render_from_list(OutStream *stream, GList *l)
  * render_url: render a bookmark url
  */
 static void
-render_url(OutStream *stream, BookMarkItem *b)
+render_url(OutStream *stream, BookmarkItem *b)
 {
 
 	RENDER("<a href=\"");
@@ -188,9 +194,11 @@ render_url(OutStream *stream, BookMarkItem *b)
 	
 	RENDER(b->url);
 	RENDER("\">");
-	if (b->pixmap == NULL)
+	if (b->pixmap_data == NULL)
 	{
-		RENDER(b->name);
+		gchar *strippedname = strip_uline_accel (b->name);
+		RENDER(strippedname);
+		g_free(strippedname);
 	}
 	else
 	{
@@ -205,7 +213,7 @@ render_url(OutStream *stream, BookMarkItem *b)
  * render_search_form: render a search form based on a smart bookmark
  */
 static void 
-render_search_form(OutStream *stream, BookMarkItem *b)
+render_search_form(OutStream *stream, BookmarkItem *b)
 {
 	gchar *path;    /* static part of URL */
 	gchar **query;  /* query string, broken into parameters */
@@ -264,10 +272,12 @@ render_search_form(OutStream *stream, BookMarkItem *b)
 	RENDER(path);
 	RENDER("\" method=get>");
 	RENDER(inputs);
-	if (b->pixmap == NULL)
+	if (b->pixmap_data == NULL)
 	{
+		gchar *strippedname = strip_uline_accel (b->name);
 		RENDER("<input type=submit value=\"");
-		RENDER(b->name);;
+		RENDER(strippedname);
+		g_free(strippedname);
 	}
 	else
 	{
