@@ -18,6 +18,24 @@
 
 #include <galeon.h>
 
+#ifdef CVS_SOURCE
+/* from nsICache.h */
+enum {
+	STORE_ANYWHERE		= 0,
+	STORE_IN_MEMORY		= 1,
+	STORE_ON_DISK		= 2,
+	STORE_ON_DISK_AS_FILE	= 3
+};
+#else
+/* from nsINetDataCacheManager.h */
+enum {
+        STORE_ANYWHERE		= 1U,
+        STORE_IN_MEMORY		= 2U,
+        STORE_ON_DISK_AS_FILE	= 4U,
+        STORE_ON_DISK		= 8U
+};
+#endif
+
 void prefs_ctree_tree_select_row_cb (GtkWidget *ctree, GtkCTreeNode *node,
 				     GtkWidget *prefs);
 void prefs_ctree_tree_unselect_row_cb (GtkWidget *ctree, GList *node,
@@ -25,6 +43,7 @@ void prefs_ctree_tree_unselect_row_cb (GtkWidget *ctree, GList *node,
 void prefs_ok_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_apply_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_cancel_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
+gboolean prefs_close_cb (GnomeDialog *dialog);
 void prefs_help_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_mime_add_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_mime_delete_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
@@ -47,7 +66,10 @@ void prefs_lang_add_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_lang_remove_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_lang_up_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
 void prefs_lang_down_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
-void prefs_clear_cache_button_clicked_cb (GtkButton *button, GtkWidget *prefs);
+void prefs_clear_disk_cache_button_clicked_cb (GtkButton *button,
+					       GtkWidget *prefs);
+void prefs_clear_memory_cache_button_clicked_cb (GtkButton *button,
+						 GtkWidget *prefs);
 void prefs_clear_location_history_button_clicked_cb (GtkButton *button,
                                                GtkWidget *prefs);
 void prefs_clear_history_button_clicked_cb (GtkButton *button,
@@ -63,9 +85,9 @@ void prefs_spinner_select_icon_cb (GnomeIconList *gil, gint num,
 				   gpointer data);
 void prefs_smart_bm_entry_width_spin_changed_cb (GtkEditable *editable,
 						 GtkSpinButton *spin);
-
-/* local functions */
-static void prefs_dialog_save_geometry (void);
+void preferences_switch_page_cb (GtkWidget *notebook, 
+				 GtkNotebookPage *notebook_page,
+				 guint page_num);
 
 /* from prefs.c */
 void mouse_wheel_controls_set_sensitive (GtkWidget *widget, gint key,
@@ -80,47 +102,37 @@ static gboolean enable_edit_mime = FALSE;
 void
 prefs_ok_button_clicked_cb (GtkButton *button, GtkWidget *prefs_dlg)
 {
-	preferences_set_config_to_dialog (GTK_WIDGET (prefs));
 	preferences_apply ();
-	prefs_dialog_save_geometry();
-	gtk_widget_destroy (prefs);
-	prefs = NULL;
+	preferences_close ();
 }
 
-/**
- * prefs_ok_button_clicked_cb: preferences dialog ok button clicked
- */
-void
-prefs_clear_cache_button_clicked_cb (GtkButton *button, GtkWidget *prefs)
-{
-	gchar *confdir = g_strconcat(g_get_home_dir(),"/.galeon",NULL);
-	gchar *cachedir = g_strconcat(confdir,"/cache",NULL);
-
-	remove_directory (cachedir, TRUE);
-
-	g_free (confdir);
-	g_free (cachedir);
-}
-
-
 /**
  * prefs_apply_button_clicked_cb: preferences dialog apply button clicked
  */
 void
 prefs_apply_button_clicked_cb (GtkButton *button, GtkWidget *prefs)
 {
-	preferences_set_config_to_dialog (GTK_WIDGET (prefs));
 	preferences_apply ();
 }
 
-
+/**
+ * prefs_cancel_button_clicked_cb: preferences dialog cancel button clicked
+ */
 void
 prefs_cancel_button_clicked_cb (GtkButton *button, GtkWidget *prefs_dlg)
 {
 	// This is ok for now, but we also need a 'revert' feature
-	prefs_dialog_save_geometry();
-	gtk_widget_destroy(prefs);
-	prefs = NULL;
+	preferences_close ();
+}
+
+/**
+ * prefs_close_cb: dialog has been closed
+ */
+gboolean
+prefs_close_cb (GnomeDialog *dialog)
+{
+	preferences_close ();	
+	return TRUE;
 }
 
 void
@@ -129,6 +141,24 @@ prefs_help_button_clicked_cb (GtkButton *button, GtkWidget *prefs)
         preferences_help ();
 }
 
+/**
+ * prefs_clear_disk_cache_button_clicked_cb: "Clear disk cache" clicked
+ */
+void
+prefs_clear_disk_cache_button_clicked_cb (GtkButton *button, GtkWidget *prefs)
+{
+	mozilla_clear_cache (STORE_ON_DISK);
+	mozilla_clear_cache (STORE_ON_DISK_AS_FILE);
+}
+
+/**
+ * prefs_clear_memory_cache_button_clicked_cb: "Clear memory cache" clicked
+ */
+void
+prefs_clear_memory_cache_button_clicked_cb (GtkButton *button, GtkWidget *prefs)
+{
+	mozilla_clear_cache (STORE_IN_MEMORY);
+}
 
 /**
  * on_prefs_add_button_clicked: mime types add button clicked
@@ -142,7 +172,7 @@ prefs_mime_add_button_clicked_cb (GtkButton *button, GtkWidget *prefs)
 		(GTK_WIDGET (button), "preferences_mime_list");
 	gint row;
 	newitem->name = g_strdup (_("New mime item"));
-	newitem->action = UNKNOWN;
+	newitem->action = MIME_UNKNOWN;
 	text[0] = newitem->name;
 	text[1] = N_("Unknown");
 	text[2] = NULL;
@@ -190,17 +220,17 @@ void prefs_mime_list_select_row_cb (GtkCList *clist,
 	selected_mime = (MimeItem*) gtk_clist_get_row_data(clist, row);
 		
 	switch (selected_mime->action) {
-	case SAVE_TO_DISK:
+	case MIME_SAVE_TO_DISK:
 		act_num = 0; /* save */
 		break;
-	case LEAVE_TO_MOZILLA:
+	case MIME_LEAVE_TO_MOZILLA:
 		act_num = 1; /* Render in Galeon */
 		break;
-	case ASK_ACTION:
+	case MIME_ASK_ACTION:
 		act_num = 2; /* Ask */
 		break;
-	case RUN_PROGRAM: /* not yet done */
-	case UNKNOWN:
+	case MIME_RUN_PROGRAM: /* not yet done */
+	case MIME_UNKNOWN:
 		act_num = 2; /* default to ask */
 		break;
 	}
@@ -267,16 +297,16 @@ void prefs_mime_edit_cb (GtkWidget *w, GtkWidget *prefs)
 	selected_mime->name = gtk_editable_get_chars (GTK_EDITABLE (name_entry), 0, -1);
 	switch (action_index) {
 	case 0:
-		selected_mime->action = SAVE_TO_DISK;
+		selected_mime->action = MIME_SAVE_TO_DISK;
 		break;
 	case 1:
-		selected_mime->action = LEAVE_TO_MOZILLA;
+		selected_mime->action = MIME_LEAVE_TO_MOZILLA;
 		break;
 	case 2:
-		selected_mime->action = ASK_ACTION;
+		selected_mime->action = MIME_ASK_ACTION;
 		break;
 	default:
-		selected_mime->action = UNKNOWN;
+		selected_mime->action = MIME_UNKNOWN;
 		break;
 	}
 
@@ -284,19 +314,19 @@ void prefs_mime_edit_cb (GtkWidget *w, GtkWidget *prefs)
 
 	text[0] = selected_mime->name;
 	switch (selected_mime->action) {
-	case LEAVE_TO_MOZILLA:
+	case MIME_LEAVE_TO_MOZILLA:
 		text[1] = N_("Render with Galeon");
 		break;
-	case SAVE_TO_DISK:
+	case MIME_SAVE_TO_DISK:
 		text[1] = N_("Save to disk");
 		break;
-	case RUN_PROGRAM:  
+	case MIME_RUN_PROGRAM:  
 		/* Should run the program associted to the mime type by gnome */
 		/* not yet done */
-	case UNKNOWN:
+	case MIME_UNKNOWN:
 		text[1] = N_("Unknown");
 		break;
-	case ASK_ACTION:
+	case MIME_ASK_ACTION:
 		text[1] = N_("Ask the user");
 		break;
 	}
@@ -563,20 +593,11 @@ void
 prefs_clear_history_question_cb (gint reply, gpointer data)
 {
 	extern GHashTable *history;
-	extern GtkCList *clHistory;
 	extern GCompletion *URLCompletion;
 
 	if (reply)
 		return;
 
-	/* clear the history list */
-	if (clHistory)
-	{
-		gtk_clist_freeze(clHistory);
-		gtk_clist_clear(clHistory);
-		gtk_clist_thaw(clHistory);
-	}
-
 	/* clear the hash table */
 	g_hash_table_freeze(history);
 	g_hash_table_foreach_remove(history, (GHRFunc) prefs_history_item_free_cb, NULL);
@@ -585,6 +606,9 @@ prefs_clear_history_question_cb (gint reply, gpointer data)
 	/* clear the autocompletion history */
 	g_completion_clear_items(URLCompletion);
 
+	/* clear the go_menu */
+	history_clear_recent_menu();
+
 	/* clear the file */
 	history_save();
 }
@@ -639,32 +663,6 @@ prefs_spinner_select_icon_cb (GnomeIconList *gil, gint num, gpointer data)
 	gtk_entry_set_text(GTK_ENTRY(entry), path);
 }
 
-static void
-prefs_dialog_save_geometry (void)
-{
-	GtkWidget *hpane;
-	gint x, y, width, height, pane_pos;
-
-	g_return_if_fail (GTK_IS_WIDGET(prefs));
-
-	hpane = glade_lookup_widget (prefs, "hpaned2");
-	if (!hpane) return;
-
-	gdk_window_get_root_origin(prefs->window, &x, &y);
-	gdk_window_get_size(prefs->window, &width, &height);
-	pane_pos = GTK_PANED(hpane)->child1_size;
-
-	gnome_config_push_prefix("/galeon/State/");
-	gnome_config_set_int("prefs_dialog_x", x);
-	gnome_config_set_int("prefs_dialog_y", y);
-	gnome_config_set_int("prefs_dialog_width", width);
-	gnome_config_set_int("prefs_dialog_height", height);
-	gnome_config_set_int("prefs_dialog_hpane", pane_pos);
-	gnome_config_pop_prefix();
-
-	gnome_config_sync();
-}
-
 void 
 prefs_smart_bm_entry_width_spin_changed_cb (GtkEditable *editable,
 					    GtkSpinButton *spin)
@@ -680,3 +678,28 @@ prefs_smart_bm_entry_width_spin_changed_cb (GtkEditable *editable,
 	gtk_widget_set_usize(entry, width, -2);
 }
 
+/**
+ * preferences_switch_page_cb: called when the user jumps to a new page
+ */
+void
+preferences_switch_page_cb (GtkWidget *notebook, 
+			    GtkNotebookPage *notebook_page,
+			    guint page_num)
+{
+	GtkWidget *themes_page;
+	GtkWidget *iconlist;
+
+	/* find the widget representing the themes page */
+	themes_page = glade_lookup_widget (GTK_WIDGET (notebook),
+					   "themes_page");
+
+	/* if it's the themes page, load spinners into the list */
+	/* this is quite slow, so we only do it if the user selects
+	 * this specific page */
+	if (notebook_page->child == themes_page)
+	{
+		iconlist = glade_lookup_widget (GTK_WIDGET (notebook),
+						"spinner_iconlist");
+		spinner_fill_iconlist (GNOME_ICON_LIST (iconlist));
+	}
+}
