@@ -19,6 +19,8 @@
 #include "galeon.h"
 
 static void free_spinner_list (void);
+static void spinner_fill_iconlist_from_dir (GnomeIconList *icon_list,
+					    gchar *base);
 
 /**
  * spinner_create: create a spinner and add it to an hbox
@@ -44,12 +46,7 @@ spinner_create (GtkWidget *hbox)
 	}
 	
 	/* try to load the first file to find the size */
-	filename = g_strdup_printf ("%s/anim_01.png", anim_dir);
-	if (!g_file_exists (filename))
-	{
-		g_free (filename);
-		filename = g_strdup_printf ("%s/rest.png", anim_dir);
-	}
+	filename = g_strdup_printf ("%s/rest.png", anim_dir);
 	first = gdk_imlib_load_image (filename);
 	if (first == NULL)
 	{
@@ -65,14 +62,14 @@ spinner_create (GtkWidget *hbox)
 
 	/* set the size of the pixmap appropriately to whether 
 	 * text is turned on in the toolbars, so it resizes okay */
-	toolbar = gnome_config_get_int("/galeon/Appearance/toolbar_type=2");
+	toolbar = gnome_config_get_int("/galeon/Appearance/toolbar_type=0");
 
 	/* downsize the image if it would bloat the toolbar */
-	if (toolbar == 0 && height > 28)
+	if (toolbar == 0 && height > 24)
 	{
 		/* preserve aspect ratio */
-		width = (width * 28) / height;
-		height = 28;
+		width = (width * 24) / height;
+		height = 24;
 	}
 
 	if (toolbar == 1 && height > 22)
@@ -94,39 +91,36 @@ spinner_create (GtkWidget *hbox)
 	gnome_animator_set_loop_type (GNOME_ANIMATOR (spinner),
 				      GNOME_ANIMATOR_LOOP_RESTART);
 
-	/* 256 is an arbitrary upper limit, in case something
+	/* 999 is an arbitrary upper limit, in case something
 	 * goes wrong with testing the existence of the files */
-	for (i = 1; i < 256; i++)
+	for (i = 0; i < 1000; i++)
 	{
 		/* build the filename */
-		filename = g_strdup_printf("%s/anim_%02d.png", anim_dir, i);
+		if (i == 0)
+		{
+			filename = g_strdup_printf("%s/rest.png", anim_dir);
+		}
+		else
+		{
+			filename = g_strdup_printf("%s/%03d.png", anim_dir, i);
+		}
 
 		if ((g_file_exists(filename) & (G_FILE_TEST_ISFILE |
 						G_FILE_TEST_ISLINK)) == 0)
 		{
+			/* free the string */
 			g_free(filename);
 
-			/* test for nautilus spinner format */
-			filename = g_strdup_printf("%s/%03d.png", anim_dir, i);
-
-			/* test for the existence of the file */
-			if ((g_file_exists(filename) & (G_FILE_TEST_ISFILE |
-							G_FILE_TEST_ISLINK)) == 0)
+			/* check if this is the first one and warn */
+			if (i == 0)
 			{
-				/* free the string */
-				g_free(filename);
-
-				/* check if this is the first one and warn */
-				if (i == 1)
-				{
-					g_warning("no frames of animation found in "
-						  "directory `%s'", anim_dir);
-					return NULL;
-				}
-
-				/* jump out of loading */
-				break;
+				g_warning("no frames of animation found in "
+					  "directory `%s'", anim_dir);
+				return NULL;
 			}
+
+			/* jump out of loading */
+			break;
 		}
 
 		/* otherwise add it in */
@@ -210,23 +204,34 @@ free_spinner_list (void)
  */
 void
 spinner_fill_iconlist (GnomeIconList *icon_list)
+{
+	/* clear spinner list */
+	free_spinner_list ();
+	gnome_icon_list_clear (GNOME_ICON_LIST (icon_list));
+	
+	/* load all the galeon spinners */
+	spinner_fill_iconlist_from_dir (icon_list, SHARE_DIR "/spinners");
+
+	/* load all the nautilus spinners */
+	/* NB: we assume nautilus is installed with the same prefix as galeon */
+	spinner_fill_iconlist_from_dir (icon_list, 
+					SHARE_DIR "/../pixmaps/nautilus");
+}
+
+static void
+spinner_fill_iconlist_from_dir (GnomeIconList *icon_list, gchar *base)
 {
 	int index;
 	gchar *icon;
-	gchar *name;
 	gchar *pref_spinner_path; 	
 	GList *node;
 	GList *list = NULL;
-	gchar *base;
 	GnomeVFSResult rc;
 
-	free_spinner_list();
-
 	pref_spinner_path = 
 		gnome_config_get_string("/galeon/Appearance/spinner_dir="
 					SHARE_DIR "/spinners/pipeon/");	
 
-	base = SHARE_DIR "/spinners";
 	rc = gnome_vfs_directory_list_load(&list,
 			base,
 			(GNOME_VFS_FILE_INFO_GET_MIME_TYPE
@@ -234,10 +239,12 @@ spinner_fill_iconlist (GnomeIconList *icon_list)
 			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS),
 			NULL);
 
-	index = 0;
-	for (node = list; node; node = node->next) {
+	index = icon_list->icons;
+	for (node = list; node; node = node->next)
+	{
 		GnomeVFSFileInfo *info;
 		gchar *path;
+		gchar *ext_path;
 
 		info = node->data;
 		if (info->name[0] == '.')
@@ -245,44 +252,38 @@ spinner_fill_iconlist (GnomeIconList *icon_list)
 		if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
 			continue;
 
-		path = g_strconcat(base, "/", info->name, "/", NULL);
-
-		/* Check to see if there is an icon.png */
-		icon = g_strconcat (path, "icon.png", NULL);
-
-		if (!g_file_exists (icon))
+		path = g_concat_dir_and_file (base, info->name);
+		ext_path = g_concat_dir_and_file (path, "throbber");
+		if (g_file_exists (ext_path))
 		{
-			/* Otherwise use the first frame */
-			g_free(icon);
-			icon = g_strconcat (path, "anim_01.png", NULL);
-			if (!g_file_exists (icon))
-			{
-				g_free (icon);
-				icon = g_strconcat (path, "rest.png", NULL);
-			}
+			g_free (path);
+			path = ext_path;
 		}
+		else
+		{
+			g_free (ext_path);
+		}
+
+		/* Check to see if there is a rest.png */
+		icon = g_concat_dir_and_file (path, "rest.png");
 
 		if (g_file_exists (icon))
 		{
-			gchar *dir;
-
-			spinner_list = g_list_append(spinner_list, g_strdup(path));
+			spinner_list = g_list_append (spinner_list, 
+						      g_strdup (path));
 
 			/* Name defaults to the directory name */
-			dir = g_dirname(path);
-			/* basename does not strdup, free dir instead */
-			name = g_basename(dir);
-			gnome_icon_list_append (icon_list, icon, name);
+			gnome_icon_list_append (icon_list, icon, 
+						g_strdup (info->name));
 
 			/* Select the icon configured in prefs */
 			if (strcmp (pref_spinner_path, path) == 0)
 				gnome_icon_list_select_icon (icon_list, index);
 
-			g_free (dir);
 			index++;
 		}
 
-		g_free(path);
+		g_free (path);
 		g_free (icon);
 	}
 
@@ -291,7 +292,8 @@ spinner_fill_iconlist (GnomeIconList *icon_list)
 }
 
 /**
- * spinner_get_path_from_index: used in prefs_callbacks.c to get the path of selected icon
+ * spinner_get_path_from_index: used in prefs_callbacks.c to get the 
+ * path of selected icon
  */
 gchar *
 spinner_get_path_from_index (int index)
