@@ -24,10 +24,11 @@ gboolean fullscreen_active = FALSE;
 
 /* local function prototypes */
 static void window_init_data (GaleonWindow *window);
-static void window_add_accelerators (GaleonWindow *window);
 static void window_set_menu_data (GaleonWindow *window);
-static void set_initial_window_properties (GaleonWindow *window, 
-					   gint width, gint height);
+static void set_initial_window_properties (GaleonWindow *window);
+
+/* clallbacks used here */
+void file_open_ok_button_clicked_cb (const gchar *file, gpointer data);
 
 /** The global list of all GaleonWindow structures */
 GList *all_windows = NULL;
@@ -53,6 +54,22 @@ const GtkTargetEntry url_drag_types[] =
 const gint url_drag_types_num_items = (sizeof (url_drag_types) /
 				       sizeof (GtkTargetEntry));
 
+typedef enum
+{
+	STRING,
+	COMPOUND_TEXT
+} SelType;
+
+/* selection types */
+const GtkTargetEntry selection_types[] =
+{
+	{ "STRING",        0, STRING        },
+	{ "COMPOUND_TEXT", 0, COMPOUND_TEXT }
+};
+const gint selection_types_num_targets = (sizeof (selection_types) /
+					  sizeof (GtkTargetEntry));
+
+
 /**
  * window_load_url: load a URL into the active embed of a window
  */
@@ -67,21 +84,39 @@ window_load_url (GaleonWindow *window, const gchar *url)
 }
  
 static void
-set_initial_window_properties (GaleonWindow *window, gint width, gint height)
+set_initial_window_properties (GaleonWindow *window)
 {
-	guint sane_width, sane_height;
- 
-	sane_width = MIN (width, gdk_screen_width ());
-	sane_height = MIN (height, gdk_screen_height ());
+	gint width, height;
 
-	gtk_window_set_default_size (GTK_WINDOW (window->WMain), 
-				     sane_width, sane_height);
+	/* find the set width and height if any */
+	if (!window->set_size)
+	{
+		/* use system defaults */
+		width = gnome_config_get_int (CONF_APPEARANCE_WINWIDTH);
+		height = gnome_config_get_int (CONF_APPEARANCE_WINHEIGHT);
+		gtk_window_set_default_size (GTK_WINDOW (window->WMain), 
+					     width, height);
+	}
 
+	/* setup other policies */
 	gtk_window_set_policy (GTK_WINDOW (window->WMain), 
-			       TRUE, TRUE, FALSE);
+			       TRUE, TRUE, TRUE);
+}
 
-	gtk_window_set_wmclass (GTK_WINDOW (window->WMain), "",
-				"galeon_browser");
+void
+window_set_visibility (GaleonWindow *window, gboolean visible)
+{
+        /* show or hide */
+	if (visible == TRUE)
+	{
+		set_settings_menu_window (window);
+		set_initial_window_properties (window);
+		gtk_widget_show (window->WMain);
+	}
+	else
+	{
+		gtk_widget_hide (window->WMain);
+	}
 }
 
 /**
@@ -95,9 +130,11 @@ window_create (void)
         GaleonWindow *window;
 	GnomeEntry *ge;
 	GtkWidget *bookmarks_item;
-	gint width, height;
-	gint tab_pos;
- 
+	gchar *layout;
+
+	/* get the saved layout string */
+	layout = gnome_config_get_string ("/galeon/Placement/Dock");
+
 	/* allocate and initialise the GaleonWindow structure */
 	window = g_new0 (GaleonWindow, 1);
 
@@ -105,18 +142,24 @@ window_create (void)
 	window->magic = GALEON_WINDOW_MAGIC;
 
 	/* create the browser window */
-	window->WMain = gnome_app_new ("Galeon", _("Galeon"));
+	window->WMain = gnome_app_new ("galeon", _("Galeon"));
+	gtk_window_set_wmclass (GTK_WINDOW (window->WMain), "",
+				"galeon_browser");
 
 	/* need to set the user_data field in all the GnomeUIInfo structs */
 	window_set_menu_data (window);
 
+	/* init dialogs data */
+	window->print_dialog = NULL;
+
 	/* create the menus and statusbar */
 	gnome_app_create_menus (GNOME_APP(window->WMain), menubar_uiinfo);
 	gnome_app_set_statusbar (GNOME_APP(window->WMain),
 				 gnome_appbar_new (TRUE, TRUE,
 						   GNOME_PREFERENCES_NEVER));
   	window_init_data (window);
-	gnome_app_install_menu_hints (GNOME_APP(window->WMain), menubar_uiinfo);
+	gnome_app_install_menu_hints (GNOME_APP(window->WMain), 
+				      menubar_uiinfo);
 
 	/* set mini icon */
 	gnome_window_icon_set_from_file (GTK_WINDOW (window->WMain), 
@@ -135,27 +178,31 @@ window_create (void)
 	toolbar_create (window);
 
 	/* set the initial state of the View_* CheckMenuItems */
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-						window->view_menubar), TRUE);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-						window->view_toolbar), TRUE);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-						window->view_statusbar), TRUE);
-
-	/* find the default width and height */
-	width = gnome_config_get_int (CONF_APPEARANCE_WINWIDTH);
-	height = gnome_config_get_int (CONF_APPEARANCE_WINHEIGHT);
-
-	/* set initial window properties */
-	set_initial_window_properties (window, width, height);
+	gtk_check_menu_item_set_active 
+		(GTK_CHECK_MENU_ITEM (window->view_menubar), TRUE);
+	gtk_check_menu_item_set_active 
+		(GTK_CHECK_MENU_ITEM (window->view_toolbar), TRUE);
+	gtk_check_menu_item_set_active 
+		(GTK_CHECK_MENU_ITEM (window->view_statusbar), TRUE);
+//	gtk_check_menu_item_set_active 
+//		(GTK_CHECK_MENU_ITEM (window->view_history_dock),
+//		 dHistory != NULL && GNOME_IS_DOCK_ITEM (dHistory));
+	gtk_check_menu_item_set_active 
+		(GTK_CHECK_MENU_ITEM (window->view_history_dock),
+		 dHistory != NULL && GNOME_IS_DOCK_ITEM (dHistory));
 
 	/* add it to the list of windows */
-	all_windows = g_list_append(all_windows, window);
+	all_windows = g_list_prepend (all_windows, window);
 
 	/* create the bookmarks menu */
 	bookmarks_create_menu (window);
 	bookmarks_create_tb (window);
 
+	/* restore the toolbar layout */
+	gnome_dock_layout_parse_string (GNOME_APP(window->WMain)->layout,
+				        layout);
+	g_free (layout);
+
 	/* setup the history dropdown menu */
 	ge = GNOME_ENTRY(window->toolbar_gnomeentry);
 	auto_completion_add_from_entry (GNOME_ENTRY(ge));
@@ -180,12 +227,12 @@ window_create (void)
 			    window);
 
 	/* set selection signal */
-	gtk_selection_add_target (GTK_WIDGET(window->WMain),
-				  GDK_SELECTION_PRIMARY, 
-				  GDK_SELECTION_TYPE_STRING, 1);
-	gtk_selection_add_target (GTK_WIDGET(window->WMain),
-				  gdk_atom_intern("CLIPBOARD",FALSE),
-				  GDK_SELECTION_TYPE_STRING, 1);
+	gtk_selection_add_targets (GTK_WIDGET(window->WMain),
+				   GDK_SELECTION_PRIMARY, 
+				   selection_types, selection_types_num_targets);
+	gtk_selection_add_targets (GTK_WIDGET(window->WMain),
+				   gdk_atom_intern("CLIPBOARD", FALSE),
+				   selection_types, selection_types_num_targets);
 	gtk_signal_connect (GTK_OBJECT(window->WMain), "selection_received",
 			    GTK_SIGNAL_FUNC (window_selection_received_cb),
 			    window);
@@ -193,11 +240,6 @@ window_create (void)
 			    GTK_SIGNAL_FUNC (window_selection_get_cb),
 			    NULL);
 
-	/* add additional accelerators */
-	/* Note: Menu accelerators are now saved/restored, so this
-	   should no longer be necessary.  --Josh  */
-//	window_add_accelerators (window);
-
 	/* make the toplevel notebobok */
 	window->notebook = gtk_notebook_new ();
 
@@ -206,9 +248,10 @@ window_create (void)
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (window->notebook), TRUE);
 	gtk_signal_connect (GTK_OBJECT (window->notebook), "switch_page",
 			    embed_notebook_switch_page_cb, window);
-	tab_pos = gnome_config_get_int (CONF_APPEARANCE_TABBED_POSITION);
-	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (window->notebook), tab_pos);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->notebook), FALSE);
+	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (window->notebook),
+		gnome_config_get_int (CONF_APPEARANCE_TABBED_POSITION));
+	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->notebook),
+		gnome_config_get_int (CONF_APPEARANCE_TABBED_ALWAYS_SHOW));
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (window->notebook), FALSE);
 
 	/* insert notebook into toplevel window */
@@ -216,22 +259,15 @@ window_create (void)
 				GTK_WIDGET (window->notebook));
 
 	/* NB: window isn't shown until we add an embed */
+	/* don't know the size at which to show it yet -- if we discover
+	 * a size (e.g. for a popup) then we'll use that, otherwise we'll
+	 * find a default, but not until the window is made visible */
+	window->set_size = FALSE;
 
 	/* return the completed GaleonWindow */
 	return window;
 }
 
-/**
- * window_show_error: show a browser error message
- */
-void 
-window_show_error (GaleonWindow *window, char *errormsg)
-{
-	/* show the error message on status bar */
-	window->tempMessage = errormsg;
-	window_update_status_bar (window);
-}
-
 /**
  * window_update_status_bar: update the status bar of the toplevel window
  */
@@ -247,6 +283,46 @@ window_update_status_bar (GaleonWindow *window)
 	embed = window->active_embed;
 	return_if_not_embed (embed);
 
+	/* get progress bar widget */
+	progress = GTK_WIDGET (gnome_appbar_get_progress 
+			       (GNOME_APPBAR(window->appbar)));
+
+	/* have we started loading? */
+	if (embed->load_started > 0 && embed->bytesLoaded > 0)
+	{
+		if (embed->loadPercent == 0) 
+		{
+			/* not really, go into activity mode */
+			gtk_progress_set_activity_mode 
+				(GTK_PROGRESS (progress), TRUE);
+			if (!(window->progress_timeout))
+			{
+				window->progress_timeout = TRUE;
+				g_timeout_add (100, (GSourceFunc)
+					       window_progress_action, 
+					       window);
+			}
+		} 
+		else
+		{
+			/* yes, show the progress in progress bar */
+			window->progress_timeout = FALSE;
+			gtk_progress_set_activity_mode 
+				(GTK_PROGRESS (progress), FALSE);
+			gnome_appbar_set_progress 
+				(GNOME_APPBAR (window->appbar), 
+				 embed->loadPercent / 100.0);
+		}
+	}
+	else
+	{
+		/* go to sleep */
+		window->progress_timeout = FALSE;
+		gtk_progress_set_activity_mode (GTK_PROGRESS (progress),
+						FALSE);
+		gnome_appbar_set_progress (GNOME_APPBAR (window->appbar), 0);
+	}
+
 	/* get rid of last message */
 	gnome_appbar_pop (GNOME_APPBAR (window->appbar));
 
@@ -286,46 +362,6 @@ window_update_status_bar (GaleonWindow *window)
 
 	/* show the new message */
 	gnome_appbar_push (GNOME_APPBAR (window->appbar), message);
-
-	/* get progress bar widget */
-	progress = GTK_WIDGET (gnome_appbar_get_progress 
-			       (GNOME_APPBAR(window->appbar)));
-
-	/* have we started loading? */
-	if (embed->load_started)
-	{
-		if (embed->loadPercent == 0) 
-		{
-			/* not really, go into activity mode */
-			gtk_progress_set_activity_mode 
-				(GTK_PROGRESS (progress), TRUE);
-			if (!(window->progress_timeout))
-			{
-				window->progress_timeout = TRUE;
-				g_timeout_add (100, (GSourceFunc)
-					       window_progress_action, 
-					       window);
-			}
-		} 
-		else
-		{
-			/* yes, show the progress in progress bar */
-			window->progress_timeout = FALSE;
-			gtk_progress_set_activity_mode 
-				(GTK_PROGRESS (progress), FALSE);
-			gnome_appbar_set_progress 
-				(GNOME_APPBAR (window->appbar), 
-				 embed->loadPercent / 100.0);
-		}
-	}
-	else
-	{
-		/* go to sleep */
-		window->progress_timeout = FALSE;
-		gtk_progress_set_activity_mode (GTK_PROGRESS (progress),
-						FALSE);
-		gnome_appbar_set_progress (GNOME_APPBAR (window->appbar), 0);
-	}
 }
 
 /**
@@ -358,27 +394,30 @@ window_update_temp_message(GaleonWindow *window, const char *message)
 void
 window_update_nav_buttons (GaleonWindow *window)
 {
-	gboolean can_go_back, can_go_forward;
+	gboolean can_go_back, can_go_up, can_go_forward;
 	GaleonEmbed *embed;
 
 	/* get active embed */
 	embed = window->active_embed;
 
-	/* query mozilla */
+	/* query mozilla / embed */
 	can_go_back    = gtk_moz_embed_can_go_back (embed->mozEmbed);
+	can_go_up      = embed_can_go_up (embed);
 	can_go_forward = gtk_moz_embed_can_go_forward (embed->mozEmbed);
 
 	/* update back and forward buttons */
 	gtk_widget_set_sensitive (GTK_WIDGET (window->BBack), 
 				  can_go_back);
+	gtk_widget_set_sensitive (GTK_WIDGET (window->BUp), 
+				  can_go_up);
 	gtk_widget_set_sensitive (GTK_WIDGET (window->BForward), 
 				  can_go_forward);
 
 	/* update stop and refresh toolbar buttons */
 	gtk_widget_set_sensitive (GTK_WIDGET (window->BStop),
-				  embed->load_started);
+				  embed->load_started > 0);
 	gtk_widget_set_sensitive (GTK_WIDGET (window->BRefresh),
-				  !(embed->load_started));
+				  embed->load_started == 0);
 }
 
 /**
@@ -387,6 +426,12 @@ window_update_nav_buttons (GaleonWindow *window)
 static void
 window_init_data (GaleonWindow *window)
 {
+	int pos; //A temp hack for backward compatibility
+#ifdef CVS_SOURCE
+	pos = 5;
+#else
+	pos = 4;
+#endif
 	/* these are all nullified */
 	window->spinner = NULL;
 	window->tempMessage = NULL;
@@ -397,11 +442,11 @@ window_init_data (GaleonWindow *window)
 	window->appbar = GNOME_APP(window->WMain)->statusbar;
 	window->menubar = GNOME_APP(window->WMain)->menubar;
 
-	window->view_source_mode = view_menu_uiinfo[1].widget;
-	window->view_menubar = view_menu_uiinfo[3].widget;
-	window->view_toolbar = view_menu_uiinfo[4].widget;
-	window->view_statusbar = view_menu_uiinfo[5].widget;
-
+	window->view_menubar = view_menu_uiinfo[2].widget;
+	window->view_toolbar = view_menu_uiinfo[3].widget;
+	window->view_statusbar = view_menu_uiinfo[4].widget;
+//	window->view_bookmarks_dock = view_menu_uiinfo[5].widget;
+	window->view_history_dock = view_menu_uiinfo[5].widget;
 	window->view_fullscreen = view_menu_uiinfo[6].widget;
 	window->load_images_always = load_images_always_uiinfo[0].widget;
 	window->load_images_from_current_server_only = 
@@ -410,12 +455,12 @@ window_init_data (GaleonWindow *window)
 	window->animate_always = animate_always_uiinfo[0].widget;
 	window->animate_once_through = animate_always_uiinfo[1].widget;
 	window->animate_never = animate_always_uiinfo[2].widget;
-	window->use_own_fonts =  settings_menu_uiinfo[4].widget;
-	window->use_own_colors = settings_menu_uiinfo[5].widget;
-	window->enable_java = settings_menu_uiinfo[6].widget;
-	window->enable_javascript = settings_menu_uiinfo[7].widget;
-	window->enable_proxy = settings_menu_uiinfo[8].widget;
-	window->use_galeon_mime_handling = settings_menu_uiinfo[9].widget;
+	window->use_own_fonts =  settings_menu_uiinfo[pos++].widget;
+	window->use_own_colors = settings_menu_uiinfo[pos++].widget;
+	window->enable_java = settings_menu_uiinfo[pos++].widget;
+	window->enable_javascript = settings_menu_uiinfo[pos++].widget;
+	window->enable_proxy = settings_menu_uiinfo[pos++].widget;
+	window->use_galeon_mime_handling = settings_menu_uiinfo[pos++].widget;
 
 	gtk_object_set_data(GTK_OBJECT(window->WMain), "WMain",
 			    window->WMain);
@@ -456,22 +501,11 @@ window_init_data (GaleonWindow *window)
 /**
  * window_show_open_dialog: show the open dialog
  */
-void window_show_open_dialog(GaleonWindow *window)
+void window_show_open_dialog (GaleonWindow *window)
 {
-	GtkWidget *fs = NULL;
-	GladeXML *gxml;
-
-	gxml = glade_xml_new(glade_file(), "fsopen");
-	glade_xml_signal_autoconnect_full (gxml, glade_signal_connect_func,
-					   window);
-	fs = glade_xml_get_widget(gxml, "fsopen");
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),
-					gnome_config_get_string(CONF_DIR_OPEN));
-
-	gtk_window_set_modal(GTK_WINDOW(fs), TRUE);
-	gtk_widget_show(fs);
-	window_set_layer(fs);
+	misc_ask_file_name (_("Open a file"), _("Select the file to open:"), 
+			    gnome_config_get_string (CONF_DIR_OPEN),
+			    file_open_ok_button_clicked_cb, NULL, TRUE, FALSE, window);
 }
 
 /**
@@ -510,11 +544,10 @@ void window_show_openurl_dialog (GaleonWindow *window)
 						   window_location_entry_key_press_cb),
 						 window);
 			if (open_url_str)
+			{
 				gtk_entry_set_text(GTK_ENTRY(child->widget),
 						   open_url_str);
-
-				gtk_entry_select_region(GTK_ENTRY(child->widget),
-							0, -1);
+			}
 			break;
 		}
 		children = children->next;
@@ -523,53 +556,6 @@ void window_show_openurl_dialog (GaleonWindow *window)
 	gnome_dialog_run(GNOME_DIALOG (dialog));
 }
 
-
-/**
- * window_show_temporary_bookmarks: show the temporary bookmarks dialog
- */
-void window_show_temporary_bookmarks (GaleonWindow *window)
-{
-	if (!temp_bookmarks_window)
-	{
-		gint x, y, width, height;
-
-		temp_bookmarks_init ();
-		gnome_config_push_prefix("/galeon/State/");
-		x = gnome_config_get_int("temp_bookmarks_x=-1");
-		y = gnome_config_get_int("temp_bookmarks_y=-1");
-		if (!(x == -1 && y == -1))
-			gtk_widget_set_uposition(temp_bookmarks_window->dialog,
-						 x, y);
-
-		width = gnome_config_get_int("temp_bookmarks_width=-1");
-		height = gnome_config_get_int("temp_bookmarks_height=-1");
-		if (!(width == -1 && height == -1))
-			gtk_window_set_default_size(
-				     GTK_WINDOW(temp_bookmarks_window->dialog),
-				     width,height);
-		gnome_config_pop_prefix ();
-	}
-	gtk_widget_show (temp_bookmarks_window->dialog);
-	window_set_layer(temp_bookmarks_window->dialog);
-
-	if (window)
-		temp_bookmarks_window->embed = window->active_embed;
-}
-
-/**
- * window_show_history: show the history dialog
- */
-void window_show_history (GaleonWindow *window)
-{
-	history_show_dialog ();
-	
-	/* this is to know wich window last invoked the history dialog */
-	gtk_object_set_data(GTK_OBJECT(dHistory), "GaleonWindow", 
-			    (gpointer)window);
-	gtk_widget_show(dHistory);
-	window_set_layer(dHistory);
-}
-
 /**
  * window_toolbar_show: show the toolbar
  */
@@ -641,7 +627,7 @@ window_close (GaleonWindow *window)
 	GList *e, *next;
 
 	return_if_not_window (window);
-	
+
 	/* hide the window, this make the app seem more responsive */
 	gtk_widget_hide (window->WMain);
 
@@ -670,14 +656,21 @@ window_close (GaleonWindow *window)
 
 		/* destroy the window, and all it contains */
 		gtk_widget_destroy (window->WMain);
-		
-		/* quit if this was the last window to go */
-		if (g_list_length (all_windows) == 0)
+
+		/* save the history out to disk -- although this entails a bit
+		 * of overhead hopefully it will mean we don't end up losing so
+		 * much of the history on a crash... MattA 10/12/2000 */
+		history_save ();
+
+		/* quit if this was the last window to go and
+		 * we're not in server mode */
+		if (g_list_length (all_windows) == 0 && !galeon_server_mode)
 		{
 			galeon_exit ();
 		}
 	}
 }
+
 /**
  * window_go_home:
  */
@@ -710,46 +703,6 @@ window_go_home(GaleonWindow *window, gboolean new_embed, gboolean new_window)
 	g_free(startpage);
 }
 
-/**
- * Add additional accelerators.
- *
- */
-static void
-window_add_accelerators (GaleonWindow *window)
-{
-#define NEW_WIN  file_menu_uiinfo[0].widget
-#define RELOAD   go_menu_uiinfo[2].widget
-#define ZOOM_IN  view_menu_uiinfo[8].widget
-#define ZOOM_OUT view_menu_uiinfo[9].widget
-	GtkAccelGroup *ag = GNOME_APP (window->WMain)->accel_group;
-	int i;
-	struct {
-		GtkWidget *widget;
-		gchar *signal_name;
-		guint key;
-		guint mods;
-		GtkAccelFlags flags;
-	} accelerators[] = {
-		{ NEW_WIN, "activate",
-		  GDK_n, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE },
-		{ RELOAD, "activate",
-		  GDK_r, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE },
-		{ ZOOM_IN, "activate",
-		  GDK_plus, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE },
-		{ ZOOM_OUT, "activate",
-		  GDK_minus, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE },
-		{ NULL, NULL, 0, 0, 0 }
-	};
-	for (i = 0; accelerators[i].widget != NULL; i++) {
-		GtkWidget *w = (accelerators[i].widget);
-
-		gtk_widget_add_accelerator(w, accelerators[i].signal_name, ag,
-					   accelerators[i].key,
-					   accelerators[i].mods,
-					   accelerators[i].flags);
-	}
-}
-
 /**
  *  Request that the widget be placed above or below the Gnome panels
  */
@@ -782,8 +735,11 @@ window_set_encoding_menu (GaleonWindow *window, GList *charset_titles)
 	encoding_menu_item = gtk_object_get_data (GTK_OBJECT(window->WMain),
 						  "encoding_menu_item");
 
-	create_charset_submenus (GTK_MENU_ITEM(encoding_menu_item),
-				 charset_titles, window);
+	if (GTK_MENU_ITEM(encoding_menu_item)->submenu == NULL)
+	{
+		create_charset_submenus (GTK_MENU_ITEM(encoding_menu_item),
+					 charset_titles, window);
+	}
 }
 
 void
@@ -845,9 +801,7 @@ window_set_fullscreen_mode (GaleonWindow *window, gboolean active)
 static void 
 window_set_menu_data (GaleonWindow *window)
 {
-	gint i, j, num_menus;
-
-	num_menus = sizeof(all_menus) / sizeof(GnomeUIInfo *);
+	gint i, j;
 
 	for (i = 0; i < num_menus; i++)
 	{
@@ -858,20 +812,20 @@ window_set_menu_data (GaleonWindow *window)
 	}
 }
 
-/* Use this function instead of glade_lookup_widget() to get the WMain
-   widget from one of its child widgets.  This is necessary because
-   libglade is no longer being used to create the main window */
+/* Use this function instead of glade_lookup_widget() to find a widget
+   you have attached to an object via gtk_object_set_data() from one
+   of its child widgets. This is necessary because libglade is no longer
+   being used to create the main window */
 GtkWidget *
-window_lookup_wmain (GtkWidget *widget)
+window_lookup_widget (GtkWidget *widget, const char *name)
 {
-	GtkWidget *WMain = NULL;
+	GtkWidget *found_widget = NULL;
 	GtkWidget *parent = NULL;
 
 	g_return_val_if_fail(widget != NULL, NULL);
-
-	while (!WMain)
+	while (!found_widget)
 	{
-		WMain = gtk_object_get_data (GTK_OBJECT(widget), "WMain");
+		found_widget = gtk_object_get_data (GTK_OBJECT(widget), name);
 
 		if (GTK_IS_MENU(widget))
 			parent = gtk_menu_get_attach_widget(GTK_MENU(widget));
@@ -884,8 +838,27 @@ window_lookup_wmain (GtkWidget *widget)
 		widget = parent;
 	}
 
-	if (!WMain)
-		g_warning("WMain not found.");
+	if (!found_widget)
+		g_warning("%s not found.", name);
 
-	return WMain;
+	return found_widget;
 }
+
+/**
+ * window_reload_all: reload all tabs in a window
+ */
+void
+window_reload_all (GaleonWindow *window)
+{
+	GList *l;
+
+	/* check arguments */
+	return_if_not_window (window);
+
+	/* reload all tabs in the window */
+	for (l = window->embed_list; l != NULL; l = g_list_next (l))
+	{
+		embed_reload ((GaleonEmbed *)(l->data));
+	}
+}
+
