@@ -55,10 +55,23 @@ window_selection_received_cb (GtkWidget *widget,
 			      GtkSelectionData *selection_data,
 			      guint time, GaleonWindow *window)
 {
+	gboolean tabbed_mode;
+	GdkModifierType modifier;
+	
 	return_if_not_window (window);
+
 	if (selection_data->data)
 	{
-		window_load_url (window, selection_data->data);
+		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+		
+		gdk_window_get_pointer (window->WMain->window,
+			NULL, NULL, &modifier);
+			
+		if (modifier & GDK_SHIFT_MASK)
+		  	embed_create_from_url (window->active_embed, selection_data->data,
+				!tabbed_mode);
+		else
+			window_load_url (window, selection_data->data);
 	}
 }
 
@@ -68,9 +81,6 @@ window_selection_received_cb (GtkWidget *widget,
 gboolean
 window_delete_cb (GtkWidget *widget, GdkEventAny *event, GaleonWindow *window)
 {
-#ifdef DEBUG_SIGNALS
-	g_print("WMain_delete\n");
-#endif
 	/* location bar history is saved automatically when the widget is
 	   destroyed. we don't want to save the text currently in the entry */
 	gtk_editable_delete_text (GTK_EDITABLE (window->toolbar_entry),
@@ -96,7 +106,7 @@ window_delete_cb (GtkWidget *widget, GdkEventAny *event, GaleonWindow *window)
  * window_back_forward_button_press_cb:
  */
 gboolean
-window_back_forward_button_press_cb (GtkWidget *widget,
+window_back_forward_button_press_cb (GtkButton *button,
 				     GdkEventButton *event,
 				     GaleonWindow *window)
 {
@@ -109,11 +119,11 @@ window_back_forward_button_press_cb (GtkWidget *widget,
 
 	if (event->button == 3)
 	{
-		if (widget == window->BForward)
+		if (button == GTK_BUTTON (window->BForward))
 		{
 			menu = create_forward_menu (embed);
 		}
-		else if (widget == window->BBack)
+		else if (button == GTK_BUTTON (window->BBack))
 		{
 			menu = create_back_menu (embed);
 		}
@@ -123,7 +133,7 @@ window_back_forward_button_press_cb (GtkWidget *widget,
 		}
 		gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu),
 						 menu_position_under_widget, 
-						 widget, NULL, NULL);
+						 button, NULL, NULL);
 		gtk_widget_destroy (GTK_WIDGET (menu));
 		return FALSE;
 	}	
@@ -132,7 +142,7 @@ window_back_forward_button_press_cb (GtkWidget *widget,
 
 /** 
  * window_start_button_clicked_cb: start button clicked
-*/
+ */
 void
 window_start_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 {
@@ -141,28 +151,145 @@ window_start_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 }
 
 /** 
- * window_start_button_press_cb:  Open home page in new window on middle click
+ * window_start_button_press_event_cb:  Open home page in new window on
+ * middle click, or display a context menu on right-click.
  */
 gboolean
-window_start_button_press_cb (GtkWidget *widget, GdkEventButton *event,
-			      GaleonWindow *window)
+window_start_button_press_event_cb (GtkButton *button, GdkEventButton *event,
+				    GaleonWindow *window)
 {
 	gboolean tabbed_mode;
-	
-	return_val_if_not_window (window, FALSE);
+	gboolean shift_modifier;
+	GaleonEmbed *embed;
+	GtkWidget *menu;
+	gint action;
 
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-	if (event->button == 2)
+	/* homepage context menu */
+	/* although there's only one entry I think it's still necessary
+	 * to have a context menu for UI consistency -- MattA */
+	static GnomeUIInfo menu_uiinfo[] =
 	{
-		window_go_home (window, TRUE, !tabbed_mode);
-	}
+		GNOMEUIINFO_ITEM_STOCK (N_("Set current page as home page"),
+					NULL, NULL, GNOME_STOCK_MENU_HOME),
+		GNOMEUIINFO_END
+	};
 	
+	/* check args */
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+
+	/* handle button accordingly */	
+	switch (event->button)
+	{
+	case 1:
+		/* nothing -- handled by click callback */
+		break;
+
+	case 2:
+		/* load in new tab or window */
+		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+		shift_modifier = (event->state & GDK_SHIFT_MASK) ? 0 : 1;
+		window_go_home (window, TRUE, tabbed_mode ^ shift_modifier);
+		break;
+
+	case 3:
+		/* show context menu */
+		menu = gnome_popup_menu_new (menu_uiinfo);
+		action = gnome_popup_menu_do_popup_modal 
+			(GTK_WIDGET (menu),
+			 menu_position_under_widget, 
+			 button, NULL, NULL);
+
+		/* perform set homepage action if chosen */
+		if (action == 0 && embed->site_location != NULL &&
+		    strlen (embed->site_location) != 0)
+		{
+			gnome_config_set_string (CONF_GENERAL_HOMEPAGE,
+						 embed->site_location);
+			gnome_config_sync ();
+		}
+
+		/* destroy the popup menu */
+		gtk_widget_destroy (GTK_WIDGET (menu));
+		break;
+	}
+
 	return TRUE;
 }
 
+/** 
+ * window_new_button_clicked_cb: new browser button pressed
+ */
+gboolean
+window_new_button_press_cb (GtkButton *button, GdkEventButton *event,
+			    GaleonWindow *window)
+{
+	gint tabbed_mode;
+	gint shift_modifier;
+	GdkModifierType modifier;
+	GaleonEmbed *embed;
+
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+
+	if (event->button != 1 && event->button != 2)
+	{
+		return FALSE;
+	}
+
+	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED) ? 1 : 0;
+	gdk_window_get_pointer (window->WMain->window, NULL, NULL, &modifier);
+	shift_modifier = (modifier & GDK_SHIFT_MASK) ? 0 : 1;
+	embed_create_default (embed, (tabbed_mode ^ shift_modifier));
+
+	return TRUE;
+}
+
+/**
+ * window_new_button_drag_data_received_cb: called when data is dropped
+ * on the "New" button
+ */
+void
+window_new_button_drag_data_received_cb (GtkWidget *widget, 
+					 GdkDragContext *drag_context,
+					 gint x, gint y,
+					 GtkSelectionData *selection_data,
+					 guint info, guint time,
+					 GaleonWindow *window)
+{
+	gboolean tabbed_mode;
+
+	/* check args */
+	g_return_if_fail (selection_data != NULL);
+	g_return_if_fail (selection_data->data != NULL);
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+
+	/* get state */
+	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+
+	/* check drop data */
+	switch (info)
+	{
+	case DND_TARGET_NETSCAPE_URL:
+	case DND_TARGET_GALEON_URL:
+	case DND_TARGET_STRING:
+		embed_create_from_url (window->active_embed,
+				       selection_data->data,
+				       !tabbed_mode);
+		break;
+
+	default:
+		g_warning ("Unknown DND type");
+		break;
+	}
+}
+
 /** 
  * window_back_button_clicked_cb: back button clicked
-*/
+ */
 void
 window_back_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 {
@@ -171,6 +298,62 @@ window_back_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 	gtk_moz_embed_go_back (window->active_embed->mozEmbed);
 }
 
+/** 
+ * window_new_button_clicked_cb: new browser button pressed
+ */
+gboolean
+window_up_button_press_event_cb (GtkButton *button, GdkEventButton *event,
+				 GaleonWindow *window)
+{
+	GtkMenu *menu;
+	gint levels;
+
+	/* check arguments */
+	return_val_if_not_window (window, FALSE);
+	return_val_if_not_embed (window->active_embed, FALSE);
+
+	/* check button */
+	if (event->button != 3)
+	{
+		return TRUE;
+	}
+
+	/* make menu */
+	menu = create_up_menu (window->active_embed);
+	if (menu == NULL)
+	{
+		return TRUE;
+	}
+
+	/* show popup menu */
+	levels = gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu),
+						  menu_position_under_widget, 
+						  button, NULL, NULL);
+
+	/* go up n levels if requested */
+	if (levels >= 0)
+	{
+		embed_go_up (window->active_embed, levels);
+	}
+
+	/* destroy the popup menu */
+	gtk_widget_destroy (GTK_WIDGET (menu));
+
+	/* success */
+	return FALSE;
+}
+
+/**
+ * window_up_button_clicked_cb: Up button clicked
+ */
+void
+window_up_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+	embed_go_up (window->active_embed, 0);
+}
+
 /** 
  * window_forward_button_clicked_cb: forward button clicked
 */
@@ -192,6 +375,72 @@ window_refresh_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 	embed_reload (window->active_embed);
 }
 
+/** 
+ * window_refresh_button_press_event_cb: refresh button pressed
+ */
+gboolean
+window_refresh_button_press_event_cb (GtkButton *button, GdkEventButton *event,
+				      GaleonWindow *window)
+{
+	GtkWidget *menu;
+	gint action;
+	GList *l;
+
+	/* reload context menu */
+	static GnomeUIInfo menu_uiinfo[] =
+	{
+		GNOMEUIINFO_ITEM_STOCK (N_("Reload all tabs"),
+					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
+		GNOMEUIINFO_ITEM_STOCK (N_("Reload all windows"),
+					NULL, NULL, GNOME_STOCK_MENU_REFRESH),
+		GNOMEUIINFO_END
+	};
+	
+	/* check args */
+	return_val_if_not_window (window, FALSE);
+
+	/* handle button accordingly */	
+	switch (event->button)
+	{
+	case 1:
+		/* nothing -- handled by click callback */
+		break;
+
+	case 2:
+		/* nothing on middle click */
+		break;
+
+	case 3:
+		/* show context menu */
+		menu = gnome_popup_menu_new (menu_uiinfo);
+		action = gnome_popup_menu_do_popup_modal 
+			(GTK_WIDGET (menu),
+			 menu_position_under_widget, 
+			 button, NULL, NULL);
+
+		/* perform set homepage action if chosen */
+		switch (action)
+		{
+		case 0: /* reload all tabs */
+			window_reload_all (window);
+			break;
+
+		case 1: /* reload all windows */
+			for (l = all_windows; l != NULL; l = g_list_next (l))
+			{
+				window_reload_all ((GaleonWindow *)(l->data));
+			}
+			break;
+		}
+
+		/* destroy the popup menu */
+		gtk_widget_destroy (GTK_WIDGET (menu));
+		break;
+	}
+
+	return TRUE;
+}
+
 /** 
  * window_stop_button_clicked_cb: stop button clicked
  */
@@ -214,8 +463,12 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	GtkEntry *entry = GTK_ENTRY(widget);
 	GtkEditable *editable = GTK_EDITABLE(widget);
 	gboolean url_dialog = FALSE;
+	GaleonEmbed *embed;
 
-	g_return_val_if_fail(GTK_IS_ENTRY(widget), TRUE);
+	g_return_val_if_fail (GTK_IS_ENTRY (widget), TRUE);
+	return_val_if_not_window (window, TRUE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, TRUE);
 
 	/* check to see if key press is in the window's location bar or
 	   the open url dialog */
@@ -272,19 +525,20 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 			before_completion = NULL;
 		} else
 		{
-			gchar *location;
-			location = gtk_moz_embed_get_location
-				(window->active_embed->mozEmbed);
-			gtk_entry_set_text(entry, location);
+			gtk_entry_set_text (entry, embed->site_location);
 		}
 		return TRUE;
 	}
 
-	/* do not toggle statusbar on Ctrl+U, since this is an editing
+	/* do not toggle statusbar on Ctrl+U/A/E, since this is an editing
 	   shortcut in GtkEntry */
 	if ((event->state & GDK_Control_L || event->state & GDK_Control_R) 
-	    && (event->keyval == GDK_U || event->keyval == GDK_u))
+	    && (event->keyval == GDK_U || event->keyval == GDK_u ||
+		event->keyval == GDK_A || event->keyval == GDK_a ||
+		event->keyval == GDK_E || event->keyval == GDK_e))
+	{
 		return TRUE;
+	}
 
 	if (event->string[0] > 32 && event->string[0] < 126) 
 	{
@@ -319,6 +573,13 @@ window_go_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 	text = gtk_editable_get_chars (GTK_EDITABLE (window->toolbar_entry),
 				       0, -1);
 
+	/* update the content of the location field of the GaleonEmbed */
+	if (window->active_embed->site_location) 
+	{
+		g_free (window->active_embed->site_location);
+	}
+	window->active_embed->site_location = g_strdup (text);
+
 	if (strcmp(text,"")!=0)
 	{
 		/* don't save the text in the entry - avoids duplicate entries */
@@ -410,25 +671,19 @@ window_drag_pixmap_drag_data_get_cb (GtkWidget *widget,
 				     GaleonWindow *window)
 {
 	GaleonEmbed *embed;
-	gchar *url, *name;
-	BookMarkItem *b;
+	BookmarkItem *b;
 	gchar *mem;
 
 	return_if_not_window (window);
 	embed = window->active_embed;
 	return_if_not_embed (embed);
 
-	url = g_strdup (gtk_moz_embed_get_location (embed->mozEmbed));
-	name = g_strdup (gtk_moz_embed_get_title (embed->mozEmbed));
-	if (name == NULL || strlen (name) == 0)
-	{
-		name = g_strdup (_("Untitled"));
-	}
-
 	switch (info)
 	{
 	case DND_TARGET_GALEON_BOOKMARK:
-		b = bookmarks_new_bookmark (SITE, name, url, NULL, NULL, NULL);
+		b = bookmarks_new_bookmark (BM_SITE, TRUE, embed->site_title,
+					    embed->site_location,
+					    NULL, NULL, NULL);
 		mem = bookmarks_item_to_string (b);
 		gtk_selection_data_set 
 			(selection_data, selection_data->target,
@@ -440,14 +695,13 @@ window_drag_pixmap_drag_data_get_cb (GtkWidget *widget,
 	case DND_TARGET_GALEON_URL:
 		gtk_selection_data_set 
 			(selection_data, selection_data->target,
-			 8, url, strlen (url));
-				break;
+			 8, embed->site_location, 
+			 strlen (embed->site_location));
+		break;
 	default:
 		g_warning ("Unknown DND type");
 		break;
 	}
-	g_free (url);
-	g_free (name);
 }
 
 /** 
@@ -542,7 +796,7 @@ window_location_gnomeentry_popwin_cb (GtkWidget *widget, GaleonWindow *window)
 		gboolean go_toolbar_hidden = FALSE;
 
 		/* check if the go toolbar is hidden */
-		if (!window->go_toolbar)
+		if (!GTK_WIDGET_VISIBLE (window->go_toolbar))
 			go_toolbar_hidden = TRUE;
 
 		/* connect a handler to the entry's "changed" signal */
